#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: etc/application_last.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20050824   batelaan    Original
#   2 20050906   batelaan    Put $(OBJECTS) before $(LDFLAGS) to ensure symbol resolution.
#   3 20051013   neill       Allow for additional install targets
#   4 20051104   batelaan    Ensure build output goes below $BUILDROOT/apps/$(PROGRAM)
#   5 20070320   batelaan    Add qmore target.
#   6 20070824   batelaan    Doc update.
#   7 20071008   batelaan    Optimise install target. Target all now invokes install
#   8 2008-708   batelaan    Add header file dependencies. clean installed program.
#   9 20080924   batelaan    Add input lib dependencies.
#  10 20081028   batelaan    Correct name of qac_forceinclude file.
#  11 20081028   batelaan    Use value of $CROSS_COMPILE_BASE
#  12 20090204   batelaan    Fix c++ support

# File: application_last.mk
# Include makefile for main applications, used in conjunction with <application_first.mk>.
# Must be included at the end of an application's makefile.
#
# See: <LINUX_EXAMPLE_APP_makefile_TEMPLATE.mk> <application_first.mk>

# Section: Application Makefile

ifeq "" "$(strip $(C_SOURCES) $(CXX_SOURCES) $(S_SOURCES))"
  $(error makefile should have defined C_SOURCES and/or CXX_SOURCES and/or S_SOURCES)
endif

all: install

clean:
	rm -rf $(BUILDDIR)

# Macro: LIB_FILES
# Full paths of all libraries, searched for by checking $(<LDFLAGS>) for -l options containing library names,
# and -L options containing library search paths.
# Then returning the paths of all libraries that can be found using those combination.
# File extensions looked for are ".a" and ".so".
#
# Thus return a list of all files that exist for all possible combinations of "{-L path}/lib{-l name}.{ext}".
LIB_FILES = $(shell echo "" $(LDFLAGS) | perl -n -e '\
  @libs = /(?:^|\s)-l\s*(\S+)/g; \
  @searchpaths = /(?:^|\s)-L\s*(\S+)/g; \
  foreach $$searchpath (@searchpaths) { \
    foreach $$lib (@libs) { \
      foreach $$ext ("a", "so") { \
        $$p = "$$searchpath/lib$$lib.$$ext"; \
        print "$$p\n" if -f $$p; }}} \
')
#$(warning LIB_FILES = $(LIB_FILES))

$(BUILDDIR)/$(PROGRAM): $(OBJECTS) $(LIB_FILES)
	$(Q)mkdir -p $(dir $@)
	@echo LD $@
	$(Q)$(CC) -o $@ $(OBJECTS) $(LDFLAGS)

install: $(INSTALL_DIR)/$(PROGRAM) $(INSTALL_EXTRA)
clean: clean_install
clean_install:
	rm -f $(INSTALL_DIR)/$(PROGRAM)

$(INSTALL_DIR)/$(PROGRAM): $(BUILDDIR)/$(PROGRAM)
	@echo 'INSTALL $(BUILDDIR)/$(PROGRAM) -> $(INSTALL_DIR)'
	$(Q)mkdir -p $(INSTALL_DIR)
	$(Q)install --preserve-timestamps -m 0755 $(BUILDDIR)/$(PROGRAM) $(INSTALL_DIR)

FORCE:;
.PHONY: FORCE all install clean clean_install qmore

#-------------------------------------------------------------------------------
# Automatic Header file dependencies
#-------------------------------------------------------------------------------

include $(wildcard $(GENERATED_DEPS_FILES))

#-------------------------------------------------------------------------------
# QMore
#-------------------------------------------------------------------------------


ifndef QMORE_HOME

  ifeq ($(MAKECMDGOALS),qmore)
    qmore:
	$(Q) echo "Error: Environment variable QMORE_HOME is not set"
	$(Q)exit 1
  endif

else

  _SDE_QMORE ?= $(QMORE_HOME)/bin/qmore

  # If _QMORE_ALL is 1 or true then automatically run qmore.
  ifneq "$(filter 1 true,$(_QMORE_ALL))" ""
    all : qmore
  endif

  # If _<dirName>_QMORE is 1 or true then automatically run qmore.
  ifneq "$(filter 1 true,$(_$(notdir $(CURDIR))_QMORE))" ""
    all : qmore
  endif

  ifdef QMOREPARENTDIR
    _SDE_QMOREOPTS += -dr $(QMOREPARENTDIR)
  else
    ifdef QMOREPRODDIR
      _SDE_QMOREOPTS += -d $(QMOREPRODDIR)
    else
      _SDE_QMOREOPTS += -d .
    endif
  endif

  _SDE_QMOREOPTS += $(_QMORE_OPTS)

  _SDE_QMORE_NOSTDINC=$(filter -nostdinc,$(CFLAGS))
  _SDE_IS_AUTO_GENERATED:=Auto-generated by SDE2. Can be deleted.

  # Perl command to massage CFLAGS to additional options,
  # to store into the generated <_SDE_QAC_VIA_FILE> file.
  define _SDE_QMORE_COPTS_TO_VIA
      perl -n \
	-e '#while (s/(\S+)/ /) { print STDERR "arg $$1\n"; }' \
	-e 'while (s/\s-isystem\s+(\S+)/ /) { print "-i $$1\n-q $$1\n"; }' \
	-e 'while (s/\s-I(\s*\S+)/ /) { print "-i $$1\n-q $$1\n"; }' \
	-e 'while (s/\s-D(\w+)(=(\"[^\"]*\"|\`[^\`]*\`|\S*))?/ /) { print "-d $$1=$$3\n\n"; }'
  endef

  # Perl command to massage CFLAGS to extract -include options,
  # to store into generated <_SDE_QAC_FORCEINCLUDE_FILE> file.
  # This applies currently to GCC.
  define _SDE_QMORE_COPTS_TO_FORCEINCLUDE
      perl -n -e 'while (/\s-include\s+(\S+)/g) { print "#include \"$$1\"\n"; }'
  endef

  # Macro: _SDE_DIR_INTERM_QMORE
  # Temp directory for storing QMore input files, and outputs.
  _SDE_DIR_INTERM_QMORE=$(BUILDDIR)/qmore

  # Macro: _SDE_QM_CI_CONF_PROJECT_INC
  # Optional include file to be appended to the generated qm_ci_info.qconf file.
  _SDE_QM_CI_CONF_PROJECT_INC=$(_TMPROJECT)/qm_ci_info.qconf.include

  # Macro: _SDE_QAC_PROJECT_FORCEINCLUDE
  # Optional C include file to be included by the generated <_SDE_QAC_FORCEINCLUDE_FILE> file.
  _SDE_QAC_PROJECT_FORCEINCLUDE=$(_TMPROJECT)/qac_project_forceinclude.h

  # Macro: _SDE_QAC_VIA_FILE
  # SDE2-generated via file for use with QMore/QAC.
  _SDE_QAC_VIA_FILE=$(_SDE_DIR_INTERM_QMORE)/qac.via

  # Macro: _SDE_QAC_FORCEINCLUDE_FILE
  # SDE2-generated header file for use with QMore/QAC.
  # This file is passed to QAC, via <_SDE_QAC_VIA_FILE>, in a -forceinclude statement.
  _SDE_QAC_FORCEINCLUDE_FILE=$(_SDE_DIR_INTERM_QMORE)/qac_forceinclude.h

  qmore: qm_ci_info.qconf
  qm_ci_info.qconf : $(_SDE_QAC_VIA_FILE) $(_SDE_DIR_INTERM_QMORE) FORCE
	@echo 'Generating $(CURDIR)/$@'
	$(Q) $(_SDE_QM_CI_CONF_CLEANUP) if [ -f $@ ]; then mv -f $@ $@.orig; fi
	$(Q) echo >$@ '# $(_SDE_IS_AUTO_GENERATED)'
	$(Q) echo >>$@ '-config_type GENERAL'
	$(Q) echo >>$@ '-check_dependency false'
	$(Q) echo '-via $(_SDE_QAC_VIA_FILE)' >>$@
	$(Q) $(foreach viafile,$(_QMORE_QAC_VIA),\
	  tmpviafile=$(_SDE_DIR_INTERM_QMORE)/_auto_$(notdir $(viafile)); \
	  sed -e 's!\$${CROSS_COMPILE_BASE}!$(CROSS_COMPILE_BASE)!g' <$(viafile) $(if $(_SDE_QMORE_NOSTDINC),| grep -ve '^[ 	]*-[iq][ 	]') >$$tmpviafile && \
	  echo "-via $$tmpviafile" >>$@ && )true
	$(Q) if [ -f $(_SDE_QM_CI_CONF_PROJECT_INC) ]; then \
	  echo '' >>$@; \
	  echo '# Including file $(_SDE_QM_CI_CONF_PROJECT_INC)' >>$@; \
	  cat $(_SDE_QM_CI_CONF_PROJECT_INC) >>$@; \
	fi

  $(_SDE_QAC_VIA_FILE): $(dir $(_SDE_QAC_VIA_FILE)) $(_SDE_QAC_FORCEINCLUDE_FILE) FORCE
	@echo 'Generating $@'
	$(Q) echo '' $(CFLAGS) $(CXXFLAGS) | $(_SDE_QMORE_COPTS_TO_VIA) >$@
	$(Q) echo '-forceinclude $(_SDE_QAC_FORCEINCLUDE_FILE)' >>$@

  $(_SDE_QAC_FORCEINCLUDE_FILE): $(dir $(_SDE_QAC_FORCEINCLUDE_FILE)) FORCE
	@echo 'Generating $@'
	$(Q) echo '#ifndef QAC_FORCEINCLUDE_H' >$@
	$(Q) echo '#define QAC_FORCEINCLUDE_H' >>$@
	$(Q) if [ -f $(_SDE_QAC_PROJECT_FORCEINCLUDE) ]; then \
	  echo '#include "$(_SDE_QAC_PROJECT_FORCEINCLUDE)"' >>$@; \
	fi
	$(Q) echo '' $(CFLAGS) $(CXXFLAGS) | $(_SDE_QMORE_COPTS_TO_FORCEINCLUDE) >>$@
	$(Q) echo '#endif' >>$@

  # Cleanup command for generated qm_ci_info.qconf file.
  _SDE_QM_CI_CONF_CLEANUP=if grep -Fe '$(_SDE_IS_AUTO_GENERATED)' qm_ci_info.qconf >&/dev/null; then rm -f qm_ci_info.qconf; fi;

  # If _SDE_QMORE_DEBUG is set, then don't clean up the files:
  ifndef _SDE_QMORE_DEBUG
    _SDE_QMORE_CLEANUP=$(_SDE_QM_CI_CONF_CLEANUP)
  endif

  # Add -o option if not yet present (-o   resultDirName    Used to specify the location of the Results Directory.)
  ifeq "$(filter -o,$(_SDE_QMOREOPTS))" ""
    qmore: $(_SDE_DIR_INTERM_QMORE)
    _SDE_QMOREOPTS += -o $(_SDE_DIR_INTERM_QMORE)
  endif

  $(_SDE_DIR_INTERM_QMORE)/: FORCE
	$(Q)rm -rf $@; mkdir -p $@

  # If _QMORE_CONTINUE is true or 1, then a qmore error will be ignored:
  _SDE_QMORE_CONTINUE_ON_ERROR=$(if $(filter true 1,$(_QMORE_CONTINUE)),-)
  qmore:
	$(Q)$(_SDE_QMORE_CONTINUE_ON_ERROR) $(_SDE_QMORE) $(_SDE_QMOREOPTS); status=$$?; \
		$(_SDE_QMORE_CLEANUP) \
		exit $$status
endif
