#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: envvars.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060301   batelaan    Initial
#   2 20060405   neill       Ensure endianess flag is set correctly
#   3 20060804   batelaan    Moved some variables to envvars_cpu.mk
#   4 20060915   batelaan    Add NFSROOT and TFTPBOOT
#   5 20070629   batelaan    Contents of envvars_cpu.mk moved to platform_cpu.mk
#                            and platform_cpu_linux.mk
#   6 20070823   batelaan    Add documentation for missing variables.
#   7 20080319   batelaan    Merge with no changes, for CM purposes.

# File: envvars.mk
# Initialises/checks/documents environment variables used by the toplevel make session.

################################################################################
# Defined externally by the environment
################################################################################

# Variable: PRJROOT
# The root of the project tree. All makefiles must be inside this root.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,PRJROOT,Env var for root of workarea)

# Variable: BUILDROOT
# The directory in which all build results are stored.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,BUILDROOT,Env var for directory in which all build results are stored)

# Variable: _TMROOT
# Path to SDE2/NDK/DVP tree.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,_TMROOT,Env var for the directory containing SDE2 "sde" directory)

# Variable: _TMSITE
# Name of the local site for the SDE2 build environment (usual value
# soubhs).
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,_TMSITE,Env var for name of the local site for the SDE2 build environment)

# Variable: _TMDIVERSITY
# String indicating to SDE2 components how they should be built.
# Format is list of words each separated with an underscore. The whole string
# starts and ends with an underscore.
#
# Example: _smallram_hddecode_pnx833x_

# Variable: DI_PRJROOT
# The root of a Design-In project tree. Optional.
# Is normally set by a setup.sh script in a Design-In directory.

################################################################################
# Defined internally by the makefiles
################################################################################

# Define: THIS_MACHINE
# Defines the architecture of the build machine. Needed for crosstool.
#
# Value: i686-pc-linux-gnu
export THIS_MACHINE:=i686-pc-linux-gnu

################################################################################
# Defined internal by make
################################################################################

# NOTE: THESE ARE NOT ENVIRONMENT VARIABLES, but are here for documentation only.

# Define: MAKECMDGOALS
# Internal make variable contains goals to be made,
# as specified on the command line.

################################################################################
# Defined by prjconfig mechanism
################################################################################

# NOTE: these are here since the makefile documentation refers to them,
# so they need documenting somewhere.
# TODO: document these elsewhere.

# Define: CONFIG_NAME
# NAME configuration variable defined by the prjconfig mechanism.
# Indicates the overall name of the configuration that is built.
# Affects <BUILDROOT>.
