#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: lnx_userspace.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060804   batelaan    Initial contents based on $PRJROOT/Makefile version 7.
#   2 20070130   batelaan    Recategorise LNXLIBS&LNXAPPS.
#                            Improve buildspeed of SDE2 comps&apps.
#   3 20070130   batelaan    Protect against empty lists of SDE2 libraries/apps.
#   4 20070130   batelaan    Ensure SDE2 does not skip binary release components when installing.
#   5 20070131   batelaan    Add LNXEXAPPS to LNXAPPS in this file.
#   6 20070207   barnish     LNXLIBS_SDE2_SRC/NOSRC also checks subbdirs of /src
#   7 20070215   barnish     V=1 option no longers supresses build output
#   8 20070330   batelaan    Fix lnxapps.clean target
#   9 20070712   batelaan    Spaces before comma in if call causes problems with gmake 3.80.
#  10 20070824   batelaan    Doc update.
#  11 20071008   batelaan    Don't call install if we are already calling make
#  12 20071012   batelaan    Fix GNU make version 3.80 problem with ; in $(eval)
#  13 20071016   batelaan    Pass -b when passing -target:install so binary-released components
#                            are not skipped.
#  14 20071019   batelaan    Pass -b to all SDE2 libs/apps so that _dependent_ libraries of _source_
#                            components/apps are not skipped.
#  15 20080208   batelaan    Stop on build errors asap to help with debugging.
#  16 20080515   batelaan    Add cpp/cxx/c++ file types in source code presence check
#  17 20080620   batelaan    Corrected help msg for <name>.lnx.clean targets.
#  18 20080814   batelaan    Update BUILD_INFO_INPUT_USERLIBS
#  19 20080911   batelaan    Use SDE2_EXISTING_CI_LOOKUP to improve resilience

# File: lnx_userspace.mk
# Makefile fragment which provides a framework for building Linux userspace libraries and applications.
# Relevant make targets starts with lnx, and variables with LNX.
#
# - All SDE2 libraries/applications are built before non-SDE2 applications.
# - All libraries are SDE2, i.e. there are no non-SDE2 libraries.
# - All non-SDE2 applications are with source, i.e. only SDE2 libraries/applications
#   can be without source.
# - There are various names of targets that do the same thing:
#   XXX, XXX.make, and XXX.install intend to do the same.

#-------------------------------- LNXLIBS --------------------------------

# Define: LNXLIBS
# List of Linux userspace libaries to be built.
# Only include libraries that are not built as dependents of other libraries.
#
# Currently only libraries that build using SDE2 are supported.
#
# See: <lnx> <LNXLIBS_NOSRC> <LNXLIBS_SRC>

# Define: LNXLIBS_NOSRC
# List of Linux libaries without source code.
# Automatically determined.
# Only the install target is invoked for these.
#
# See: <lnx> <LNXLIBS>

# Define: LNXLIBS_SRC
# List of Linux libaries with source code.
# Automatically determined.
#
# See: <lnx> <LNXLIBS>

#-------------------------------- LNXAPPS --------------------------------

# Define: LNXEXAPPS
# Customer example applications.
# Source code is always available.
# Added to <LNXAPPS>.
#
# See: <lnx> <LNXAPPS>

# Define: LNXDEVAPPS
# Optional development applications. May not be present in the workarea.
# Each entry is added to <LNXAPPS> if it exists in the workarea.
#
# See: <lnx> <LNXAPPS>

# Define: LNXAPPS
# List of Linux userspace applications to be built.
# Automatically derived from <LNXEXAPPS> and <LNXDEVAPPS>.
#
# See: <lnx> <LNXAPPS_NOSRC> <LNXAPPS_SRC>

# Define: LNXAPPS_NOSRC
# List of Linux applications without source code.
# Automatically determined.
# Only the install target is invoked for these.
#
# See: <lnx> <lnxapps.install> <LNXAPPS>

# Define: LNXAPPS_SRC
# List of Linux applications with source code.
# Automatically determined.
#
# See: <lnx> <lnxapps.make> <lnxapps.install> <LNXAPPS>



# Target: lnx
# Builds userspace Linux libraries/applications:-
# - <lnxlibs>
# - <lnxapps>
#
# See also:-
# - <lnx.clean>
# - <lnx.make>
lnx: lnxapps
.PHONY: lnx

# Target: lnxlibs
# Builds userspace Linux libraries:-
#
# See also: <lnxlibs.clean> <lnxlibs.make>
.PHONY: lnxlibs

# Target: lnxapps
# Builds userspace Linux applications,
#
# See also: <lnxapps.clean> <lnxapps.make>
.PHONY: lnxapps

#-------------------------------- help --------------------------------

lnx.help: lnxlibs.help lnxapps.help
	@echo 'lnx              - Builds and installs userspace LINUX libraries/applications'
	@echo 'lnx.clean        - Cleans userspace LINUX libraries/applications'
	@echo 'lnx.help         - Lists userspace LINUX libraries/applications'
	@echo 'lnxlibs          - Builds and installs userspace LINUX libraries'
	@echo 'lnxlibs.clean    - Cleans userspace LINUX libraries'
	@echo '<name>.lnx       - Builds and installs userspace LINUX library/application <name>'
	@echo '<name>.lnx.clean - Cleans userspace LINUX library/application <name>'

.PHONY: lnx.help

lnxapps.help:
	@echo -n "  Userspace applications:"
	@echo '$(LNXAPPS)' | $(LIST2INDENTEDLINES)
.PHONY: lnxapps.help

lnxlibs.help:
	@echo -n "  Userspace libraries:"
	@echo '$(LNXLIBS)' | $(LIST2INDENTEDLINES)
.PHONY: lnxlibs.help

#-------------------------------- clean --------------------------------

# Target: lnx.clean
# Cleans all (<LNXLIBS> and <LNXAPPS>) Linux source libraries/applications.
#
# See: <lnxlibs.clean> <lnxapps.clean>
lnx.clean: lnxlibs.clean lnxapps.clean
.PHONY: lnx.clean

clean:: lnx.clean

# Target: lnxlibs.clean
# Cleans all (<LNXLIBS>) Linux source libraries.
.PHONY: lnxlibs.clean

# Target: lnxapps.clean
# Cleans all (<LNXAPPS>) Linux source applications.
.PHONY: lnxapps.clean

# Target: %.lnx.clean
# Generic pattern rules for Linux clean.
# Cleans Linux component or application.
# Distinquishes between SDE2 and non-SDE2 makefiles by grepping makefile
# for 'environment.mk' string.
#
# See: <%.lnx.make> <%.lnx.install> <CBUILD_LNXOPTS>
%.lnx.clean:
	@echo '  LNXCLEAN $*'
	$(Q)-$(if $(call BUILDS_WITH_SDE2,$*), \
	  cbuild $(CBUILD_LNXOPTS) -c $* , \
	  cd $(call SDE2_EXISTING_CI_LOOKUP,$*) && $(MAKECMD) clean \
	)


#-------------------------------- make/install --------------------------------

# Target: lnx.make
# Builds and installs all Linux libraries/applications.
# This is an alias for the <lnx> target.
#
# See: <lnxlibs.make> <lnxapps.make>
lnx.make: lnx
.PHONY: lnx.make

# Target: lnxlibs.make
# Builds and installs all Linux source (<LNXLIBS_SRC>) libraries.
# This is an alias for the <lnxlibs> target.
#
lnxlibs.make: lnxlibs
.PHONY: lnxlibs.make

# Target: lnxapps.make
# Builds and installs all Linux source (<LNXAPPS_SRC>) applications.
# This is an alias for the <lnxapps> target.
#
lnxapps.make: lnxapps
.PHONY: lnxapps.make

# Target: %.lnx.make
# Generic pattern rules for Linux builds and installs.
# Makes Linux component or application.
# Distinquishes between SDE2 and non-SDE2 makefiles by grepping makefile
# for 'environment.mk' string.
#
# See: <%.lnx.clean> <%.lnx.install> <CBUILD_LNXOPTS>
%.lnx.make:
	@echo '  LNXMAKE $*'
	$(Q)$(if $(call BUILDS_WITH_SDE2,$*), \
	  cbuild $(CBUILD_LNXOPTS) -r $* , \
	  cd $(call SDE2_EXISTING_CI_LOOKUP,$*) && $(MAKECMD) \
	)

#-------------------------------- install --------------------------------

# Target: lnx.install
# Builds/installs all Linux source and non-source libraries/applications.
# This is an alias for the <lnx> target.
#
# See: <LNXLIBS> <LNXAPPS>
# <lnxlibs.install> <lnxapps.install>
lnx.install: lnx
.PHONY: lnx.install

# Target: lnxlibs.install
# Installs all Linux source (<LNXLIBS_SRC>) and Linux non-source (<LNXLIBS_NOSRC>) libraries.
# This is an alias for the <lnxlibs> target.
#
# See: <LNXLIBS>
lnxlibs.install: lnxlibs
.PHONY: lnxlibs.install

# Target: lnxapps.install
# Installs all Linux source (<LNXAPPS_SRC>) and Linux non-source (<LNXAPPS_NOSRC>) applications.
# This is an alias for the <lnxapps> target.
#
# See: <LNXAPPS>
lnxapps.install: lnxapps
.PHONY: lnxapps.install


# Target: %.lnx.install
# Generic pattern rules for Linux installs:-
# - for SDE2 components/applications, it does a recursive build, with target 'install'.
# - for non-SDE2 components/applications, it just calls ${<MAKECMD>}.
#
# See: <%.lnx.clean> <%.lnx.make> <CBUILD_LNXOPTS>
%.lnx.install:
	@echo '  LNXINSTALL $*'
	$(Q)$(if $(call BUILDS_WITH_SDE2,$*), \
	  cbuild $(CBUILD_LNXOPTS) _TMNODEPENDENCIES=1 -r -B -b -target:install -- $* , \
	  cd $(call SDE2_EXISTING_CI_LOOKUP,$*) && $(MAKECMD) \
	)

#-------------------------------- LNX_USERSPACE_END --------------------------------

# Macro: SDE2_BUILD_EXE_OPTS
# Common options for SDE2 build_exe.pl script.
SDE2_BUILD_EXE_OPTS =

# Macro: LNX_SOURCE_CODE_EXTENSIONS
# List of source code file extensions.
#
# See: <LNX_HAS_SOURCE_CODE>
LNX_SOURCE_CODE_EXTENSIONS = s S c C cpp CPP cxx CXX c++ C++ cc CC

# Macro: LNX_HAS_SOURCE_CODE
# Returns a non-empty string if the component has source code files.
#
# Usage: $(call LNX_HAS_SOURCE_CODE,myComponentDirectory)
#
# See: <LNX_SOURCE_CODE_EXTENSIONS>
LNX_HAS_SOURCE_CODE = $(wildcard $(foreach x,$(LNX_SOURCE_CODE_EXTENSIONS),$(strip $1)/src/*.$x $(strip $1)/src/*/*.$x))

# Function: LNX_USERSPACE_END
# Indicates the end of LNX userspace macro definitions.
# This triggers target definitions and dependency setup.
#
# Usage:- $(call LNX_USERSPACE_END)
LNX_USERSPACE_END=$(eval $(value LNX_USERSPACE_END1))
define LNX_USERSPACE_END1

  #### Variables for libraries. ####

  # Add entries from LNXDEVLIBS to LNXLIBS only if they exist in the workarea.
  $(foreach c,$(LNXDEVLIBS),\
    $(if $(wildcard $(call SDE2_CI_LOOKUP,$c)),\
      $(eval LNXLIBS += $c)))

  # We don't have any non-SDE2 libraries.
  LNXLIBS_SDE2 = $(LNXLIBS)
  LNXLIBS_SRC = $(LNXLIBS_SDE2_SRC)
  LNXLIBS_NOSRC = $(LNXLIBS_SDE2_NOSRC)
  BUILD_INFO_INPUT_USERLIBS += $(LNXLIBS_SDE2)

  # Determine what are source and non-source libraries.
  # Initialise to empty so they are always defined.
  LNXLIBS_SDE2_SRC=
  LNXLIBS_SDE2_NOSRC=
  # for each lib entry check /src and one sub dir below it for ASM/C/C++ source files
  # & add entry to SRC or NOSRC list as appropriate
  $(foreach c, $(LNXLIBS_SDE2), \
	$(eval \
		$(if $(call LNX_HAS_SOURCE_CODE,$(call SDE2_EXISTING_CI_LOOKUP, $c)), \
			LNXLIBS_SDE2_SRC, \
			LNXLIBS_SDE2_NOSRC \
		 ) += $c \
	 ) \
  )

  #### Targets to make/install libaries. ####

  # Perform 'make' only on source libraries.
  # Perform 'install' on non-source libraries.
    lnxlibs:
  ifneq "$(strip $(LNXLIBS_SDE2_SRC))" ""
	@echo '  LNXLIBS MAKE (SDE2)'
	$(Q)perl $$_TMROOT/sde/scripts/build_exe.pl $(SDE2_BUILD_EXE_OPTS) $(foreach c,$(LNXLIBS_SDE2_SRC),$(call SDE2_EXISTING_CI_LOOKUP,$c))
  else
	@true
  endif
  ifneq "$(strip $(LNXLIBS_SDE2_NOSRC))" ""
	@echo '  LNXLIBS INSTALL (SDE2 non-source)'
	$(Q)_TMNODEPENDENCIES=1 perl $$_TMROOT/sde/scripts/build_exe.pl \
		-b $(SDE2_BUILD_EXE_OPTS) -target:install \
		$(foreach c,$(LNXLIBS_SDE2),\
			$(call SDE2_EXISTING_CI_LOOKUP,$c))
  else
	@true
  endif

  #### Variables for applications. ####

  LNXAPPS += $(LNXPLFAPPS) $(LNXEXAPPS)

  # Add entries from LNXDEVAPPS to LNXAPPS only if they exist in the workarea.
  $(foreach c,$(LNXDEVAPPS),\
    $(if $(wildcard $(call SDE2_CI_LOOKUP,$c)),\
      $(eval LNXAPPS += $c)))

  # Determine what are SDE2/non-SDE2 applications.
  # Initialise to empty so they are always defined.
  LNXAPPS_SDE2=
  LNXAPPS_NONSDE2=
  $(foreach c,$(LNXAPPS),$(eval $(if $(call BUILDS_WITH_SDE2,$c),LNXAPPS_SDE2,LNXAPPS_NONSDE2) += $c))

  # Determine what are source/non-source SDE2 applications.
  # Initialise to empty so they are always defined.
  LNXAPPS_SDE2_SRC=
  LNXAPPS_SDE2_NOSRC=
  $(foreach c,$(LNXAPPS_SDE2),$(eval $(if $(wildcard $(call SDE2_EXISTING_CI_LOOKUP,$c)/src/*.c),LNXAPPS_SDE2_SRC,LNXAPPS_SDE2_NOSRC) += $c))

  # Non-SDE2 apps are always with source code.

  LNXAPPS_SRC   = $(LNXAPPS_SDE2_SRC)   $(LNXAPPS_NONSDE2)
  LNXAPPS_NOSRC = $(LNXAPPS_SDE2_NOSRC)

  #### Targets to make/install applications. ####

  # First build SDE2 libs/apps, then non-SDE2 apps.
  lnxapps: lnxapps.nonsde2
  lnxapps.nonsde2: lnx.sde2

  lnxapps.nonsde2 lnxapps.nonsde2.make lnxapps.nonsde2.install: $(patsubst %,%.lnx.make,$(LNXAPPS_NONSDE2))

  # Perform make/install on all (source and non-source) SDE2 libraries/applications.
  lnx.sde2:
  ifneq "$(strip $(LNXLIBS_SDE2_SRC) $(LNXAPPS_SDE2_SRC))" ""
	@echo '  LNX MAKE (SDE2)'
	$(Q)perl $$_TMROOT/sde/scripts/build_exe.pl $(SDE2_BUILD_EXE_OPTS) $(foreach c,$(LNXLIBS_SDE2_SRC) $(LNXAPPS_SDE2_SRC),$(call SDE2_EXISTING_CI_LOOKUP,$c))
  else
	@true
  endif
  ifneq "$(strip $(LNXLIBS_SDE2_NOSRC) $(LNXAPPS_SDE2_NOSRC))" ""
	@echo '  LNX INSTALL (SDE2 non-source)'
	$(Q)perl $$_TMROOT/sde/scripts/build_exe.pl \
		-b $(SDE2_BUILD_EXE_OPTS) -target:install \
		$(foreach c,$(LNXLIBS_SDE2) $(LNXAPPS_SDE2), \
			$(call SDE2_EXISTING_CI_LOOKUP,$c))
  else
	@true
  endif

  # Create a shortcut target for each Linux library and application.
  # The definition depends on there being source available:
  # - with source: phStbRpc.lnx phStbRpc.lnx.install: phStbRpc.lnx.make
  # - without source: phStbRpc.lnx phStbRpc.lnx.make: phStbRpc.lnx.install
  #
  # Non source applications only perform the install step.
  $(foreach t, $(LNXLIBS_SRC)   $(LNXAPPS_SRC),   $(eval $t.lnx $t.lnx.install: $t.lnx.make\;))
  $(foreach t, $(LNXLIBS_NOSRC) $(LNXAPPS_NOSRC), $(eval $t.lnx $t.lnx.make: $t.lnx.install\;))

  # Cleaning
  lnxlibs.clean: $(patsubst %,%.lnx.clean,$(LNXLIBS) $(LNXLIBS_NOSRC))
  lnxapps.clean: $(patsubst %,%.lnx.clean,$(LNXAPPS) $(LNXAPPS_NOSRC))

endef
