#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: package.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060301    batelaan    Initial
#   2 20060802    batelaan    Add ALLBUILDPKGS ALLSKIPPKGS
#                             Use CPUBUILDROOT instead of BUILDROOT.
#   3 20060907    batelaan    Allow PKG_SETUP_VARS to be executed more than once.
#   4 20060920    batelaan    Fix help target: always runs.
#                             Delay evaluation of variables in tar and patch target commands
#                             to allow reuse.
#   5 20061025    batelaan    Introduce concept of <xxx_VARIANT>.
#   6 20061206    steel       change /opt/philips to /opt/nxp
#   7 20061207    neill       use _TMTGTPREFIX
# 7.1.1 20070525  steel       optimise for size
#   9 20070606    barnish     optimise according to prjconfig
#  10 20070606    barnish     rework
#  11 20070629    batelaan    Use $(MAKE) instead of make.
#  12 20070709    batelaan    Add symlink without package version
#                             to versioned package directory.
#  13 20070802    amccurdy    patch_CMDS: exclude .svn dirs when using rsync to apply overlay
#  14 20070823    batelaan    Correct use of $(PKG).patch_EXTRACMDS. Doc update.
#  15 20071016    steel       add -msoft-float when available
#  16 20071105    batelaan    Add -g if _TMTGTREL is set to debug.
#  17 20080214    batelaan    Make $(PKG).{patch,config,make,install} depend on pkg Makefile
#                             Add $(PKG).{helpfirst,helpmid} targets.
#  19 20080305    batelaan    Use CONFIG_xxx_VERSION
#  20 20080318    batelaan    Add xxx_DI_SRCDIR
#  21 20080319    laird       Add ability to set debug and optimisation flags.
#  22 20080703    batelaan    Mv env vars from DEFCONFIGOPTS to Configure
#  23 20080913    ocourson    Force rm to remove directories
#  24 20081211    batelaan    Fix xxx_DI_SRCDIR var to work with symlinks to $PRJROOT.
#  25 20090304    laird       Updates so that gstreamer plugins can be built.
#  26 20090324    laird       Make install-strip the default.
#  27 20090401    batelaan    Rm packages/<pkg> symlink in <pkg>.clean targets
#  xx 20090812    amccurdy    Only find files in $(PKG)_OVERLAYDIR to pass to OBJ_TARGET_OTHER_DEPS

# File: package.mk
# Handles building and installation of Linux (mainly open source) packages.
#
# Input variables:
# - PKG name of package, e.g. modutils.
# - <xxx_VERSION> version of package, e.g. 2.3.4 .
# - <xxx_VERSIONSEP> separator character used between PKG and <xxx_VERSION>, e.g. "-" (the default).
# - <xxx_VARIANT> optional variant of the package.

# Define: PKGSTDTARGETS
# Types of standard targets for a package.
#
# Value:
# help clean untar config make install
PKGSTDTARGETS:=help clean untar patch config make install

clean_STDPREDECESSOR =
clean_PREDECESSOR = $(clean_STDPREDECESSOR)

untar_STDPREDECESSOR =
untar_PREDECESSOR = $(untar_STDPREDECESSOR)

patch_STDPREDECESSOR = untar
patch_PREDECESSOR = $(patch_STDPREDECESSOR)

config_STDPREDECESSOR = patch
config_PREDECESSOR = $(config_STDPREDECESSOR)

make_STDPREDECESSOR = config
make_PREDECESSOR = $(make_STDPREDECESSOR)

install_STDPREDECESSOR = make
install_PREDECESSOR = $(install_STDPREDECESSOR)

EXTPKGSTDTARGETS:=help clean untar $(foreach f,patch config make install,$f)

help::
	@echo "pkg.help         - Help on building packages"
	@echo "pkg.versions     - List package versions"

# Target: pkg.help
# Shows help on building packages.
pkg.help:
	@echo 'Make command line variables to help build/debug packages:'
	@echo '(these should never be set in Makefiles, but only on the make command line)'
	@echo 'XCF	- eXtra Flags passed into CFLAGS+CPPFLAGS env vars for configure'
	@echo 'XLF	- eXtra Flags passed into LDFLAGS env var for configure'
	@echo 'XCE	- eXtra Environment variables set before configure is called'
	@echo 'XME	- eXtra Environment variables set before make is called'
	@echo 'XIE	- eXtra Environment variables set before make install is called'
	@echo 'XCO	- eXtra Options passed to configure'
	@echo 'XMO	- eXtra Options passed to make'
	@echo 'XIO	- eXtra Options passed to make install'
.PHONY: pkg.help

# Target: pkg.versions
# Lists all package versions.
#
# See:-
# <ALLPKGFULLNAMES>
pkg.versions:
	@for p in $(sort $(ALLPKGFULLNAMES)); do echo $$p; done
.PHONY: pkg.versions

# Target: pkg.clean
# Runs the clean target for each package in <ALLBUILDPKGS>.
pkg.clean:
.PHONY: pkg.clean

# Define: ALLPKGS
# Contains all instances of <PKG>.
# <PKG> is appended everytime <PKG_SETUP_VARS> is called.
#
# See:-
# <ALLBUILDPKGS> <ALLSKIPPKGS>
ALLPKGS=

# Define: ALLBUILDPKGS
# Contains all entries from <ALLPKGS>, for which <xxx_SKIPBUILD> is not true.
#
# See:-
# <pkg.clean> <ALLPKGS> <ALLSKIPPKGS>
ALLBUILDPKGS=$(foreach p,$(ALLPKGS),$(if $(filter true,$($(p)_SKIPBUILD)),,$p))

# Define: ALLSKIPPKGS
# Contains all entries from <ALLPKGS>, for which <xxx_SKIPBUILD> is true.
#
# See:-
# <ALLPKGS> <ALLBUILDPKGS>
ALLSKIPPKGS=$(foreach p,$(ALLPKGS),$(if $(filter true,$($(p)_SKIPBUILD)),$p)

# Define: ALLPKGFULLNAMES
# Contains all instances of <xxx_FULLNAME> from <ALLPKGS>.
ALLPKGFULLNAMES=$(foreach p,$(ALLPKGS),$($(p)_FULLNAME))

# Define: TARBALL_EXTS
# Extensions of tarball files that we can cope with.
TARBALL_EXTS := tar.gz tgz tar.bz2 tar.gz zip

# Target: pkg.listtarballs
# Prints the values of the <xxx_TARBALL> variables,
# if it is not empty, for each package in <ALLPKGS>.
.PHONY: pkg.listtarballs

# This target rule will print the value of the <xxx_TARBALL> variable for a package,
# if it is not empty.
# Note: In <PKG_SETUP> <PKG>.pkg.listtarball is added to the dependencies of the
# <pkg.listtarballs> target.
%.pkg.listtarball:
	@if [ ! -z "$($*_TARBALL)" ]; then echo $($*_TARBALL); fi

################################################################################
# Setup
################################################################################

# EG for making modutils:
export BUILDCC = gcc

# Define: PKGPATCH
# Indicates if we should generally patch packages (value is true) or not (value is false).
# The default is true, i.e. we do patch.
# For each individual package one can override this by setting the variable
# <xxx_PATCH>.
#
# This variable can be set by specifying the <nopatch> target on the make command line.
#
# See:-
# <nopatch> <xxx_PATCH> <xxx.patch>
PKGPATCH := $(if $(filter nopatch,$(MAKECMDGOALS)),false,true)
$(call CHECK_VAR_VALUE,PKGPATCH,false true)

.PHONY: nopatch
# Target: nopatch
# Sets variable <PKGPATCH> to false. This means that patching is skipped!
#
# See:-
# <PKGPATCH> <xxx_PATCH> <xxx.patch>
nopatch:
	$(eval PKGPATCH:=false)

################################################################################
# PKG_START
################################################################################

# Define: PKG_START
# Indicates the start of a package definition.
#
# Usage:- $(eval $(PKG_START))
# $(eval $(PKG_START))

define PKG_START
  $(eval $(PKG_UNDEF))
endef

define PKG_UNDEF
  PKG=
endef

# Define: PKG
# Defines the name (without version) of a package. Required in a package Makefile.
# Also referred to sometimes as xxx, for example as in <xxx_VERSION>.

# Define: xxx
# Used in variable names to indicate the value of a <PKG>.
# See for example <xxx_VERSION>.

# Define: xxx_VERSION
# Defines the version of a package. Usually required in a package Makefile
# if the input of a package is a tarball with a version in its name.

# Define: xxx_VERSIONSEP
# Defines the separator between the name and the version of a package.
# Default: "-"
# Usually not required in a package Makefile.

# Define: xxx_VARIANT
# Defines the variant of a package.
# Default: <PKG_VARIANT_DEFAULT>
# Usually not required in a package Makefile.

# Define: PKG_VARIANT_DEFAULT
# Default value of <xxx_VARIANT> ("default").
#
# See also:- <PKG_VARIANT_SELECT>

# Define: xxx_SKIPBUILD
# If true, then the package should not be built (or cleaned) during a 'normal' build.
# For most packages the default value is 'false' (it is set in <PKG_SETUP_VARS> to false
# if the variable has not yet been defined).
#
# Note that any <xxx_SKIPBUILD> variable must be defined before the Makefile
# defining the package is included.
#
# See also:- <PKG_SETUP_VARS> <ALLBUILDPKGS> <ALLSKIPPKGS>

################################################################################
# Default C/C++ Opts (excluding includes and defines)
################################################################################
# Define: DEFOPTIMISATION
# The optimisation value selected for either size or speed by the diversity flag
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_0),y)
DEFOPTIMISATION=-O0
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_1),y)
DEFOPTIMISATION=-O1
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_2),y)
DEFOPTIMISATION=-O2
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_3),y)
DEFOPTIMISATION=-O3
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_S),y)
DEFOPTIMISATION=-Os
endif

# Define: DEFDEBUGGING
# The debug level that is used if we have set debug in prjconfig.
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_1),y)
DEFDEBUGGING=-g1
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_2),y)
DEFDEBUGGING=-g2
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_3),y)
DEFDEBUGGING=-g3
endif

# Define: DEFCOPTS
# Default C options (excluding includes and defines).
# If <_TMTGTREL> is set to debug, <DEFDEBUGGING> is added.
#
# See also:- <xxx_CFLAGS> <xxx_DEFCOPTS> <DEFCPPOPTS>
DEFCOPTS:=$(DEFOPTIMISATION) $(DEFDEBUGGING)
ifeq "$(ARCH)" "mips"
    DEFCOPTS+=-march=mips32r2 -mtune=4kc
endif
ifneq "$(filter mipsel-linux- mips-linux-,$(CROSS_COMPILE))" ""
    DEFCOPTS+=-msoft-float
endif

# Define: xxx_DEFCOPTS
# Default C opts  (excluding includes and defines) passed to the
# CFLAGS environment variable value for a package <xxx.config> target.
# Formed from <DEFCOPTS>. Can be changed in a package's Makefile.
#
# See also:- <xxx_CFLAGS>

# Define: DEFCPPOPTS
# Default C++ options (excluding includes and defines).
#
# See also:- <xxx_CPPFLAGS> <xxx_DEFCPPOPTS> <DEFCOPTS>
DEFCPPOPTS:=$(DEFCOPTS)

# Define: xxx_DEFCPPOPTS
# Default C++ opts  (excluding includes and defines) passed to the
# CPPFLAGS environment variable value for a package <xxx.config> target.
# Formed from <DEFCPPOPTS>. Can be changed in a package's Makefile.
#
# See also:- <xxx_CPPFLAGS>

################################################################################
# Default C/C++ Includes
################################################################################

# Define: DEFCINCLUDES
# Default C/C++ includes.
# Default value:
# - -I$(_TMROOT)/inc
# - -I$(BUILDROOT)
# - -I$(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)
# - -I$(<STAGINGDIR>)$(_TMTGTPREFIX)/include
# - -I$(<STAGINGDIR>)/usr/local/include
# - -I$(<STAGINGDIR>)/usr/include
# - -I$(<KSRC>)/include
DEFCINCLUDES:=	\
	-I$(_TMROOT)/inc \
	-I$(BUILDROOT) \
	-I$(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE) \
	-I$(STAGINGDIR)$(_TMTGTPREFIX)/include \
	-I$(STAGINGDIR)/usr/local/include \
	-I$(STAGINGDIR)/usr/include \
	-I$(STAGINGDIR)/include

# Define: xxx_DEFCINCLUDES
# Default C includes passed to the CFLAGS environment variable value for a package <xxx.config> target.
# Formed from <DEFCINCLUDES>. Can be changed in a package's Makefile.
#
# See also:- <xxx_CFLAGS>

################################################################################
# Default C/C++ Defines
################################################################################

# Define: DEFCDEFINES
# Default C/C++ defines. Values should include -D, for example -DPNX8550=1.
#
# Default value: empty.
#
# See:- <xxx_CFLAGS>
DEFCDEFINES:=

# Define: xxx_DEFCDEFINES
# Default C defines passed to the CFLAGS environment variable value for a package <xxx.config> target.
# Formed from <DEFCDEFINES>. Can be changed in a package's Makefile.
#
# See also:- <xxx_CFLAGS>

################################################################################
# Default LD Flags
################################################################################

# Define: DEFLDFLAGS
# Default LD flags.
#
# Default value: -L$(<ROOTFS>)$(_TMTGTPREFIX)/lib -L$(<ROOTFS>)/usr/local/lib -L$(<ROOTFS>)/usr/lib -L$(<ROOTFS>)/lib
#
# See:- <xxx_CFLAGS>
DEFLDFLAGS:=-L$(ROOTFS)$(_TMTGTPREFIX)/lib -L$(ROOTFS)/usr/local/lib -L$(ROOTFS)/usr/lib -L$(ROOTFS)/lib

################################################################################
# X*F
################################################################################

# Define: XCF
# Variable for environment variable CFLAGS to set before calling configure
# in target <xxx.config>.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_CFLAGS>
#
# See:- <xxx_CONFIGENV> <xxx.config>
XCF:=

# Define: XLF
# Variable for environment variable LDFLAGS to set before calling configure
# in target <xxx.config>.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_LDFLAGS>
#
# See:- <xxx_CONFIGENV> <xxx.config>
XLF:=

################################################################################
# xxx_DEF...FLAGS
################################################################################

# Define: xxx_DEFCFLAGS
# Default CFLAGS environment variable value for a package <xxx.config> target.
# Value:-
# - <xxx_DEFCOPTS>
# - <xxx_DEFCDEFINES>
# - <xxx_DEFCINCLUDES>.
#
# See also:- <xxx_CFLAGS>

# Define: xxx_DEFCPPFLAGS
# Default CPPFLAGS environment variable value for a package <xxx.config> target.
# Value:-
# - <xxx_DEFCPPOPTS>
# - <xxx_DEFCDEFINES>
# - <xxx_DEFCINCLUDES>.
#
# See also:- <xxx_CPPFLAGS>

# Define: xxx_DEFLDFLAGS
# Default LDFLAGS environment variable value for a package <xxx.config> target.
#
# See also:- <xxx_LDFLAGS>

################################################################################
# xxx_...FLAGS
################################################################################

# Define: xxx_CFLAGS
# CFLAGS environment variable value for a package <xxx.config> target.
# Formed from <xxx_DEFCFLAGS> and <XCF>.

# Define: xxx_CPPFLAGS
# CPPFLAGS environment variable value for a package <xxx.config> target.
# Formed from <xxx_DEFCPPFLAGS> and <XCF>.

# Define: xxx_LDFLAGS
# LDFLAGS environment variable value for a package <xxx.config> target.
# Formed from <xxx_DEFLDFLAGS> and <XLF>.

################################################################################
# DEF...OPTS
################################################################################

# Define: DEFCONFIGOPTS
# Default options for the configure command.
#
# Value:-
# - if <CROSS_COMPILE> is empty: --prefix=$(ROOTFS)
# - otherwise:
#   - --prefix=/usr --exec-prefix=/usr
#   - --sysconfdir=/etc
#   - --build=$(THIS_MACHINE)
#   - --host=$(TARGET)
#
# By default we install in $(<ROOTFS>), if you want to install for example in $(<ROOTFS>)/usr
# then use this:-
# (start code)
# xxx_DEFCONFIGOPTS := $(patsubst --prefix=%,--prefix=%/usr,$(xxx_DEFCONFIGOPTS))
# (end)
#
# See also:-
# - <xxx.config>
# - <XCO>
# - <ROOTFS>
# - <THIS_MACHINE>
# - <TARGET>
DEFCONFIGOPTS:= \
  --prefix=/usr \
  --exec-prefix=/usr \
  --sysconfdir=/etc \
  $(if $(CROSS_COMPILE), --build=$(THIS_MACHINE) --host=$(TARGET))

# Define: DEFMAKEOPTS
# Default options for the make command.
#
# Value:
# CC_FOR_BUILD=gcc AS_FOR_BUILD=as AR_FOR_BUILD=ar LD_FOR_BUILD=ld RANLIB_FOR_BUILD=ranlib
#
# See also:- <xxx.make> <XMO>
DEFMAKEOPTS:=CC_FOR_BUILD=gcc AS_FOR_BUILD=as AR_FOR_BUILD=ar LD_FOR_BUILD=ld RANLIB_FOR_BUILD=ranlib

# Define: DEFINSTALLOPTS
# Default options for the make install command.
#
# Value is DESTDIR=$(<ROOTFS>) by default.
#
# See also:- <xxx.install> <XIO>
DEFINSTALLOPTS:= DESTDIR=$(ROOTFS)

################################################################################
# X*E - eXtra...Environment variable
################################################################################

# Define: XCE
# Variable for additional environment variables to set before calling configure
# in target <xxx.config>.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_CONFIGENV>
#
# See:- <xxx_CONFIGENV> <xxx.config>
XCE:=

# Define: XME
# Variable for additional environment variables to set before calling make
# in target <xxx.make>.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_MAKEENV>
#
# See:- <xxx_MAKEENV> <xxx.make>
XME:=

# Define: XIE
# Variable for additional environment variables to set before calling make install
# in target <xxx.install>.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_INSTALLENV>
#
# See:- <xxx_INSTALLENV> <xxx.install>
XIE:=

################################################################################
# DEF...ENV
################################################################################

# Define: DEFCONFIGENV
# This variable added to the start of the default value for <xxx_DEFCONFIGENV>.
#
# See also:- <xxx_CONFIGENV> <xxx.config>
DEFCONFIGENV:= \
  $(if $(CROSS_COMPILE), \
  PKG_CONFIG_SYSROOT_DIR="$(STAGINGDIR)" \
  PKG_CONFIG="$(BUILDROOT)/packages/buildroot/build_mipsel/host_dir/usr/bin/pkg-config" \
  PKG_CONFIG_PATH="$(STAGINGDIR)/usr/lib/pkgconfig/")

# Define: DEFMAKEENV
# Default environment variables to set before calling make in target <xxx.make>.
# Default value is empty.
#
# See also:- <xxx_MAKEENV> <xxx.make>
DEFMAKEENV:=

# Define: DEFINSTALLENV
# Default environment variables to set before calling make install in target <xxx.install>.
# Default value is empty.
#
# See also:- <xxx_INSTALLENV> <xxx.install>
DEFINSTALLENV:=

################################################################################
# xxx_DEF...ENV
################################################################################

# Define: xxx_DEFCONFIGENV
# Default environment variables to set before calling configure for this package.
# <xxx_CONFIGENV> is formed from this variable, and <XCE>.
# Default value for this variable is:-
#
# <DEFCONFIGENV> CFLAGS='${<xxx_CFLAGS>}' CPPFLAGS='${<xxx_CPPFLAGS>}' LDFLAGS='${<xxx_LDFLAGS>}'
#
#
# See:-
# <xxx.config>.

# Define: xxx_DEFMAKEENV
# Default environment variables to set before calling make for this package.
# <xxx_MAKEENV> is formed from this variable, and <XME>.
# Default value for this variable is <DEFMAKEENV>.
#
# See:-
# <xxx.make>.

# Define: xxx_DEFINSTALLENV
# Default environment variables to set before calling make install for this package.
# <xxx_INSTALLENV> is formed from this variable, and <XIE>.
# Default value for this variable is  <DEFINSTALLENV>.
#
# See:-
# <xxx.install>.

################################################################################
# xxx_...ENV
################################################################################

# Define: xxx_CONFIGENV
# This variable contains the environment variables set before running configure for target <xxx.config>.
# It is made up of:-
# - <xxx_DEFCONFIGENV>
# - <XCE>

# Define: xxx_MAKEENV
# This variable contains the environment variables set before running make for target <xxx.make>.
# It is made up of:-
# - <xxx_DEFMAKEENV>
# - <XME>

# Define: xxx_INSTALLENV
# This variable contains the environment variables set before running make install for target <xxx.install>.
# It is made up of:-
# - <xxx_DEFINSTALLENV>
# - <XIE>

################################################################################
# xxx_DEF...OPTS
################################################################################

# Define: xxx_DEFCONFIGOPTS
# Default options for the configure command for this package.
# <xxx_CONFIGOPTS> is formed from this variable, and <XCO>.
# Default value for this optional variable is <DEFCONFIGOPTS>.
#
# See:-
# <xxx.config>.


# Define: xxx_DEFMAKEOPTS
# Default options for the make command for this package.
# <xxx_MAKEOPTS> is formed from this variable, and <XMO>.
# Default value for this optional variable is <DEFMAKEOPTS>.
#
# See:-
# <xxx.make>.

# Define: xxx_DEFINSTALLOPTS
# Default options for the make install command for this package.
# <xxx_INSTALLOPTS> is formed from this variable, and <XIO>.
# Default value for this optional variable is <DEFINSTALLOPTS>.
#
# See:-
# <xxx.install>.

################################################################################
# X*O - eXtra...Options
################################################################################

# Define: XCO
# Variable for additional options to pass to the configure command (if used) for ANY package.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_CONFIGOPTS>
#
# See:- <xxx_CONFIGOPTS> <xxx.config>
XCO:=

# Define: XMO
# Variable for additional options to pass to the make command (if used) for ANY package.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_MAKEOPTS>
#
# See:- <xxx_MAKEOPTS> <xxx.make>
XMO:=

# Define: XIO
# Variable for additional options to pass to the make install command (if used) for ANY package.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
# Is automatically appended to <xxx_INSTALLOPTS>
#
# See:- <xxx_INSTALLOPTS> <xxx.install>
XIO:=

################################################################################
# xxx_...OPTS
################################################################################

# Define: xxx_CONFIGOPTS
# The total list of options for the configure command for this package.
# Automatically derived from <xxx_DEFCONFIGOPTS>, and <XCO>.
# Defined by calling <PKG_SETUP_VARS>.
#
# See:-
# <xxx.config>.

# Define: xxx_MAKEOPTS
# The total list of options for the make command for this package.
# Automatically derived from <xxx_DEFMAKEOPTS>, and <XMO>.
# Defined by calling <PKG_SETUP_VARS>.
#
# See:-
# <xxx.make>.

# Define: xxx_INSTALLOPTS
# The total list of options for the make install command for this package.
# Automatically derived from <xxx_DEFINSTALLOPTS>, and <XIO>.
# Defined by calling <PKG_SETUP_VARS>.
#
# See:-
# <xxx.install>.

################################################################################
# xxx_...CMDS
################################################################################

# Define: xxx.help_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.help> target.

# Define: xxx.help_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.help> target.

# Define: xxx.clean_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.clean> target.

# Define: xxx.clean_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.clean> target.

# Define: xxx.untar_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.untar> target.
# Run after untarring and patching.
#
# See:-
# <xxx.untar_POSTUNTARCMDS>

# Define: xxx.untar_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.untar> target.
# Run after untarring and patching.
#
# See:-
# <xxx.untar_POSTUNTARCMDS>

# Define: xxx.config_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.config> target.

# Define: xxx.config_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.config> target.

# Define: xxx.make_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.make> target.

# Define: xxx.make_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.make> target.

# Define: xxx.install_EXTRACMDSBEFORE
# Optional definition of extra commands executed before the <xxx.install> target.

# Define: xxx.install_EXTRACMDS
# Optional definition of extra commands executed after the <xxx.install> target.

################################################################################
# PKG_SETUP_VARS
################################################################################

# Define: PKG_SETUP_VARS
# Sets up the package variables from the input variables (replace <xxx> with value of <PKG>)
# - <PKG>
# - <xxx_VERSION> (define as empty if not versioned)
# - <xxx_VERSIONSEP> (optional; default is a minus sign)
# - <xxx_VARIANT> (optional; default is <PKG_VARIANT_DEFAULT>)
# - <xxx_DEFCONFIGOPTS> (optional)
# - <xxx_DEFMAKEOPTS> (optional)
# - <xxx_DEFINSTALLOPTS> (optional)
# - <xxx_PATCH> (optional)
#
# Automatically called from <PKG_BUILD_USING_CONFIG> and <PKG_BUILD_USING_MAKE>.
#
# The package variables defined are:-
# - <xxx_FULLNAME>
# - <xxx_SRCDIR>
# - <xxx_DI_SRCDIR>
# - <xxx_BUILDDIR>
# - <xxx_TARBALL>
# - <xxx_UNTARINSIDE>
# - <xxx_PATCH>
# - <xxx_PATCHES>
# - <xxx_SKIPBUILD>
# - <xxx_DEFCDEFINES>
# - <xxx_DEFCINCLUDES>
# - <xxx_DEFCFLAGS> <xxx_CFLAGS>
# - <xxx_DEFCPPFLAGS> <xxx_CPPFLAGS>
# - <xxx_DEFLDFLAGS> <xxx_LDFLAGS>
# - <xxx_DEFCONFIGENV> <xxx_CONFIGENV>
# - <xxx_DEFCONFIGOPTS> <xxx_CONFIGOPTS>
# - <xxx_DEFMAKEENV> <xxx_MAKEENV>
# - <xxx_DEFMAKEOPTS> <xxx_MAKEOPTS>
# - <xxx_DEFINSTALLENV> <xxx_INSTALLENV>
# - <xxx_DEFINSTALLOPTS> <xxx_INSTALLOPTS>
#
# Usage:
# $(eval $(PKG_SETUP_VARS))

PKG_SETUP_VARS=$(if $($(PKG)_SETUP_VARS_DONE),,$(PKG_SETUP_VARS1))

define PKG_SETUP_VARS1
  $(PKG)_SETUP_VARS_DONE:=true
  ALLPKGS += $(PKG)
  .PHONY: $(PKG)

  ifndef $(PKG)_VERSIONSEP
    ifneq ($($(PKG)_VERSION),)
      $(PKG)_VERSIONSEP:=-
    endif
  endif

  $(PKG)_VARIANT ?= $(PKG_VARIANT_DEFAULT)

  $(PKG)_FULLNAME := $(PKG)$$(if $($(PKG)_VERSION),$$($(PKG)_VERSIONSEP))$($(PKG)_VERSION)

  $(PKG)_SKIPBUILD ?= false

  $(PKG)_DEFCONFIGOPTS ?= $$(DEFCONFIGOPTS)
  $(PKG)_CONFIGOPTS = $$($(PKG)_DEFCONFIGOPTS) $$(XCO)

  $(PKG)_DEFCOPTS ?= $$(DEFCOPTS)
  $(PKG)_DEFCPPOPTS ?= $$(DEFCPPOPTS)

  $(PKG)_DEFCDEFINES ?= $$(DEFCDEFINES)

  $(PKG)_DEFCINCLUDES ?= $$(DEFCINCLUDES)

  $(PKG)_DEFCFLAGS ?= $$($(PKG)_DEFCOPTS) $$($(PKG)_DEFCDEFINES) $$($(PKG)_DEFCINCLUDES)
  $(PKG)_CFLAGS = $$($(PKG)_DEFCFLAGS) $$(XCF)

  $(PKG)_DEFCPPFLAGS ?= $$($(PKG)_DEFCPPOPTS) $$($(PKG)_DEFCDEFINES) $$($(PKG)_DEFCINCLUDES)
  $(PKG)_CPPFLAGS = $$($(PKG)_DEFCPPFLAGS) $$(XCF)

  $(PKG)_DEFLDFLAGS ?= $$(DEFLDFLAGS)
  $(PKG)_LDFLAGS = $$($(PKG)_DEFLDFLAGS) $$(XLF)

  $(PKG)_DEFCONFIGENV ?= $$(DEFCONFIGENV) CFLAGS='$$($(PKG)_CFLAGS)' CPPFLAGS='$$($(PKG)_CPPFLAGS)' LDFLAGS='$$($(PKG)_LDFLAGS)'
  $(PKG)_CONFIGENV = $$($(PKG)_DEFCONFIGENV) $$(XCE)


  $(PKG)_DEFMAKEENV ?= $$(DEFMAKEENV)
  $(PKG)_MAKEENV = $$($(PKG)_DEFMAKEENV) $$(XME)

  $(PKG)_DEFMAKEOPTS ?= $$(DEFMAKEOPTS)
  $(PKG)_MAKEOPTS = $$($(PKG)_DEFMAKEOPTS) $$(XMO)

  $(PKG)_DEFINSTALLENV ?= $$(DEFINSTALLENV)
  $(PKG)_INSTALLENV = $$($(PKG)_DEFINSTALLENV) $$(XIE)

  $(PKG)_DEFINSTALLOPTS ?= $$(DEFINSTALLOPTS)
  $(PKG)_INSTALLOPTS = $$($(PKG)_DEFINSTALLOPTS) $$(XIO)

  $(PKG)_SRCDIR := $(THISDIR)
  $(PKG)_DI_SRCDIR := $(firstword $(wildcard $(if $(DI_PRJROOT),$(patsubst $(realpath $(PRJROOT))/%,$(DI_PRJROOT)/%,$(THISDIR))) $(THISDIR)))
  $(PKG)_BUILDDIR := $(CPUBUILDROOT)/packages/$$($(PKG)_FULLNAME)
  $(PKG)_TARBALL := $$(firstword $$(wildcard $$(foreach d,$(THISDIR) $(TARBALLS),$$(foreach x,$(TARBALL_EXTS),$$d/$$($(PKG)_FULLNAME).$$x))))
  $(PKG)_UNTARINSIDE ?=
  $$(call DOTRACE,'$(PKG)_TARBALL = $$($(PKG)_TARBALL)')

  $(PKG)_PATCH := $(PKGPATCH)
  $(PKG)_PATCHDIR = $$(firstword $$(wildcard \
    $$($(PKG)_SRCDIR)/$$(call PKG_VARIANT_SELECT,$(PKG),patches)-$$($(PKG)_VERSION) \
    $$($(PKG)_SRCDIR)/$$(call PKG_VARIANT_SELECT,$(PKG),patches)))
  $(PKG)_PATCHES = $$(wildcard $$($(PKG)_PATCHDIR)/*.patch)
  $(PKG)_OVERLAYDIR = $$(firstword $$(wildcard \
    $$($(PKG)_SRCDIR)/$$(call PKG_VARIANT_SELECT,$(PKG),overlay)-$$($(PKG)_VERSION) \
    $$($(PKG)_SRCDIR)/$$(call PKG_VARIANT_SELECT,$(PKG),overlay)))

  $(PKG).untar_POSTUNTARCMDS ?=

  $$(call CHECK_VAR_VALUE,$(PKG)_PATCH,true false)
  pkg.listtarballs: $(PKG).pkg.listtarball
endef

# Define: xxx_VERSION
# The version of the package. If the package has no version then define as empty.


# Define: xxx_VERSIONSEP
# The version separator of the package. Optional (default is the minus sign).


# Define: xxx_FULLNAME
# The name of the package with the version info.
#
# Automatically derived by appending
# - <PKG>
# - <xxx_VERSIONSEP>
# - <xxx_VERSION>
#
# Defined by calling <PKG_SETUP_VARS>.


# Define: xxx_SRCDIR
# The source directory of the package.
# Automatically derived from the directory containing the makefile (<THISDIR>).
# Defined by calling <PKG_SETUP_VARS>.
#
# See: <linux_DI_SRCDIR>


# Define: xxx_DI_SRCDIR
# Optional 'source' directory for the package inside <DI_PRJROOT>.
#
# By default this is the same as <xxx_SRCDIR>,
# unless <DI_PRJROOT> is defined, and a directory below <DI_PRJROOT>
# exists with the same path as <xxx_SRCDIR> has below <PRJROOT>.
#
# Defined by calling <PKG_SETUP_VARS>.
#
# See: <linux_SRCDIR>


# Define: xxx_PATCHDIR
# The directory containing patches for the package.
# Automatically derived from <xxx_SRCDIR>, <xxx_VERSION>, and <xxx_VARIANT>.
# The first directory that exists in the following list is used.
# If we use the default variant (<xxx_VARIANT> == <PKG_VARIANT_DEFAULT>):
# - <xxx_SRCDIR>/patches-<xxx_VERSION>
# - <xxx_SRCDIR>/patches
# Otherwise:
# - <xxx_SRCDIR>/patches_<xxx_VARIANT>-<xxx_VERSION>
# - <xxx_SRCDIR>/patches_<xxx_VARIANT>
# If none of these exist, it is empty.
# Defined by calling <PKG_SETUP_VARS>.

# Define: xxx_OVERLAYDIR
# The directory containing the overlay tree for the package.
# Automatically derived from <xxx_SRCDIR>, <xxx_VERSION>, and <xxx_VARIANT>.
# The first directory that exists in the following list is used.
# If we use the default variant (<xxx_VARIANT> == <PKG_VARIANT_DEFAULT>):
# - <xxx_SRCDIR>/overlay-<xxx_VERSION>
# - <xxx_SRCDIR>/overlay
# Otherwise:
# - <xxx_SRCDIR>/overlay_<xxx_VARIANT>-<xxx_VERSION>
# - <xxx_SRCDIR>/overlay_<xxx_VARIANT>
# If none of these exist, it is empty.
# Defined by calling <PKG_SETUP_VARS>.


# Define: xxx_BUILDDIR
# The build directory of the package.
#
# Defined by calling <PKG_SETUP_VARS>.
#
# Default Value:
# <CPUBUILDROOT>/packages/<xxx_FULLNAME>


# Define: xxx_TARBALL
# The tarball of the package. May be empty if no tarball was found.
# Defined by calling <PKG_SETUP_VARS>.

# Function: PKG_VARIANT_SELECT
# Append "_${<xxx_VARIANT>}" (if it is different from <PKG_VARIANT_DEFAULT>) to its argument.
#
# Arguments:-
# - 1: Name of package.
# - 2: Name to embellish with <xxx_VARIANT> value.
#
# Example:
# $(call PKG_VARIANT_SELECT,linux,patches)
PKG_VARIANT_SELECT=$(strip $(2))$(if $(filter-out $(PKG_VARIANT_DEFAULT),$($(1)_VARIANT)),_$($(1)_VARIANT))

################################################################################
# Target: xxx.help
# Prints help for a package. This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the help argument.
# If <xxx.help_EXTRACMDSBEFORE> or <xxx.help_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
#
# For flexibility in the order of printing, the following is implemented.
# The xxx.help target depends on xxx.helpmid, which depends on xxx.helpfirst .
# Thus the xxx.help target will execute commands in the following order:-
# - xxx.helpfirst (double colon target, prints out list of targets defined by <PKG_DEFINE_TARGETS>).
# - xxx.helpmid (double colon target).
# - xxx.help (single colon target, no commands defined by default).
# Packages can add dependencies on any of these to these help targets.
define PKG_DEFINE_TARGET_help
  $(PKG): $(PKG).help
  $(PKG).help: $(PKG).helpmid
  $(PKG).helpmid:: $(PKG).helpfirst
  $(PKG).helpfirst::
	$$($(PKG).help_EXTRACMDSBEFORE)
	@echo "Make-targets for $(PKG):"
	@for t in $(PKG).help $$($(PKG)_MAKETARGETS); do echo "$(MAKE) $$$$t"; done
	$$($(PKG).help_EXTRACMDS)
  $(PKG).help_EXTRACMDSBEFORE?=
  $(PKG).help_EXTRACMDS?=
endef

################################################################################
# Target: xxx.clean
# Cleans the build products for a package. It removes <xxx_BUILDDIR>.
#
# This is the target that normally comes before <xxx.untar>.
#
# This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the clean argument.
# If <xxx.clean_EXTRACMDSBEFORE> or <xxx.clean_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
define PKG_DEFINE_TARGET_clean
  ifndef $(PKG).clean_CMDS
    define $(PKG).clean_CMDS
	@echo "  CLEAN Package $(PKG)"
	$$($(PKG).clean_EXTRACMDSBEFORE)
	$(Q)rm -rf $($(PKG)_BUILDDIR) $$(dir $($(PKG)_BUILDDIR))$(PKG)
	$$($(PKG).clean_EXTRACMDS)
	$(Q)rm -f $(call TSFILE,$(PKG).*)
    endef
  endif
  $(PKG).clean_EXTRACMDSBEFORE?=
  $(PKG).clean_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).clean,)
endef

################################################################################
# Target: xxx.untar
# Untars the source of a package.
#
# This is the target that normally comes first when building a package from source.
#
# If <xxx_UNTARINSIDE> is non-empty, then <xxx_BUILDDIR> is first created,
# and untarring is done inside it,
# otherwise untarring itself should create directory <xxx_BUILDDIR>.
#
# Also creates a symlink called <PKG> to <PKG>-<xxx_VERSION>.
#
# Patching is not included! (See <xxx.patch>)
#
# This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the untar argument.
# If <xxx.untar_EXTRACMDSBEFORE> or <xxx.untar_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
#
# See:-
#
# <xxx.untar_POSTUNTARCMDS> for extra command to be run after untarring but before patching.
#
# <xxx_UNTARINSIDE>
#
# <xxx.patch>

# Define: xxx_UNTARINSIDE
# If set to true, then we perform the untarring inside <xxx_BUILDDIR>, rather than expecting
# the untarring to create <xxx_BUILDDIR>.

define PKG_DEFINE_TARGET_untar
  ifndef $(PKG).untar_CMDS
    define $(PKG).untar_CMDS
	@echo "  UNTAR Package $(PKG)"
	$$$($(PKG).untar_EXTRACMDSBEFORE)
	$(Q)if [ -z "$$($(PKG)_TARBALL)" ]; then \
	  echo "No tarball file found for package $(PKG)"; \
	  echo "Check TARBALL envvar"; exit 1; \
	fi
	$(Q)if [ -d $$($(PKG)_BUILDDIR) ]; then \
	  echo "Already exists: $$($(PKG)_BUILDDIR)"; echo "Make target '$(PKG).clean' (or 'clean') first."; \
	  exit 1; fi
	$(Q)mkdir -p $$(if $$($(PKG)_UNTARINSIDE),$$($(PKG)_BUILDDIR),$$(dir $$($(PKG)_BUILDDIR)))
	$(Q)cd $$(dir $$($(PKG)_BUILDDIR)) && rm -rf $(PKG) && ln -s $$(notdir $$($(PKG)_BUILDDIR)) $(PKG)
	$(Q)cd $$(if $$($(PKG)_UNTARINSIDE),$$($(PKG)_BUILDDIR),$$(dir $$($(PKG)_BUILDDIR))) && \
	  $$(PRJROOT)/bin/untarball $$($(PKG)_TARBALL)
	$$$($(PKG).untar_POSTUNTARCMDS)
	$$(if $$($(PKG)_UNTARINSIDE),,$(Q)if [ ! -d $$($(PKG)_BUILDDIR) ]; then echo 'Untar did not create $$($(PKG)_BUILDDIR) !'; exit 1; fi)
	$$($(PKG).untar_EXTRACMDS)
    endef
  endif
  $(PKG).untar_EXTRACMDSBEFORE?=
  $(PKG).untar_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).untar,)
endef

# Define: xxx.untar_POSTUNTARCMDS
# Optional definition of extra commands for the <xxx.untar> target.
# Run after untarring, but before applying patches.
#
# See:-
# <xxx.untar_EXTRACMDS>

# This is required to escape filenames that contain hashes and dollar
# signs when we produce the list of files to make sure the target
# depends on them. It's not ideal but it does seem to work. Without
# this we risk the list of dependent files being truncated by the
# first one that contains a hash (and unfortunately emacs temporary
# files do.)
PKG_ESCAPE_FOR_MAKE:=sed -e 's/\#/\\\\\#/g' -e 's/\$$/\$$\$$/g'

################################################################################
# Target: xxx.patch
# Applies patches and overlays for a package.
#
# This is the target that normally comes after <xxx.untar> and before <xxx.config>.
#
# Patching is done using the patchfiles script. A package can pass options to this
# using variable <xxx_PATCHOPTS>.
# To make this target a no-op, see <xxx_PATCH>.
#
# This target is dependent on file <xxx_SRCDIR>/Makefile.
#
# See:-
# <xxx_PATCHES> <xxx_PATCH> <xxx_OVERLAYDIR>.

define PKG_DEFINE_TARGET_patch
  ifndef $(PKG).patch_CMDS
    define $(PKG).patch_CMDS
	@echo "  PATCH/OVERLAY Package $(PKG)"
	$$(if $$(filter true,$$($(PKG)_PATCH)),$$($(PKG).patch_EXTRACMDSBEFORE))
	$(Q)if [ ! -d "$$($(PKG)_BUILDDIR)" ]; then echo 'Does not exist: $$($(PKG)_BUILDDIR) !'; exit 1; fi
	$$(if $$(filter true,$$($(PKG)_PATCH)),\
	  $$(if $$($(PKG)_OVERLAYDIR),\
	    $(Q)if [ -d $$($(PKG)_OVERLAYDIR) ]; then \
	      echo "Copying overlay tree ($$($(PKG)_OVERLAYDIR))"; \
	      rsync -arv --exclude='.svn' $$($(PKG)_OVERLAYDIR)/ $$($(PKG)_BUILDDIR); fi) \
	, @echo '**INFO: skipping patching/overlay of $(PKG) because $(PKG)_PATCH!=true')
	$$(if $$(filter true,$$($(PKG)_PATCH)),$$(if $$(strip $$($(PKG)_PATCHES)), $(Q)patchfiles $$(if $$(findstring undefined,$$(origin $(PKG)_PATCHOPTS)),,$$($(PKG)_PATCHOPTS)) $$($(PKG)_BUILDDIR) $$($(PKG)_PATCHES)))
	$$(if $$(filter true,$$($(PKG)_PATCH)),$$($(PKG).patch_EXTRACMDS))
    endef
  endif
  $(PKG).patch_EXTRACMDSBEFORE?=
  $(PKG).patch_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).patch, $(PKG).$(patch_PREDECESSOR))
  $(call OBJ_TARGET_OTHER_DEPS,$(PKG).patch, $($(PKG)_PATCHES))
  $(if $($(PKG)_OVERLAYDIR),$(call OBJ_TARGET_OTHER_DEPS,$(PKG).patch, $(shell find $($(PKG)_OVERLAYDIR) -type f|$(PKG_ESCAPE_FOR_MAKE))))
  $$(call OBJ_TARGET_OTHER_DEPS, $(PKG).patch, $($(PKG)_SRCDIR)/Makefile)
endef

# Define: xxx_PATCHOPTS
# Optional package variable with options to pass to patchfiles script:-
# - --multipatch: if passed, then a file can be patched by multiple patch files,
# otherwise a file can only be patched by a single patch file.

# Define: xxx_PATCHES
# List of absolute file names to patch files for package <xxx>,
# to be applied during the <xxx.untar> target.
#
# Setup by default by <PKG_SETUP_VARS>.
#
# See:-
# <nopatch> <PKGPATCH> <xxx_PATCH> <xxx_PATCHDIR> <xxx.patch>

# Define: xxx_PATCH
# Indicates if we should patch the package xxx (value is true) or not (value is false).
# The default is specified by the value of <PKGPATCH>.
#
# See:-
# <nopatch> <PKGPATCH> <xxx.patch>

################################################################################
# Target: xxx.config
# Configures the source package. This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the config argument.
#
# This is the target that normally comes after <xxx.patch> and before <xxx.make>.
#
# If <xxx.config_EXTRACMDSBEFORE> or <xxx.config_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
#
# <xxx_CONFIGOPTS> is passed to the configure invocation for this package.
# If you write your own xxx.config target implementation you must pass
# <xxx_CONFIGOPTS> to the configure command.
#
# This target is dependent on file <xxx_SRCDIR>/Makefile.
#
# See:-
# <xxx_CONFIGENV> <xxx_CONFIGOPTS> <xxx.config_EXTRACMDS>
define PKG_DEFINE_TARGET_config
  ifndef $(PKG).config_CMDS
    define $(PKG).config_CMDS
	@echo "  CONFIG Package $(PKG)"
	$$($(PKG).config_EXTRACMDSBEFORE)
	$(Q)cd $($(PKG)_BUILDDIR) && $$($(PKG)_CONFIGENV) sh ./configure $$($(PKG)_CONFIGOPTS)
	$$($(PKG).config_EXTRACMDS)
    endef
  endif
  $(PKG).config_EXTRACMDSBEFORE?=
  $(PKG).config_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).config, $(PKG).$(config_PREDECESSOR))
  $$(call OBJ_TARGET_OTHER_DEPS, $(PKG).config, $($(PKG)_SRCDIR)/Makefile)
endef

################################################################################
# Target: xxx.make
# Makes the source package. This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the make argument.
#
# This is the target that normally comes after <xxx.config> and before <xxx.install>.
#
# If <xxx.make_EXTRACMDSBEFORE> or <xxx.make_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
#
# <xxx_MAKEOPTS> is passed to the make invocation for this package.
# If you write your own xxx.make target implementation, you must pass
# <xxx_MAKEOPTS> to the make command.
#
# This target is dependent on file <xxx_SRCDIR>/Makefile.
#
# See:-
# <xxx_MAKEOPTS>
define PKG_DEFINE_TARGET_make
  ifndef $(PKG).make_CMDS
    define $(PKG).make_CMDS
	@echo "  MAKE Package $(PKG)"
	$$($(PKG).make_EXTRACMDSBEFORE)
	$(Q)cd $($(PKG)_BUILDDIR) && $$($(PKG)_MAKEENV) $(MAKECMD) $$($(PKG)_MAKEOPTS)
	$$($(PKG).make_EXTRACMDS)
    endef
  endif
  $(PKG).make_EXTRACMDSBEFORE?=
  $(PKG).make_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).make, $(PKG).$(make_PREDECESSOR))
  $$(call OBJ_TARGET_OTHER_DEPS, $(PKG).make, $($(PKG)_SRCDIR)/Makefile)
endef

################################################################################
# Target: xxx.install
# Installs the package that was built from source. This target can be defined by calling
# <PKG_DEFINE_TARGETS> with the install argument.
#
# This is the target that normally comes after <xxx.make>.
#
# If <xxx.install_EXTRACMDSBEFORE> or <xxx.install_EXTRACMDS> are defined before doing this,
# then they should contain additional command lines to be executed by this target.
#
# <xxx_INSTALLOPTS> is passed to the make install invocation for this package.
# If you write your own xxx.install target implementation you must pass
# <xxx_INSTALLOPTS> to the installure command.
#
# This target is dependent on file <xxx_SRCDIR>/Makefile.
#
# See:-
# <xxx_INSTALLOPTS> <xxx_DEFINSTALLOPTS>
define PKG_DEFINE_TARGET_install
  ifndef $(PKG).install_CMDS
    define $(PKG).install_CMDS
	@echo "  INSTALL Package $(PKG)"
	$$($(PKG).install_EXTRACMDSBEFORE)
	$(Q)cd $($(PKG)_BUILDDIR) && $$($(PKG)_INSTALLENV) $(MAKECMD) $(filter-out DESTDIR=%,$($(PKG)_INSTALLOPTS)) DESTDIR=$(STAGINGDIR) install
	$(Q)cd $($(PKG)_BUILDDIR) && $$($(PKG)_INSTALLENV) $(MAKECMD) $$($(PKG)_INSTALLOPTS) install-strip
	$$($(PKG).install_EXTRACMDS)
    endef
  endif
  $(PKG).install_EXTRACMDSBEFORE?=
  $(PKG).install_EXTRACMDS?=
  $$(call PKG_TARGET, $(PKG).install, $(PKG).$(install_PREDECESSOR))
  $$(call OBJ_TARGET_OTHER_DEPS, $(PKG).install, $($(PKG)_SRCDIR)/Makefile)
endef

################################################################################
# PKG_DEFINE_TARGETS
################################################################################

# Function: PKG_DEFINE_TARGETS
# Defines standard targets for the package.
#
# Argument 1:
# One or more target names from <PKGSTDTARGETS>.
#
# Example:
# $(call PKG_DEFINE_TARGETS,help clean untar)
PKG_DEFINE_TARGETS = $(foreach t,$(1),$(eval $(PKG_DEFINE_TARGET_$t)))

################################################################################
# PKG_BUILD_USING_CONFIG
# Usage: $(eval $(PKG_BUILD_USING_CONFIG))
################################################################################
# Define: PKG_BUILD_USING_CONFIG
# Calls <PKG_SETUP_VARS> and <PKG_DEFINE_TARGETS> on <PKGSTDTARGETS>.
#
# Usage:
# $(eval $(PKG_BUILD_USING_CONFIG))
define PKG_BUILD_USING_CONFIG
  $(eval $(PKG_SETUP_VARS))
  $(call PKG_DEFINE_TARGETS, $(PKGSTDTARGETS))
endef

################################################################################
# PKG_BUILD_USING_MAKE
# Usage: $(eval $(PKG_BUILD_USING_MAKE))
################################################################################
# Define: PKG_BUILD_USING_MAKE
# Calls <PKG_SETUP_VARS> and <PKG_DEFINE_TARGETS> on <PKGSTDTARGETS> excluding config.
#
# Usage:
# $(eval $(PKG_BUILD_USING_MAKE))
define PKG_BUILD_USING_MAKE
  $(eval $(PKG_SETUP_VARS))
  $(eval make_PREDECESSOR = $(config_PREDECESSOR))
  $(call PKG_DEFINE_TARGETS, $(filter-out config,$(PKGSTDTARGETS)))
  make_PREDECESSOR = $(make_STDPREDECESSOR)
endef

################################################################################
# PKG_END
################################################################################

# Define: PKG_END
# Indicates the end of a package definition. It:-
# - adds <xxx.clean> to the dependents for <pkg.clean> if <xxx.clean> has been defined,
# unless <xxx_SKIPBUILD> equals true.
# - if directory <xxx_BUILDDIR> does not exist for the <PKG>, then it
# deletes any timestamp files for <PKG> by calling <TSUNSET> with argument ${<PKG>}.*
#
# Usage: $(eval $(PKG_END))

define PKG_END
  $(if $(filter true,$($(PKG)_SKIPBUILD)),,$(if $(filter $(PKG).clean,$($(PKG)_MAKETARGETS)),pkg.clean: $(PKG).clean))
  $(if $(shell test -d '$($(PKG)_BUILDDIR)' && echo exists),,$(shell $(call TSUNSET,$(PKG).*)))
  $(eval $(PKG_UNDEF))
endef

################################################################################
# PKG_TARGET
################################################################################
# Function: PKG_TARGET
# Sets up a definition of a package target. The commands to build the target
# must have been specified as a variable <ttt_CMDS>. Arguments are:-
# - the target name <ttt>
# - predecessor targets p (may be empty).
# It declares target <ttt> to depend on the predecessor target if specified.
#
# Example:
# (start code)
# define gcc.config_CMDS
# 	...commands...
# endef
# $(call PKG_TARGET, gcc.config, gcc.patch)
# (end)
#
# See:- <OBJ_TARGETSETUP>
PKG_TARGET = $(eval $(call OBJ_TARGETSETUP,$(strip $(1)),$(strip $(2)),PKG))
