#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2007 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_control_cpu.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#  20 20070208   barnish     Added tmdlAudio
#  21 20070212   barnish     Added tmdlAudioIO
#  22 20070214   batelaan    Merge drv&plf into src directory.
#  22.1.1 20070301 batelaan  Set _TMTGTCOPYLIB=1 if doing a release build.
#  23 20070301   awest       Add wmt directory to SDE2_REGISTER_PROJECTS.
#  24 20070302   batelaan    Merge.
#  25 20070323   neill       Add exStbFb and exStbResolution
#  26 20070327   neill       Add exStbLED and exStbHwAcc
#  27 20070501   awest       Add phStbDFBVideoProviderASF and phStbDemuxSW
#  28 20070619   batelaan    Reworked structure for WinCE.
#  29 20070824   batelaan    Include optional customisation files.
#  30 20090327   batelaan    Include toolchains/<CONFIG_TOOLCHAIN>.mk

# File: platform_control_cpu.mk
# Makefile fragment which builds the control cpu in STB225, and the only cpu in STB220.
#
# Includes:-
# - <first.mk>
# - toolchain/<CONFIG_TOOLCHAIN>.mk
# - <platform_cpu.mk>
# - <linux/Makefile> (usually)
# - <rootfs/Makefile>
# - <platform_control_cpu_linux.mk> if <CONFIG_OS_LINUX> is set.
# - <platform_control_cpu_wince.mk> if <CONFIG_OS_WINCE> is set.
# - Optionally <platform_control_cpu_dev.mk> (may not exist).
# - Optionally $<DI_PRJROOT>/etc/design_in_control_cpu.mk (may not exist).
# - <last.mk>

include $(PRJROOT)/etc/first.mk
include $(call GET_TOOLCHAIN_SETUP_FILE, CONFIG_TOOLCHAIN)
include $(PRJROOT)/etc/platform_cpu.mk

MAKEFILE_TITLE := STB22x Control CPU build Makefile

ifneq "$(findstring _dsp_,$(_TMDIVERSITY))" ""
  $(error This Makefile should not be used for this environment.)
endif

# Only register relevant additional SDE2 project directories not mentioned
# the other platform_cpu*.mk files:
$(call SDE2_REGISTER_PROJECTS,\
  $(PRJROOT)/src/streamingSystem \
)

# If we are building the release we need to create binary releases of our components/apps.
ifeq "$(PRJRELEASEBUILD)" "true"
  # But don't polute a developer's workarea (when in 'development mode'),
  # i.e. when we run the release build on a developer's workarea.
  # See ../bin/stbmkrc
  ifneq "$(PRJBUILDRELEASEDEVMODE)" "true"
    _TMTGTCOPYLIB=1
    export _TMTGTCOPYLIB
  endif
endif

ifeq "$(CONFIG_OS_LINUX)" "y"
include $(PRJROOT)/etc/platform_control_cpu_linux.mk
endif

ifeq "$(CONFIG_OS_WINCE)" "y"
include $(PRJROOT)/etc/platform_control_cpu_wince.mk
endif

# Design-In project extensions:
ifdef DI_PRJROOT
  -include $(DI_PRJROOT)/etc/design_in_control_cpu.mk
endif

ifneq "$(PRJRELEASEBUILD)" "true"
  # Development extensions:
  -include $(PRJROOT)/etc/platform_control_cpu_dev.mk
endif

$(PLATFORM_CPU_END)

include $(PRJROOT)/etc/last.mk
