#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_cpu.mk
#
# Include makefile to build cpu subsystem
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060810   batelaan    Initial
#   2 20060810   batelaan    Crosscheck fix
#   3 20060823   batelaan    Simplify makefile.
#   4 20061103   batelaan    Include $(BUILDROOT)/.config
#   5 20061205   batelaan    Include $(BUILDROOT)/.prjconfig instead
#   6 20070131   batelaan    Include of $(BUILDROOT)/.prjconfig mv'ed to ../Makefile
#   7 20070214   batelaan    Merge drv&plf into src directory.
#   8 20070619   batelaan    Reworked structure for WinCE.
#   9 20070629   batelaan    Add env var checks from envvars_cpu.mk
#  10 20070810   batelaan    Register design-in related SDE2 project directories.
#  11 20070824   batelaan    Add cpiDrivers.
#  12 20071009   batelaan    Rename _TMTGTBUILDROOT directory to sde2.
#  13 20080814   batelaan    Add include of roster.mk
#  14 20081222   batelaan    Add src/open to SDE2_REGISTER_PROJECTS call

# File: platform_cpu.mk
# Include makefile to build cpu subsystem.
#
# Included from:-
# - <platform_control_cpu.mk>
# - <platform_dsp_cpu.mk>
#
# Includes:-
# - <roster.mk>

include $(PRJROOT)/etc/roster.mk

#-------------------------------------------------------------------------------
# ENVIRONMENT CHECK
#-------------------------------------------------------------------------------

# Variable: CPUBUILDROOT
# The directory in which all build results are stored.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,CPUBUILDROOT,This environment variable is the directory in which all build results for the current CPU are stored)

# Variable: _TMTGTBUILDROOT
# SDE2 build results directory (usual value
# $(<CPUBUILDROOT>)/sde2).
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,_TMTGTBUILDROOT,This environment variable is the SDE2 build results directory)

# Variable: QMORE_WORKAREA
# QMore tool workarea. Stores tmp files.
#
# See: <_TMQMOREWORKAREAS>

# Variable: _TMQMOREWORKAREAS
# QMore tool workareas. Each subdirectory is a <QMORE_WORKAREA> for a specific SDE2 <_TMDIVERSITY>.


# Variable: _QMORE_QAC_VIA
# QMore QAC tool configuration file(s).
$(call CHECK_VAR_DEFINED,_QMORE_QAC_VIA,Compiler personality file or files to use with QAC/Qmore)

#-------------------------------------------------------------------------------
# Initial definition and help
#-------------------------------------------------------------------------------

default: help
help.first:: help.platform_cpu

#-------------------------------------------------------------------------------
# ENVIRONMENT CHECK
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

$(call SDE2_REGISTER_PROJECTS,\
  $(PRJROOT)/src \
  $(PRJROOT)/src/open \
  $(PRJROOT)/src/cpiDrivers \
  $(if $(DI_PRJROOT),$(DI_PRJROOT)/src) \
  $(DI_PRJDIR1) \
  $(DI_PRJDIR2) \
)

################################################################################
# ALL
################################################################################

# Define: BUILD_ORDER_FIRST
# Defines the first set of goals to be built by target <all>.
#
# See also: <ALL_ORDER>
BUILD_ORDER_FIRST =

# Define: BUILD_ORDER_PKGS_EXCLUDE
# Defines the set of packages that should not be built, or is regarded as special
# by target <all>.
# However, some of these are still built by default if they are listed as dependencies
# of non-excluded packages.
#
# See also: <ALL_ORDER> <ALL_EXCLUDED>
BUILD_ORDER_PKGS_EXCLUDE =

# Macro: BUILD_ORDER_PKGS_ANY
# Evaluates to the alphabetically sorted set of packages to be built by target <all>.
#
# See also: <ALL_ORDER> <ALLBUILDPKGS>
BUILD_ORDER_PKGS_ANY = \
	$(sort $(filter-out $(BUILD_ORDER_PKGS_EXCLUDE), $(ALLBUILDPKGS)))

# Define: BUILD_ORDER_LAST
# Defines the last set of goals to be built by target <all>.
#
# See also: <ALL_ORDER>
BUILD_ORDER_LAST =

# Define: ALL_ORDER
# Defines the set of package goals to be built by target <all>:-
# - <BUILD_ORDER_FIRST>
# - <BUILD_ORDER_PKGS_ANY>
# - <BUILD_ORDER_LAST>
#
# See also: <all>
ALL_ORDER = \
	$(BUILD_ORDER_FIRST) \
	$(foreach p,$(BUILD_ORDER_PKGS_ANY),$(p).install) \
	$(BUILD_ORDER_LAST)

# Target: all
# Builds all packages, as specified by the goals <ALL_ORDER>,
# from source
#
# See also: <ALL_ORDER>

# Define: PLATFORM_CPU_END
# Code to be evaluated at the end of a platform cpu makefile.
# It is intended mainly for adding dependencies to targets
# that depend on all makefile having been read in.
#
# Add to it using this construct: PLATFORM_CPU_END += $(eval $(SOME_VARIABLE))
PLATFORM_CPU_END += $(eval $(PLATFORM_CPU_END1))
define PLATFORM_CPU_END1
  all: $(ALL_ORDER)
  help::
	@printf '\nType make help.util for additional features.\n'
endef # PLATFORM_CPU_END1

.PHONY: all

################################################################################
# CLEAN
################################################################################

clean:: pkg.clean
.PHONY: clean

clean:: sde2.clean

sde2.clean:
	@echo '  CLEAN $$_TMTGTBUILDROOT'
	$(Q)rm -rf $(_TMTGTBUILDROOT)
.PHONY: sde2.clean

################################################################################
# HELP
################################################################################

# Variable: MAKEFILE_TITLE
# Should be defined by the makefile that includes this file.
# It is then printed out by the <help> target.
MAKEFILE_TITLE ?= $(firstword $(MAKEFILE_LIST))
help.platform_cpu::
	@printf '\n      ---- $(MAKEFILE_TITLE) ----\n'
	@echo
	@echo "all              - Build all"
	@echo "all.order        - Prints the build order of target 'all'"
	@echo "all.excluded     - Prints the packages that are by default excluded from building by target 'all'"
	@echo 'clean            - Remove all build products'
	@echo
.PHONY: help.platform_cpu

# Target: all.order
# Prints the names of packages in order of building by target <all>.
#
# See also: <ALL_ORDER> <all.excluded>
all.order:
	@echo '$(ALL_ORDER)' | $(LIST2INDENTEDLINES)
.PHONY: all.order

# Define: ALL_EXCLUDED
# The names of packages that are by default excluded from building by target <all>.
# This is different from <BUILD_ORDER_PKGS_EXCLUDE> since it takes into account package dependencies.
#
# See also: <all.excluded> <all.order> <ALL_ORDER> <ALLPKGS>
ALL_EXCLUDED=$(filter-out $(shell echo '$(foreach t,$(ALL_ORDER),$(call OBJ_TGT_EXPAND_DEPENDENCIES,$(t)))' | sed -e 's/\.[^ ]*//g'),$(ALLPKGS))

# Target: all.excluded
# Prints the names of packages that are by default excluded from building by target <all>.
#
# See also: <ALL_EXCLUDED> <all.order>
all.excluded:
	@echo '$(sort $(ALL_EXCLUDED))' | $(LIST2INDENTEDLINES)
.PHONY: all.excluded
