#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: platforms.mk
#
#
# Rev Date        Author      Comments
#-------------------------------------------------------------------------------
#  30 20071023    batelaan    Allow toolchain to be overridden from the environment.
#  31 20071023    batelaan    Change way of handling _TMDIVERSITY.
#  32 20080201    laird       Change GCC_PREFIX for gnu toolchain.
#  33 20080305    batelaan    Removed stb220 and linux_VARIANT
#  34 20080306    batelaan    Moved toolchains to toolchains subdir.
#  35 20080314    laird       KSRC now set elsewhere.
#  36 20080829    batelaan    Redirect docs targets to control sub-make
#  37 20081105    batelaan    Remove unused HOST_SIMULATION var, and pnx8330/1 configs.
#  38 20090112    batelaan    Use <config>_ALIAS in help message.
#  39 20090309    batelaan    Add .DEFAULT catch-all target to hint about make <cpu> sub=xxx
#  40 20090327    batelaan    Derive TARGET envvar from GCC_PREFIX.

# File: platforms.mk
# Makefile for building STB22x platforms.
#
# Includes:
# - <util.mk>
# - <envvars.mk>
# - <envs.mk>
# - <subgoals.mk>
# - <platform_stb225.mk> if <CONFIG_SWPLF_STB225> is y.

#-------------------------------------------------------------------------------
default: help
help:: help.thisfile
all: platforms
clean:: all.clean

help.thisfile:
	@echo 'NXP STB22x Platform Build System'
	@echo ''
	@echo 'The following platform(s) can be built: $(platforms)'
	@echo 'Use "$(MAKE) all" to build everything.'

#-------------------------------------------------------------------------------
# .DEFAULT target
#-------------------------------------------------------------------------------

.DEFAULT:
	@echo "make[$(shell dc -e '1 $(MAKELEVEL) + p')]: *** No rule to make target \`"'$@'"'.  Stop."
	@echo 'Perhaps the target you want is available at CPU build level, using:'
	@echo "  prjmake <cpu_config> sub='$(if $(MAKEFLAGS),$(MAKEFLAGS) )"'$@'"'"
	@echo 'E.G.'
	@echo "  prjmake $(call ALIASED_NAME,$(firstword $(configs))) sub='"'$@'"'"
	@exit 1

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Variable: MAKEOVERRIDES.orig
# Contains the original value of the make variable <MAKEOVERRIDES>.
MAKEOVERRIDES.orig:=$(MAKEOVERRIDES)

# Variable: MAKEOVERRIDES
# Contains the make command line variable definitions. We reset it here to empty
# because by default we don't want to pass anything down to sub-make invocations.
MAKEOVERRIDES=

# Contains the original value of the make variable <MAKEFLAGS>.
MAKEFLAGS.orig:=$(MAKEFLAGS)

# Variable: MAKEFLAGS
# Contains the make command line flags and variable definitions. We reset it here to empty
# because by default we don't want to pass anything down to sub-make invocations.
MAKEFLAGS=

include $(PRJROOT)/etc/util.mk
include $(PRJROOT)/etc/envvars.mk
include $(PRJROOT)/etc/envs.mk
include $(PRJROOT)/etc/subgoals.mk

ifneq "$(MAKECMDGOALS)" "envs.list"
  ILLEGAL_USE_MSG = ARCH/TARGET/CROSS_COMPILE/CPUBUILDROOT/KSRC/ROOTFS/STAGINGDIR/_TMBSL should be undefined when using etc/platforms.mk.
  $(call CHECK_VAR_UNDEFINED,ARCH,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,TARGET,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,CROSS_COMPILE,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,CPUBUILDROOT,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,KSRC,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,ROOTFS,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,STAGINGDIR,$(ILLEGAL_USE_MSG))
  $(call CHECK_VAR_UNDEFINED,_TMBSL,$(ILLEGAL_USE_MSG))
endif

#-------------------------------------------------------------------------------
# exports
#-------------------------------------------------------------------------------

export ARCH
export CPUBUILDROOT
export CYGWINROOT
export IC_EMULATION
export KSRC
export ROOTFS
export STAGINGDIR
export TARGET
export _TMBSL
export _TMDIVERSITY
export _TMLINKTYPE
export _TMREPORTS
export _TMTGTBUILDROOT
export _TMTGTCPUCLASS
export _TMTGTCPUTYPE
export _TMTGTENDIAN
export _TMTGTOS
export _TMTGTOSCLASS
export _TMTGTPREFIX
export _TMTOOLCHAIN
export UNAME
export _TMQMOREWORKAREAS
export QMORE_WORKAREA
export _QMORE_QAC_VIA

#-------------------------------------------------------------------------------
# environments
#-------------------------------------------------------------------------------

# NOTE!!! Regarding defining *.env macros:
# NOTE!!! Use double $$ to delay evaluation of variables.
# NOTE!!! This will allow customising variables using later-defined variables.

define stb22x.env
  $(call ENVS_ENTER,stb22x.env)
  $(call setenvVar,STAGINGDIR,$$(CPUBUILDROOT)/staging)
  $(call setenvVar,ROOTFS,$$(CPUBUILDROOT)/rootfs)
  $(call setenvVar,_TMTGTPREFIX,/opt/nxp)
  $(sde2.common.env)
  $(call ENVS_LEAVE,stb22x.env)
endef

ifndef TMP
  export TMP:=/tmp
endif

ifndef TEMP
  export TEMP:=/tmp
endif

# Value of UNAME depends on the OS we are running the build on.
UNAME_output := $(shell uname)
ifeq "$(findstring Linux,$(UNAME_output))" "Linux"
  UNAME:=linux
else
  ifeq "$(findstring CYGWIN,$(UNAME_output))" "CYGWIN"
    UNAME:=cygwin
  else
    $(error Cannot determine UNAME - uname output did not contain Linux or CYGWIN)
  endif
endif

define sde2.common.env
  $(call ENVS_ENTER,sde2.common.env)
  $(call setenvVar,_TMTGTBUILDROOT,$$(CPUBUILDROOT)/sde2)
  $(call setenvVar,_TMREPORTS,$$(_TMTGTBUILDROOT))
  $(call setenvVar,_TMLINKTYPE,dynamic)
  $(call _TMDIVERSITY.addValue,$$(EXTRA_TMDIVERSITY))
  $(if $(filter 1 true,$(PRJMOREUSECHECK)),
    $(call setenvVar,_TMQMOREWORKAREAS,$$(BUILDROOT)/qmore_workareas),
    $(call setenvVar,QMORE_WORKAREA,$$(CPUBUILDROOT)/qmore_workarea))
  $(call setenvVar,_QMORE_QAC_VIA,$(PRJROOT)/etc/qac/$$(CROSS_COMPILE)gcc.p_c)
  $(call ENVS_LEAVE,sde2.common.env)
endef

# Ensure that _TMDIVERSITY is reset. We will always call
# the _TMDIVERSITY.addValue macro to set it.
# This means the order of adding values to it is irrelevant.
_TMDIVERSITY:=

define sde2.os.linux.env
  $(call ENVS_ENTER,sde2.os.linux.env)
  $(call setenvVar,_TMTGTOS,linux)
  $(call setenvVar,_TMTGTOSCLASS,linux)
  $(call setenvVar,_TMTOOLCHAIN,gnu)
  $(call ENVS_LEAVE,sde2.os.linux.env)
endef

# We set CYGWINROOT to empty so that SDE2 will define ECHO to echo,
# (see ../sde2/project/sites/soustb/cygwin.mk)
# instead of to <somepath>echo. This means we will use the search path
# for the Unix command that SDE2 uses. That means again that
# we are not restricted to using Cygwin, but can use a different set
# of Unix utilities under Windows.
define sde2.os.wince.env
  $(call ENVS_ENTER,sde2.os.wince.env)
  $(call setenvVar,_TMTGTOS,ce600)
  $(call setenvVar,_TMTGTOSCLASS,ce)
  $(call setenvVar,_TMTOOLCHAIN,)
  $(call setenvVar,CYGWINROOT,)
  $(call ENVS_LEAVE,sde2.os.wince.env)
endef

define cpu/mips.env
  $(call ENVS_ENTER,cpu/mips.env)
  $(call setenvVar,ARCH,mips)
  $(call setenvVar,_TMTGTCPUTYPE,4KEc)
  $(call setenvVar,_TMTGTCPUCLASS,mips)
  $(call setenvVar,TARGET,$$(GCC_PREFIX))
  $(call ENVS_LEAVE,cpu/mips.env)
endef

define cpu/pnx833x.env
  $(call ENVS_ENTER,cpu/pnx833x.env)
  $(cpu/mips.env)
  $(toolchain/$(toolchain).env)
  $(call _TMDIVERSITY.addValue,pnx833x)
  $(call ENVS_LEAVE,cpu/pnx833x.env)
endef

define cpu/x86-pc.env
  $(call ENVS_ENTER,cpu/x86-pc.env)
  $(call setenvVar,ARCH,i386)
  $(call setenvVar,_TMTGTCPUTYPE,i486)
  $(call setenvVar,_TMTGTCPUCLASS,x86)
  $(call setenvVar,_TMTGTENDIAN,el)
  $(call setenvVar,TARGET,i386-linux-gnu)
  $(call setenvVar,KSRC,/usr/src/linux)
  $(call prependenvVar,PATH,$$(ROOTFS)$$(_TMTGTPREFIX)/bin:$$(ROOTFS)/usr/local/bin:)
  $(toolchain/x86.native_gcc.env)
  $(call ENVS_LEAVE,cpu/x86-pc.env)
endef

define toolchain/x86.native_gcc.env
  $(call ENVS_ENTER,toolchain/x86.native_gcc.env)
  $(call setenvVar,GCC_BASE,/)
  $(call setenvVar,CROSS_COMPILE,)
  $(call setenvVar,CROSS_COMPILE_BASE,$$(GCC_BASE))
  $(call setenvVar,CROSS_COMPILE_ROOTFS,/)
  $(call setenvVar,CROSS_COMPILE_LIBS,$$(CROSS_COMPILE_BASE)/target/usr/lib)
  $(call setenvVar,GCC_PREFIX,)
  $(call setenvVar,CROSS_COMPILE_PREFIX,$$(GCC_PREFIX))
  #$(call setenvVar,CROSS_GCC,)
  $(call ENVS_LEAVE,toolchain/x86.native_gcc.env)
endef

#-------------------------------------------------------------------------------
# platforms
#-------------------------------------------------------------------------------

ifeq "$(CONFIG_SWPLF_STB225)" "y"
  include etc/platform_stb225.mk
else
  $(error Unsupported value ($(CONFIG_SWPLF_STB225)) for CONFIG_SWPLF_STB225 - should be y)
endif

platforms: $(platforms)

$(foreach plf,$(platforms),\
  $(eval $(plf): platform=$(plf)) \
)

#-------------------------------------------------------------------------------
# Platform Sub Goals
#-------------------------------------------------------------------------------

help:: help.subGoals
help.subGoals::
	@printf '\nThe following standard subgoals exist for each platform target:\n'
$(call defineSubGoal,all,Builds everything, including install)
$(call defineSubGoal,clean,Removes all build results)
$(call defineSubGoal,help,Prints more help)
help.subGoals::
	@printf '  %-18s %s\n' '<target>.shell' 'Creates a sub-shell with the correct environment for building <target>'

help.subGoals::
	@printf 'Alternatively, use: sub=<subGoal>, for example:\n  $(MAKE) $(call ALIASED_NAME,$(firstword $(configs))) sub=linux.menuconfig\n'

#-------------------------------------------------------------------------------
# docs Goals
#-------------------------------------------------------------------------------

# Redirect docs goals to sub-make for the control processor.
.PHONY: docs
$(foreach t,$(filter docs docs.%,$(MAKECMDGOALS)),\
  $(eval .PHONY: $t) \
  $(eval $t:; $(MAKECMD) control sub=$t) \
)

#-------------------------------------------------------------------------------
# configs

# Variable: configs
# List of possible configurations.

configs_count = $(words $(configs))

ifeq "$(configs_count)" "1"
  config.printenv: $(configs:%=%.printenv)
else
  config.printenv:
	@echo "config.printenv: can only execute if configs contains a single entry"; exit 1
endif

# Macro: ALIASED_NAME
# Return aliased version of $1, or $1 itself
# if no alias is defined for $1.
# An alias is defined as a variable with name "$(1)_ALIAS".
# Usage: $(call ALIASED_NAME,configName)
ALIASED_NAME=$(firstword $($(strip $1)_ALIAS) $1)

#-------------------------------------------------------------------------------

help::
	@echo
	@printf '%-23s - %s\n' 'docs[.*]' 'Invoke the docs[.*] target for the control CPU'
	@printf '%-23s - %s\n' 'help.util' 'Print help on for additional features'

