#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: etc/roster.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20070717   batelaan    Initial
#   2 20070719   neill       Add phStbCc
#   3 20070725   neill       Move CC encoding into phStbVo
#3.1.1 20070801  awest       remove phStbAsf
#3.1.2 20070806  batelaan    Rename tmosal to phStbOsal and
#                            ph833xCore to phStbCore
#3.1.3 20070809  batelaan    WinCE update of modules.
#   5 20070917   batelaan    Remove incorrect entries.
#   6 20070921   batelaan    phStbDemuxSW is not a kernel module
#   7 20071003   batelaan    WinCE enhancements
#   8 20071030   Shane Ogilvie Add AFTE
#   9 20071031   sjones      Merge parallel versions
#  10 20071121   burningh    Remove SbmDev
#  11 20071212   hutchinson  Merge in EB's changes from 071128 & 071212
#11.1.1 20080105 hutch       Removed components not needed by EB.
#11.1.2 20080222 hutch       Added new components phStbAv and phStbMspSource
#                            added phStbVo
#11.1.3 20080314 burningh    Remove ph833xVidUseCase
#11.1.4 20080314 laird       I have removed tmPerf now
#11.1.5 20080318 neill       Conditionally include SCART and HDMI drivers
#  13 20080528   batelaan    Move build control into this file.
#15.1 20081020   cgrosse     add cx24120
#15.2 20081126   wguo        add tdhl5_sam
#  16 20080708   batelaan    Add uda3000, phStbCommonInterface.
#                            Now optional: saa1064, tda9983, phStbRCProtocol
#  17 20080707   batelaan    Build kernel modules as external modules.
#  18 20080715   batelaan    All as loadable modules on MIPS1
#  19 20080710   hnguyen     Add tda9984
#  20 20080814   batelaan    Update BUILD_INFO_* vars; rm uda3000
#  21 20081106   estarkie    Add phStbHdmi
#  22 20081203   laird       Merged from 'roster.mk-21:makefile:SOUsb225#1' and 'roster.mk-CFR_STB#15.2:makefile:SOUsb225#1' by laird.
#  23 20081210   batelaan    Merge in tdhl5_sam
#                            Always build phStbCommonInterface.
#  23 20081204   batelaan    Merge in cx24116p
#  25 20090106   batelaan    Test $PRJPROPRIETARY to select kmodules to build. rm wince.
# c25 20090113   cgrosse     Import phStbSmc stuff
#  26 20090115   batelaan    Merge c25 (add phStbSmc)
#  27 20090121   batelaan    uda3000 and phStbIpc are GPL
#  28 20090320   batelaan    Add optional include of $DI_PRJROOT/etc/design_in_roster.mk
#  29 20090401   batelaan    Use delayed eval for KMODULE_*NAMES vars so design_in_roster.mk works
#  xx 20091028   amccurdy    Make _phStbSmc_KCONFIG conditional on CONFIG_SMART_CARD_PRESENT
#  xx 20091202   amccurdy    Make phStbCommonInterface conditional on CONFIG_COMMON_INTERFACE_PRESENT
#  xx 20091203   amccurdy    Remove phPnx833xAfte
#  xx 20100209   amccurdy    Make _phStbHdmi_KCONFIG conditional on CONFIG_PHSTBHDMI_WRAPPER
#  xx 20100216   amccurdy    Make _at24c64_KCONFIG conditional on CONFIG_EEPROM_PRESENT

# File: roster.mk
# Contains kernel module names used in the platform software.
# Currently only supports kernel modules compiled using SDE2.
#
# Includes:
# - ${<BUILDROOT>}/.config
# - ${<PRJROOT>}/etc/proprietary.mk (optionally)
# - ${<DI_PRJROOT>}/etc/<design_in_roster.mk> (optionally)

# Macro: KMODULE_NAMES
# List of kernel module (driver) names.
#
# The variable <PRJPROPRIETARY> is checked, and if set then the list includes
# proprietary kernel modules (see <KMODULE_NAMES_PROPRIETARY>.
# Otherwise only open source modules are included (see <KMODULE_NAMES_OPEN_SOURCE>).
#
# The order of the names is significant:
# - it determines compilation order.
# - For Linux it determines initialisation order for any drivers that are built-ins!
#
# See also: <design_in_roster.mk>

# Macro: KMODULE_NAMES_PROPRIETARY
# List of proprietary non-open source kernel module (driver) names.
# Included in <KMODULE_NAMES> if variable <PRJPROPRIETARY> is set.

# Macro: KMODULE_NAMES_OPEN_SOURCE
# List of open source kernel module (driver) names.
# Included in <KMODULE_NAMES>.

include $(BUILDROOT)/.config
-include $(PRJROOT)/etc/proprietary.mk

# Map SDE2 kernel driver component names to KCONFIG values used by
# ../sde2/sde/linux/kmodule.mk ...

# Define: _mmm_KCONFIG
# Indicates build method for a kernel module <mmm>:
# - if "y": built-in.
# - if "m": loadable module.
# - if "n" or empty or undefined: not built.

ifeq "$(findstring _dsp_,$(_TMDIVERSITY))" "_dsp_"

  # AUDIO MIPS (MIPS2).
  # Only a very few kernel modules are used for MIPS2, and they are all built-in:
  _phStbDbg_KCONFIG           = y
  _phStbNetDev_KCONFIG        = y
  _phStbOsal_KCONFIG          = y

else # ifeq "$(findstring _dsp_,$(_TMDIVERSITY))" "_dsp_"

  # MAIN MIPS.
  # These kernel modules always need to be built, because they are
  # - always needed and/OR
  # - not distributed as source code.
  #   TODO: build but don't install the module if prjconfig does not enable the module.
  # Note: whether insmod is performed may depend on a prjconfig option!
  # See ../src/rootfs/etc/init.d/*.sh scripts.
  # But on BrightSign we don't want to build stuff we don't care about so we've made them option dependent.
  _cx24116p_KCONFIG             = $(if $(CONFIG_CX24116P),m)
  _cx24120b_KCONFIG             = $(if $(CONFIG_CX24120B),m)
  _phKernelIfce_KCONFIG         = m
  _phStbBootDsp_KCONFIG         = m
  _phStbCore_KCONFIG            = m
  _phStbDbg_KCONFIG             = m
  _phStbDemux833x_KCONFIG       = m
  _phStbDraw_KCONFIG            = m
  _phStbFB_KCONFIG              = m
  _phStbGpio_KCONFIG            = m
  _crash-kernel_KCONFIG         = m
  _phStbIpc_KCONFIG             = m
  _phStbKBufMgr_KCONFIG         = m
  _phStbNetDev_KCONFIG          = m
  _phStbOsal_KCONFIG            = m
  _phStbRCProtocol_KCONFIG      = $(if $(CONFIG_RC_PROTOCOL),m)
  _phStbRSServer_KCONFIG        = m
  _phStbVideoRenderer_KCONFIG   = m
  _phStbVo_KCONFIG              = m
  _saa1064_KCONFIG              = $(if $(CONFIG_SAA1064),m)

  # Only build the HDMI drivers we actually need (BrightSign change)
  _phStbHdmi_KCONFIG            = $(if $(CONFIG_PHSTBHDMI_WRAPPER),m)
  _tda9983_KCONFIG              = $(if $(CONFIG_HDMI_TDA9981_2_3),m)
  _tda9984_KCONFIG              = $(if $(CONFIG_HDMI_TDA9984),m)
  _hdmi84_KCONFIG               = $(if $(CONFIG_HDMI_NEW_TDA9984),m)
  _hdmi84cec_KCONFIG		= m


  # Configuration-dependent.
  # Note that any component mentioned here should either be:
  # - distributed as source code (see ci_release_stb225.spec file)
  # - a frozen prjconfig option.
  # If this is not the case, then a customer could receive a release
  # based on a configuration file where the option is disabled, and the
  # component thus not built, but s/he can enable it, but not rebuild it
  # because s/he did not get the source code!

  # Source code:
  _ak470x_KCONFIG             = $(if $(CONFIG_SCART_AK470X),m)
  _at24c64_KCONFIG            = $(if $(CONFIG_EEPROM_PRESENT),m)
  _uda3000_KCONFIG            = $(if $(CONFIG_SCART_UDA3000),m)
  # Frozen:
  _phStbCommonInterface_KCONFIG = $(if $(CONFIG_COMMON_INTERFACE_PRESENT),m)
  # Source code:
  _phStbAlsa_KCONFIG          = $(if $(CONFIG_ALSA_MODULES),m)
  # Source code:
  _phStbIR_KCONFIG            = $(if $(CONFIG_INFRA_RED),m)
  # Source code:
  _phStbKey_KCONFIG           = $(if $(CONFIG_FRONT_PANEL_KEYBOARD),m)
  # Source code:
  _phStbLCD_KCONFIG           = $(if $(CONFIG_FRONT_PANEL_LCD),m)
  # Source code:
  _phStbMmio_KCONFIG          = $(if $(CONFIG_DEV_MMIO),m)
  # Frozen:
  _phStbWmdrmPd_KCONFIG       = $(if $(CONFIG_WMDRMPD),m)
  # Frozen:
  _phStbSmc_KCONFIG           = $(if $(CONFIG_SMART_CARD_PRESENT),m)

endif # ifeq "$(findstring _dsp_,$(_TMDIVERSITY))" "_dsp_"

KMODULE_NAMES_OPEN_SOURCE = \
    phKernelIfce \
    phStbAlsa \
    phStbBootDsp \
    phStbDbg \
    phStbFB \
    phStbHdmi \
    hdmi84 \
    hdmi84cec \
    phStbIpc \
    phStbIR \
    phStbKBufMgr \
    phStbKey \
    phStbLCD \
    phStbMmio \
    phStbNetDev \
    phStbOsal \
    tdhl5_sam \
    uda3000

KMODULE_NAMES_PROPRIETARY = \
    ak470x \
    at24c64 \
    cx24116p \
    cx24120b \
    phStbCommonInterface \
    phStbCore \
    phStbDemux833x \
    phStbDraw \
    phStbGpio \
    crash-kernel \
    phStbRCProtocol \
    phStbRSServer \
    phStbSmc \
    phStbVideoRenderer \
    phStbVo \
    phStbWmdrmPd \
    saa1064 \
    tda9983 \
    tda9984

KMODULE_NAMES = \
    $(KMODULE_NAMES_OPEN_SOURCE) \
    $(if $(PRJPROPRIETARY),$(KMODULE_NAMES_PROPRIETARY))

# To allow design-in projects adding SDE2 kernel modules to be built using
# the standard linux.make (and linux.modules.make etc) target:
ifdef DI_PRJROOT
  -include $(DI_PRJROOT)/etc/design_in_roster.mk
endif

# Macro: KMODULE_COMP_NAMES
# Top-level SDE2 component names for each of the <KMODULE_NAMES> values.
# Synonym for <KMODULE_NAMES>.
KMODULE_COMP_NAMES = $(KMODULE_NAMES)

BUILD_INFO_INPUT_KMODULES += $(KMODULE_COMP_NAMES)
