# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
# This source code and any compilation or derivative thereof is the
# proprietary information of NXP B.V. and is confidential in nature.
#
# @file  setup_productversion.mk
# @brief Define version of product
#
# Rev Date       Author     Comments
#-------------------------------------------------------------------------------
#   1 20070306   rtaylor    Original
#   2 20070323   rtaylor    Change MICROBE_BUILD_ID to PRODUCTVERSION
#   3 20070925   batelaan   Set default value to dev
#   4 20080624    batelaan    Add documentation.

# File: setup_productversion.mk
# Setup the product version (<PRODUCTVERSION>) ready for display at boot time.

# Define: PRODUCTVERSION
# The product version string. Used in various places
# to identify the version of the product that is or has been built,
# e.g. at boot time.
#
# The PRODUCTVERSION line is replaced with the appropriate product version
# during the release build.
#
# - Do not use spaces in the value as spaces are removed by the build system.
# - Do not use quotes around the value as these will appear in version number.
# - If the value is "dev", then it is an uncontrolled (non-baselined) version.
#PRODUCTVERSION=pr15_sjo_r6106_b1

ifeq ($(wildcard $(BUILDROOT)/generated_setup_productversion.mk),)
# We don't have a version file. It must be a custom source release.
PRODUCTVERSION=custom
else
# We get the product version from a build-specific directory so that
# it can change between configurations without forcing a rebuild.
include $(BUILDROOT)/generated_setup_productversion.mk
endif

# Sanity checks:
ifeq "$(PRODUCTVERSION)" ""
  $(error PRODUCTVERSION must not be empty)
endif
ifneq "$(strip $(PRODUCTVERSION))" "$(PRODUCTVERSION)"
  $(error PRODUCTVERSION must not have surrounding spaces)
endif
