#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2006 Koninklijke Philips Electronics N.V.,
# All Rights Reserved.
#
# Filename: subgoals.mk
#
# Utility makefile for subgoals.
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060728   batelaan    Initial

# File: subgoals.mk
# Utility makefile for subgoals. Subgoals are goals named {goal}.{subgoal}, which
# then depend on {goal}, but with the sub variable defined to {subgoal}.

#-------------------------------------------------------------------------------
# Macro: defineSubGoal
# Defines a subgoal name ($1), with help string $2.
#
# Subgoals are goals named {goal}.{subgoal}. These are declared to
# depend on {goal}, with the sub variable defined to {subgoal}.
# Usually the {goal} target is defined to run a sub-make.
#
# $2 is used to add a (double-colon) help.subGoals target which prints
# a line as follows:-
# (start code)
#      <target>.$1    $2
# (end)
# Example:-
# (start code)
# $(call defineSubGoal,clean,Removes all build results)
# $(call defineSubGoal,all,Builds everything, including install)
# $(call defineSubGoal,install,Install part of build)
# $(call defineSubGoal,help,Prints more help)
# stb220:
#	$(MAKE) -$(MAKEFLAGS) -f stb220.mk $(call getSubGoals,all)
# (end)
defineSubGoal=$(eval $(call defineSubGoal1,$(strip $(1)),$(2)))
define defineSubGoal1
  %.$1: %;
  %.$1: sub=$1
  help.subGoals::
	@printf '  %-18s %s\n' '<target>.$1' '$2'
endef

# Function: getSubGoals
# If variable 'sub' is defined it returns $(sub), otherwise it returns its second argument.
# Argument 1 is by convention the name of the current goal ($@), but can be anything else.
#
# Example:-
# $(MAKE) -$(MAKEFLAGS) -f stb220.mk $(call getSubGoals,$@,all)
getSubGoals=$(if $(sub),$(sub),$(2))

testSubGoal:
	@echo 'Target $@ - sub = "$(sub)"'
