#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2007 NXP B.V.,
# All Rights Reserved.
#
# Filename: userrc.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20070220   batelaan    Initial

# File: userrc.mk
# General make utility macros and functions for running user-specific command
# during a build.

# Variable: USERRC
# Path to default program/script to be run by <%.userrc> target.
#
# Default: $(HOME)/.prjmake.userrc
USERRC?=$(HOME)/.prjmake.userrc

# Function: USERRCCMD
# Function macro which expands to the command executed by <%.userrc> and <userrc>. 
# Takes one argument, which is passed to the ${<USERRC>} program/script.
#
# Example: $(call USERRCCMD,rootfs.copy)
define USERRCCMD
  $(Q)if [ -x "$(USERRC)" ]; then "$(USERRC)" --makecmdgoals='$(MAKECMDGOALS)' '$(strip $1)'; fi
endef

# Target: %.userrc
# Invokes ${<USERRC>} program/script if it exists and is executable.
# It passes as arguments:-
# - --makecmdgoals='${<MAKECMDGOALS>}'
# - Value of % ($*)
#
# Examples:-
# (start code)
#   some-make-goal: ... some-make-goal.userrc
# (end code)
# This will ensure that when some-make-goal is executed, that
# the <USERRC> program/script is run with argument 'some-make-goal'.
#
# (start code)
#   some-make-goal: 
#           ...
#           $(call USERRCCMD,some-make-goal)
# (end code)
# Similar effect as above, except _when_ it is executed.
#
# See: <USERRCCMD>
%.userrc:
	$(call USERRCCMD,$*)

# Target: userrc
# Alias for target 'default.userrc'. Invokes <USERRCCMD> with argument 'default'.
#
# See: <%.userrc>
userrc: default.userrc

################################################################################

help.userrc::
	@printf 'userrc\t\t\t- Runs $$USERRC with argument default\n'
	@printf '%%.userrc\t\t- Runs $$USERRC with argument <%%>\n'
	@printf 'Variable USERRC\t\t- Default: $(USERRC)\n'
