#-----------------------------------------------------------------------
# Copyright (c) 2007, NXP Semiconductors 
#
# sde/common.mk
#
# Description: Defines dependency check rules
#              Defines rules for generating objects
#              Defines rules for making option files
#
# Initial Author             : Rogier Wester
# Maintaince and development : STA/SDM/SDE2 Team 
#
# changes to this file force rebuild
#-----------------------------------------------------------------------
ifndef _SDE_INCLUDE_SDE_COMMON_MK_
_SDE_DEP_MAKEFILES +=                                                         \
                makefile                                                      \
                $(DIR_SDE)/common.mk

.PHONY: FORCE

#----------------------------------------------------------------------
# This makefile is always defined. The reason for '-' sign is not to 
# print error message in the beginning

-include                $(LOC_LIST_MKFILENAME)
# _SDE_DEP_MAKEFILES +=   $(LOC_LIST_MKFILENAME)

# To enable project wide settings
-include $(_SDE_PROJECT)/project_include0.mk

configuration:: makelist
ifeq ($(_SDE_BUILD_TARGET),LIB)
ifeq ($(SDE_IN_SDE),.)

ifeq ($(_$(notdir $(DIR_LOCAL))_DIR),)
	@$(ECHO) 'SDE2 can not find definition of _$(notdir $(DIR_LOCAL))_DIR. Update your prjlist.txt or loc_list.* files !!!'
else
ifeq ($(wildcard $(_$(notdir $(DIR_LOCAL))_DIR)),)
$(LOC_LIST_FILENAME)::
	@$(ECHO) Updating loc_list.txt, because directory  $(_$(notdir $(DIR_LOCAL))_DIR) was not found ...; \
	$(RM) $@; \
	$(MKDIR) $(_SDE_TMTGTBUILDROOT)/project ;\
	for comp in $(_SDE_ALL_COMPS_DIR) ; \
	do \
	if [ -d $$comp ] ; then \
	$(ECHO) "`basename $$comp` $$comp">> $@; \
	fi ;\
	done

$(LOC_LIST_MKFILENAME)::
	@$(ECHO) Updating loc_list.mk, because directory  $(_$(notdir $(DIR_LOCAL))_DIR) was not found ... ; \
	$(MKDIR) $(_SDE_TMTGTBUILDROOT)/project ;\
	$(ECHO) \# $@ >$@ ;\
	$(ECHO) \# DO NOT EDIT. This file has been generated automatically. >>$@
	for comp in $(_SDE_ALL_COMPS_DIR) ; \
	do \
	if [ -d $$comp ] ; then \
	$(ECHO) "_`basename $$comp`_DIR := $$comp">> $@; \
	fi ;\
	done

-include                $(LOC_LIST_MKFILENAME)

endif
endif

endif
endif

#-----------------------------------------------------------------------
# add REQUIRES_INTFS, REQUIRES_COMPS, PROVIDES_INTFS to REQUIRES and PROVIDES
#-----------------------------------------------------------------------
REQUIRES += $(strip $(REQUIRES_INTFS) $(REQUIRES_COMPS) $(REQUIRES_TYPES))
PROVIDES += $(PROVIDES_INTFS)

#-------------------------------------------------------------------------
# Look for incorrect C/CXX/CFG_SOURCES entries
#-------------------------------------------------------------------------
_SDE_OTHER_CXX_SOURCES := $(filter-out %.cpp,$(CXX_SOURCES))
_SDE_OTHER_C_SOURCES := $(filter-out %.c,$(C_SOURCES))
_SDE_OTHER_CFG_SOURCES := $(filter-out %.c,$(CFG_SOURCES))
_SDE_OTHER_S_SOURCES := $(filter-out %.s %.S %.asm %.arm %.t,$(S_SOURCES))
ifneq ($(_SDE_OTHER_CXX_SOURCES),)
$(error CXX_SOURCES should not contain $(_SDE_OTHER_CXX_SOURCES))
endif
ifneq ($(_SDE_OTHER_C_SOURCES),)
$(error C_SOURCES should not contain $(_SDE_OTHER_C_SOURCES))
endif
ifneq ($(_SDE_OTHER_CFG_SOURCES),)
$(error CFG_SOURCES should not contain $(_SDE_OTHER_CFG_SOURCES))
endif
ifneq ($(_SDE_OTHER_S_SOURCES),)
$(error S_SOURCES should not contain $(_SDE_OTHER_S_SOURCES))
endif

#-----------------------------------------------------------------------
# compute dependency files from sources
#-----------------------------------------------------------------------
ifneq ($(_TMNODEPENDENCIES),1)
_SDE_DEPENDENCIES+=\
        $(CXX_SOURCES:%.cpp=$(DIR_INTERM)/%.d) \
        $(C_SOURCES:%.c=$(DIR_INTERM)/%.d) \
        $(CFG_SOURCES:%.c=$(DIR_INTERM)/%.d)
endif

#-----------------------------------------------------------------------
# include dependencies before default rules. The generation of the
# dependecy files for gcc depend on the options file too (c.opt etc are not
# generated at the time the includes of the dependency files are done,
# and thus gmake starts to build them before building any other targets)
# This then overwrites the specific rule in the dependency file since
# it is ealier in this file
#-----------------------------------------------------------------------
# This defines that all objects depend on _SDE_DEP_MAKEFILES
$(_SDE_OBJECTS):$(_SDE_DEP_MAKEFILES)

-include preventwarning $(_SDE_DEPENDENCIES)

#-----------------------------------------------------------------------
# Configuration check (passive part)
#------------------------------------------------------------------------
ifndef _TMLINKTYPE
_TMLINKTYPE ?= static
endif
ifneq ($(_TMLINKTYPE),static)
ifneq ($(_TMLINKTYPE),dynamic)
ifneq ($(_SDE_BUILD_TARGET),LIB)
_SDE_ERROR = _TMLINKTYPE does not have a legal value.\nLegal values are:\n\tstatic, dynamic
endif
endif
endif


#-----------------------------------------------------------------------
# Option files used to store compile settings
#-----------------------------------------------------------------------
_SDE_ALL_AOPTS=$(_SDE_AOPTS_FILE1) $(_SDE_AOPTS_FILE2) $(_SDE_AOPTS_FILE3) $(_SDE_AOPTS_FILE4) $(_TMTGTAOPTS)
_SDE_ALL_COPTS=$(_SDE_COPTS_FILE1) $(_SDE_COPTS_FILE2) $(_SDE_COPTS_FILE3) $(_SDE_COPTS_FILE4) $(_SDE_PROVIDED_BY_FLAGS) $(_TMTGTCOPTS)
_SDE_ALL_CXXOPTS=$(_SDE_CXXOPTS_FILE1) $(_SDE_CXXOPTS_FILE2) $(_SDE_CXXOPTS_FILE3) $(_SDE_CXXOPTS_FILE4) $(_SDE_PROVIDED_BY_FLAGS) $(_TMTGTCXXOPTS)

# We keep the option files only for mips_psos configuration
ifeq ($(_TMTGTCPUCLASS),mips)
ifeq ($(_TMTGTOSCLASS),psos)

_SDE_AOPTFILENAME := $(DIR_INTERM)/a.opt
_SDE_COPTFILENAME := $(DIR_INTERM)/c.opt
_SDE_CXXOPTFILENAME := $(DIR_INTERM)/cxx.opt
_SDE_LOPTFILENAME := $(DIR_INTERM)/l.opt
_SDE_OPTIONFILES += $(_SDE_AOPTFILENAME) $(_SDE_COPTFILENAME)


ifndef _SDE_AOPTFILE_DEPENDS
_SDE_AOPTFILE_DEPENDS=$(_SDE_DEP_MAKEFILES)
endif
$(_SDE_AOPTFILENAME): $(_SDE_AOPTFILE_DEPENDS)
	$(_SDE_ECHO)$(MKDIR) $(DIR_INTERM) ; \
	$(ECHO) $(_SDE_ALL_AOPTS)   >  $@

ifndef _SDE_COPTFILE_DEPENDS
_SDE_COPTFILE_DEPENDS=$(_SDE_DEP_MAKEFILES)
endif
$(_SDE_COPTFILENAME): $(_SDE_COPTFILE_DEPENDS)
	$(_SDE_ECHO)$(MKDIR) $(DIR_INTERM) ; \
	$(ECHO) $(_SDE_ALL_COPTS)  >  $@

ifndef _SDE_CXXOPTFILE_DEPENDS
_SDE_CXXOPTFILE_DEPENDS=$(_SDE_DEP_MAKEFILES)
endif
$(_SDE_CXXOPTFILENAME): $(_SDE_CXXOPTFILE_DEPENDS)
	$(_SDE_ECHO)$(MKDIR) $(DIR_INTERM) ; \
	$(ECHO) $(_SDE_ALL_CXXOPTS)   > $@

ifndef _SDE_LOPTFILE_DEPENDS
_SDE_LOPTFILE_DEPENDS=$(_SDE_DEP_MAKEFILES)
endif
$(_SDE_LOPTFILENAME):: $(_SDE_LOPTFILE_DEPENDS)
	$(_SDE_ECHO)$(MKDIR) $(DIR_INTERM) ; \
	$(ECHO) ''                > $@

endif
endif

#-----------------------------------------------------------------------
# Rules for intermediate files
#-----------------------------------------------------------------------
ifneq ($(findstring _sde_preprocess,$(MAKECMDGOALS)),)
_TMTGTCPP=1
endif
ifeq ($(_TMTGTCPP),1)
_SDE_I = i

_SDE_CPP_FILES= $(CXX_SOURCES:%.cpp=$(DIR_INTERM)/%.$(_SDE_I))\
                $(C_SOURCES:%.c=$(DIR_INTERM)/%.$(_SDE_I)) \
                $(CFG_SOURCES:%.c=$(DIR_INTERM)/%.$(_SDE_I))

$(DIR_INTERM)/%.i: %.cpp
	$(_SDE_ECHO)$(MKDIR) $(dir $@);                                       \
	$(ECHO) generating $@ ;\
	$(CXX) $(_SDE_CXXOPTS) $(TARGET_CFLAGS) -E $< > $@

$(DIR_INTERM)/%.i: %.c
	$(_SDE_ECHO)$(MKDIR) $(dir $@);                                       \
	$(ECHO) generating $@ ;\
	$(CC) $(_SDE_COPTS) $(TARGET_CFLAGS) -E $< > $@

_sde_preprocess : $(_SDE_CPP_FILES)


target all: _sde_preprocess
lib:: _sde_preprocess
endif # ifeq ($(_TMTGTCPP),1)

#-----------------------------------------------------------------------
# create the component name.
#-----------------------------------------------------------------------
ifeq ($(_SDE_BUILD_TARGET),LIB)
TARGET=$(notdir $(DIR_LOCAL))
endif

#-----------------------------------------------------------------------
# Binary release location for applications.
#-----------------------------------------------------------------------
ifeq ($(_SDE_BUILD_TARGET),EXE)
_SDE_LOCAL_TARGET_DIR :=$(CURDIR)/$(_SDE_DIR_BIN_EXTPATH)
endif

#-----------------------------------------------------------------------
# Binary release availability checking
#-----------------------------------------------------------------------

check_binary_release:
	@$(ECHO) 'check_binary_release=$(BINARY_RELEASE_AVAILABILITY)'

PRESENT_BINARY_RELEASE_FILES = \
     $(wildcard $(CHECK_BINARY_RELEASE_FILES))

MISSING_BINARY_RELEASE_FILES = $(filter-out \
     $(PRESENT_BINARY_RELEASE_FILES), \
     $(CHECK_BINARY_RELEASE_FILES))

BINARY_RELEASE_AVAILABILITY=$(strip \
  $(if $(filter 1,$(_TMECHO)),\
    $(foreach f,$(MISSING_BINARY_RELEASE_FILES),$(shell $(ECHO) 'check_binary_release: missing: $f' >&2)) \
    $(foreach f,$(PRESENT_BINARY_RELEASE_FILES),$(shell $(ECHO) 'check_binary_release: present: $f' >&2))) \
  $(if $(MISSING_BINARY_RELEASE_FILES), \
    false, \
    true))

binary_release_available_message:
	@$(ECHO) 'Binary release available for $(DIR_LOCAL)'

#-----------------------------------------------------------------------
# Compute how many directories we have to go back to get at the root level
#-----------------------------------------------------------------------
_empty_ :=
_space_ :=      $(_empty_) $(_empty_)

# Do not add extra spaces in this line
_SDE_DIR_REL_TO_LOCAL_ROOT := $(subst $(_space_),/,$(patsubst %,..,$(filter-out .,$(subst /, ,$(DIR_LOCAL)))))

#-----------------------------------------------------------------------
# complicated stuff to reduce the -I paths, do not include
# interface directories which are not there.
#-----------------------------------------------------------------------
ifneq ($(strip $(REQUIRES) $(PROVIDES)), )
ifeq ($(_TMNESTEDINCLUDE),1)

_SDE_IMPORT_REQUIRES :=                                                       \
    $(shell                                                                   \
    $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .r $(REQUIRES) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic_requires.awk                \
    )
_SDE_MISSING_RFILES := $(filter ERR_R_%, $(_SDE_IMPORT_REQUIRES))
_SDE_MISSING_RFILES := $(patsubst ERR_R_%,%,$(_SDE_MISSING_RFILES))
_SDE_RFILES_NOTREQD := $(foreach intfs,$(_SDE_MISSING_RFILES),$(if $(_I$(intfs)_DIR),$(intfs),))
_SDE_MISSING_RFILES := $(strip $(filter-out $(_SDE_RFILES_NOTREQD),$(_SDE_MISSING_RFILES)))
_comma_ := ,
_SDE_IMPORT_REQUIRES := $(patsubst ERR_R_%,%,$(_SDE_IMPORT_REQUIRES))
ifneq ($(_SDE_MISSING_RFILES),)
_comma_ := ,
_SDE_WARNING    += 'Warning: $(subst $(_space_),$(_comma_),$(sort $(patsubst %,%.r,$(_SDE_MISSING_RFILES)))) file(s) in $(_SDE_TMTGTBUILDROOT)/comps/<CompName>/tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY) is/are missing. Please read Chapter 3.7.4 of the User Manual!!!\\n'
endif
_SDE_REQUIRED_EXTPROJECT_INTERFACES := $(foreach z,$(strip $(_SDE_IMPORT_REQUIRES)),         \
                                        $(if $(_I$(z)_DIR), $(wildcard $(_I$(z)_DIR)/inc),)  \
                                        $(if $(_$(z)_DIR), $(wildcard $(_$(z)_DIR)/inc),)    \
                                       )
else
_SDE_AVAILABLE_REQUIRED_INTERFACES :=  $(foreach z,$(strip $(REQUIRES) $(PROVIDES)),   \
                                           $(if $(wildcard $(_I$(z)_DIR)/inc), $(z),)  \
					   $(if $(wildcard $(_I$(z)_DIR)/idl), $(z),)  \
                                           $(if $(wildcard $(_$(z)_DIR)/inc), $(z),)   \
                                        )

_SDE_MISSING_REQUIRED_INTERFACES := $(filter-out $(_SDE_AVAILABLE_REQUIRED_INTERFACES),$(strip $(REQUIRES) $(PROVIDES)))

_SDE_MISSING_INTERFACES := $(filter-out PROVIDED_BY, $(_SDE_MISSING_REQUIRED_INTERFACES))

ifneq ($(_SDE_MISSING_INTERFACES),)
_SDE_WARNING += 'Warning: The interfaces $(_SDE_MISSING_INTERFACES) are missing in the project tree\n'
endif

_SDE_REQUIRED_EXTPROJECT_INTERFACES := $(foreach z,$(strip $(REQUIRES) $(PROVIDES)),            \
                                           $(if $(_I$(z)_DIR), $(wildcard $(_I$(z)_DIR)/inc),)  \
                                           $(if $(_$(z)_DIR), $(wildcard $(_$(z)_DIR)/inc),)    \
                                        )
endif
endif

_SDE_REQUIRED_INTERFACES := $(strip $(_SDE_REQUIRED_EXTPROJECT_INTERFACES))
#-----------------------------------------------------------------------
# provided_by add-on. Remove provided_by.mk if you do not use provided_by.mk mechanism
#-----------------------------------------------------------------------
-include $(DIR_SDE)/provided_by.mk
#-----------------------------------------------------------------------
# obscure commands to filter -D -I and -U from the command
# line arguments to the compiler, pass that to gcc -M to
# produce dependencies, then parse that to make .d dependency
#
# The cat -v is to get printable ^M and the sed commands filters that out,
# otherwise options get a ^M attached to them and cpp says no-comprendo.
#-----------------------------------------------------------------------
_SDE_GCC_CXXOPTIONS=$(filter -D% -I% -U%, $(_SDE_ALL_CXXOPTS)) $(_SDE_PROVIDED_BY_FLAGS) -w

_SDE_GCC_COPTIONS=$(filter -D% -I% -U%, $(_SDE_ALL_COPTS)) $(_SDE_PROVIDED_BY_FLAGS) $(_SDE_MSC) -w

_SDE_GCC_AOPTIONS=$(filter -D% -I% -U% -x%, $(_SDE_ALL_AOPTS) $(_SDE_INCLUDES)) $(_SDE_PROVIDED_BY_FLAGS) -w

#==============================================================================
# Default includes for the interfaces
#==============================================================================
CPU_OS:=$(_TMTGTCPUCLASS)_$(_TMTGTOSCLASS)

_SDE_INCLUDES  = $(DIR_INCLUDE:%=-I%)

_SDE_TMP_INCLUDES1:=inc src cfg types
_SDE_TMP_INCLUDES2:=$(DIR_CONFIG) $(CPU_OS) $(_TMTGTCPUCLASS) $(_TMTGTOSCLASS)

define _SDE_EXPAND_INCLUDES
$(foreach y,$(_SDE_TMP_INCLUDES1),$(if $(wildcard $(1)/$(y)),-I$(1)/$(y)) \
            $(foreach z,$(_SDE_TMP_INCLUDES2),$(if $(wildcard $(1)/$(y)/$(z)),-I$(1)/$(y)/$(z))))
endef

define _SDE_EXPAND_VPATH
$(1)/$(2) $(foreach z,$(_SDE_TMP_INCLUDES2),$(1)/$(2)/$(z))
endef

_SDE_INCLUDES += $(call _SDE_EXPAND_INCLUDES,$(CURDIR))
_SDE_INCLUDES += $(call _SDE_EXPAND_INCLUDES,$(_SDE_DIR_REL_TO_LOCAL_ROOT))

_SDE_INCLUDES += $(LOCAL_INCLUDES:%=-I%)

#==============================================================================
# Rules to make a dependency file from a C or C++ or ASM source file:
#

ifneq ($(findstring $(DIR_CONFIG),x86ddk_nt x86osci_nt),)
define makecxxdeps
$(strip $(subst $(_space_)\,$(_space_),$(shell $(MKDIR) -p $(dir $@);$(GCC) --no-warn -D_M_IX86=1 -M -MP -MT $@ -MF $(@:.$(_SDE_O)=.d) $(_SDE_GCC_CXXOPTIONS) $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<))))
endef
else
define makecxxdeps
$(strip $(subst $(_space_)\,$(_space_),$(shell $(MKDIR) -p $(dir $@);$(GCC) --no-warn -M -MP -MT $@ -MF $(@:.$(_SDE_O)=.d) $(_SDE_GCC_CXXOPTIONS) $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<))))
endef
endif

ifneq ($(findstring $(DIR_CONFIG),x86ddk_nt x86osci_nt),)
define makecdeps
$(strip $(subst $(_space_)\,$(_space_),$(shell $(MKDIR) -p $(dir $@);$(GCC) --no-warn -D_M_IX86=1 -M -MP -MT $@ -MF $(@:.$(_SDE_O)=.d) $(_SDE_GCC_COPTIONS) $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<))))
endef
else
define makecdeps
$(strip $(subst $(_space_)\,$(_space_),$(shell $(MKDIR) -p $(dir $@);$(GCC) --no-warn -M -MP -MT $@ -MF $(@:.$(_SDE_O)=.d) $(_SDE_GCC_COPTIONS) $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<))))
endef
endif

define makeasmdeps
$(strip $(subst $(_space_)\,$(_space_),$(shell $(MKDIR) -p $(dir $@);$(GCC) --no-warn -M -MP -MT $@ -MF $(@:.$(_SDE_O)=.d) $(_SDE_GCC_AOPTIONS) $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<))))
endef

_SDE_INCLUDES += -I$(_TMROOT)/inc
_SDE_INCLUDES += $(if $(wildcard $(_TMROOT)/types),-I$(_TMROOT)/types)
_SDE_INCLUDES += $(if $(wildcard $(_TMROOT)/inc/cfg),-I$(_TMROOT)/inc/cfg)
ifneq ($(_TMPROJECT),)
_SDE_INCLUDES += $(if $(wildcard $(_TMPROJECT)/inc/cfg),-I$(_TMPROJECT)/inc/cfg)
_SDE_INCLUDES += $(if $(wildcard $(_TMPROJECT)/types),-I$(_TMPROJECT)/types)
endif
_SDE_INCLUDES += $(call _SDE_EXPAND_INCLUDES,$(_TMROOT))
_SDE_INCLUDES += -I$(_SDE_DIR_LIB)                	# for tmFlags.h
_SDE_OSAPI_INC_DIR := $(foreach COMPS, $(_SDE_REQPRJ), \
                      $(wildcard $(COMPS)/types/tmStdTypes/inc))
_SDE_OSAPI_INC_DIR := $(subst $(_space_),,$(_SDE_OSAPI_INC_DIR))
ifeq ($(_SDE_OSAPI_INC_DIR),)
_SDE_OSAPI_INC_DIR := $(_TMROOT)/sd/os/osapi/types/tmStdTypes/inc
endif
_SDE_AVSFW_INC_DIR := $(foreach COMPS, $(_SDE_REQPRJ), \
                      $(wildcard $(COMPS)/types/tmAvTypes/inc))
_SDE_AVSFW_INC_DIR := $(subst $(_space_),,$(_SDE_AVSFW_INC_DIR))
ifeq ($(_SDE_AVSFW_INC_DIR),)
_SDE_AVSFW_INC_DIR := $(_TMROOT)/sd/avs/avsfw/types/tmAvTypes/inc
endif
_SDE_INCLUDES += $(if $(wildcard $(_SDE_AVSFW_INC_DIR)),                \
                 -I$(_SDE_AVSFW_INC_DIR))
_SDE_INCLUDES += $(foreach z,$(_SDE_TMP_INCLUDES2),$(if $(wildcard $(_SDE_AVSFW_INC_DIR)/$(z)),-I$(_SDE_AVSFW_INC_DIR)/$(z)))
_SDE_INCLUDES += $(if $(wildcard $(_SDE_OSAPI_INC_DIR)),                \
                 -I$(_SDE_OSAPI_INC_DIR))
_SDE_INCLUDES += $(foreach z,$(_SDE_TMP_INCLUDES2),$(if $(wildcard $(_SDE_OSAPI_INC_DIR)/$(z)),-I$(_SDE_OSAPI_INC_DIR)/$(z)))
_SDE_INCLUDES += $(_SDE_REQUIRED_INTERFACES:%=-I%)
_SDE_INCLUDES += $(foreach COMPS, $(_SDE_REQPRJ), $(call _SDE_EXPAND_INCLUDES,$(COMPS)))

#-----------------------------------------------------------------------
# Add path to tmCustomSysCfg.h
#-----------------------------------------------------------------------
ifneq ($(_TM_USER_DEF_TMCUSTOMSYSCFG_LOC),)
_SDE_INCLUDES += -I$(_TM_USER_DEF_TMCUSTOMSYSCFG_LOC)
endif

#-----------------------------------------------------------------------
# All platform specific directories and _TMTGTINCLUDES are 
# included in configuration specific directory.
#-----------------------------------------------------------------------

define dependencies
$(patsubst %.o:, $@:,\
$(if $(filter %.c,$<),$(makecdeps),\
$(if $(filter %.cpp,$<),$(makecxxdeps),\
$(if $(filter %.s %.S %.asm %.t,$<),$(makeasmdeps)\
))))
endef

ifneq ($(_TMNODEPENDENCIES),1)
define run_deps
$(_SDE_ECHO)$(ECHO) 'Making dependencies for $(notdir $<) ...'
$(dependencies)
endef
else
define run_deps
$(_SDE_ECHO)$(ECHO) 'Not Generating dependencies for $(notdir $<) ...'
endef
endif
#------------------------------------------------------------------------
# qac add ons. Remove qac.mk if you do not use qac rules
#------------------------------------------------------------------------
-include $(DIR_SDE)/qac.mk

#------------------------------------------------------------------------
# PC Lint add ons. Remove lint.mk if you do not use lint rules
#------------------------------------------------------------------------
-include $(DIR_SDE)/lint.mk

#------------------------------------------------------------------------
# qmore add ons. Remove qmore.mk if you do not use qmore rule
#------------------------------------------------------------------------
-include $(DIR_SDE)/qmore.mk

#------------------------------------------------------------------------
# rgd add ons. Remove rgd.mk if you do not use rgd rules
#------------------------------------------------------------------------
-include $(DIR_SDE)/rgd.mk

#------------------------------------------------------------------------
# Doxygen autodocumentation. Remove autodoc.mk if you do not use autodocumentation
#------------------------------------------------------------------------
ifneq ($(filter userdoc devdoc,$(_TMRUNDOXYGEN) $(MAKECMDGOALS)),)
-include $(DIR_SDE)/autodoc/autodoc.mk
endif

#------------------------------------------------------------------------
# Docjet autodocumentation. Remove docjet.mk if you do not use docjet 
#------------------------------------------------------------------------
ifneq ($(filter docjet,$(MAKECMDGOALS)),)
-include $(DIR_SDE)/docjet/docjet.mk
endif

#------------------------------------------------------------------------
# To enable easy integration of 3rd party tools. 
# Users needs to update $(_TMPROJECT)/project_include.mk and include
# the new *.mk file for integrating new tools.
#------------------------------------------------------------------------
-include $(_SDE_PROJECT)/project_include.mk

#------------------------------------------------------------------------
# For backward compatibility with 1.1.5
#------------------------------------------------------------------------
ifdef DLLS
LIBS += $(DLLS)
endif
LIBS := $(LIBS:%_g=%)
LIBS := $(LIBS:%_a=%)
LIBS := $(LIBS:%_t=%)

# Remove the self component reference of LIBS, if any for only components
ifeq ($(_SDE_BUILD_TARGET),LIB)
ifeq ($(filter $(notdir $(DIR_LOCAL)),$(LIBS)),$(notdir $(DIR_LOCAL)))
LIBS := $(filter-out $(notdir $(DIR_LOCAL)), $(LIBS))
_SDE_WARNING += 'Warning: $(notdir $(DIR_LOCAL)) is removed from the LIBS section, as it is a self reference\\n'
endif
endif


#------------------------------------------------------------------------
# target rule for run time configuration
#------------------------------------------------------------------------

ifneq ($(CFG_SOURCES),)

lib :: run_time_config

_SDE_DIR_CONFIGURATION=$(_TMTGTBUILDROOT)/$(DIR_LOCAL)/cfg

run_time_config :: $(_SDE_DIR_CONFIGURATION)

$(_SDE_DIR_CONFIGURATION) :: $(wildcard cfg)
ifneq ($(strip $(_TMTGTBUILDROOT)),$(strip $(_TMROOT)))
	$(_SDE_ECHO)$(ECHO) copying the configuration files to $(_SDE_DIR_CONFIGURATION); \
	$(RM) -r $(_SDE_DIR_CONFIGURATION); \
	$(MKDIR) $(_SDE_DIR_CONFIGURATION); \
	$(CP) -r $(wildcard cfg/*) $(_SDE_DIR_CONFIGURATION)
endif

endif

#------------------------------------------------------------------------
# target rule for cleaning this directory
#------------------------------------------------------------------------
clean_target::
	$(_SDE_ECHO)$(RM) -r	$(_SDE_DIR_BIN)		\
				$(DIR_INTERM)/*		\
				$(_SDE_L_DIR)
	$(_SDE_ECHO)if [ x$(PSOS_CONFIG) != x ] ;then \
		$(RM) -r $(DIR_INTERM)/$(PSOS_CONFIG)/* ; \
	fi

clean_lib::
	$(_SDE_ECHO)$(RM) -r $(_SDE_THISLIB) $(_SDE_L_FILE) $(_SDE_R_FILE)
ifeq (retail,$(_TMTGTREL))
	$(_SDE_ECHO)$(RM) -r									\
	$(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)/*.*	\
	$(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)/src	\
	$(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)/cfg	
else
	$(_SDE_ECHO)$(RM) -r									\
	$(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)/*
endif
ifneq (,$(_SDE_THISDLL))
	$(_SDE_ECHO)$(RM) -r $(_SDE_THISDLL:%.$(_SDE_DLL_EXTN)=%.*)
endif
ifneq ($(strip $(_TMTGTBUILDROOT)),$(strip $(_TMROOT)))
	$(if $(_SDE_DIR_CONFIGURATION),$(_SDE_ECHO)$(RM) -r $(_SDE_DIR_CONFIGURATION))
endif

clean_all::
	$(_SDE_ECHO)$(ECHO) Start cleaning all ...
ifeq ($(strip $(_TMROOT)),$(strip $(_TMTGTBUILDROOT)))
	$(_SDE_ECHO)$(ECHO) Removing $(_TMROOT)/comps/generated;\
	$(RM) -r $(_TMROOT)/comps/generated;        \
	$(ECHO) Removing $(_TMROOT)/$(DIR_LOCAL)/tmp;           \
	$(RM) -r $(_TMROOT)/$(DIR_LOCAL)/tmp;       \
	$(ECHO) Removing $(_TMROOT)/intfs/generated;            \
	$(RM) -r $(_TMROOT)/intfs/generated;        \
	$(ECHO) Removing $(_TMROOT)/tmp;                        \
	$(RM) -r $(_TMROOT)/tmp;                    \
	$(ECHO) Removing $(_TMROOT)/$(DIR_LOCAL)/bin;           \
	$(RM) -r $(_TMROOT)/$(DIR_LOCAL)/bin;       \
	$(ECHO) Removing $(_TMROOT)/project/loc_list.*;         \
	$(RM) -r $(_TMROOT)/project/loc_list.*
else
ifneq (,$(strip $(_TMTGTBUILDROOT)))
	$(_SDE_ECHO)$(ECHO) Removing $(_TMTGTBUILDROOT)/comps/generated;\
	$(RM) -r $(_TMTGTBUILDROOT)/comps/generated;        \
	$(ECHO) Removing $(_TMTGTBUILDROOT)/$(DIR_LOCAL); 		\
	$(RM) -r $(_TMTGTBUILDROOT)/$(DIR_LOCAL); 		\
	$(ECHO) Removing $(_TMTGTBUILDROOT)/intfs/generated; 		\
	$(RM) -r $(_TMTGTBUILDROOT)/intfs/generated; 	\
	$(ECHO) Removing $(_TMTGTBUILDROOT)/tmp; 			\
	$(RM) -r $(_TMTGTBUILDROOT)/tmp; 			\
	$(ECHO) Removing $(_TMTGTBUILDROOT)/project/loc_list.*;	 	\
	$(RM) -r $(_TMTGTBUILDROOT)/project/loc_list.*; 	\
	$(ECHO) ; 							\
	$(ECHO) Please remove the $(_TMTGTBUILDROOT) directory manually
else
	$(_SDE_ECHO)$(ECHO) Removing $(_TMROOT)/comps/generated;\
	$(RM) -r $(_TMROOT)/comps/generated;        \
	$(ECHO) Removing $(_TMROOT)/$(DIR_LOCAL)/tmp;           \
	$(RM) -r $(_TMROOT)/$(DIR_LOCAL)/tmp;       \
	$(ECHO) Removing $(_TMROOT)/intfs/generated;            \
	$(RM) -r $(_TMROOT)/intfs/generated;        \
	$(ECHO) Removing $(_TMROOT)/tmp;                        \
	$(RM) -r $(_TMROOT)/tmp;                    \
	$(ECHO) Removing $(_TMROOT)/$(DIR_LOCAL)/bin;           \
	$(RM) -r $(_TMROOT)/$(DIR_LOCAL)/bin;       \
	$(ECHO) Removing $(_TMROOT)/project/loc_list.*;         \
	$(RM) -r $(_TMROOT)/project/loc_list.*
endif
endif
#-----------------------------------------------------------------------
# Configuration check
#------------------------------------------------------------------------
configuration:: local_configuration _sde_libs
ifneq ($(_SDE_ERROR),)
	$(_SDE_ECHO)$(ECHO) $(ECHOFLAGS) '$(_SDE_ERROR)'
	$(_SDE_ECHO)exit 1
endif


#-----------------------------------------------------------------------
# Include module for DependsOn relation and flavours
#-----------------------------------------------------------------------
include $(DIR_SDE)/libs.mk

#-----------------------------------------------------------------------
# IDL support
#-----------------------------------------------------------------------
-include $(DIR_SDE)/idl.mk

#-----------------------------------------------------------------------
# Generated the dependent Makefiles list.
#-----------------------------------------------------------------------
ifneq ($(MAKE_VERSION),3.79.1)
_SDE_TEMP_MAKEFILES += %loc_list.mk %tmFlags.mk %phFlags.mk %.d  %.cfg \
                       %$(UNAME).mk %default_host.mk %.cmd
_SDE_DEP_MAKEFILES := $(filter-out $(_SDE_TEMP_MAKEFILES),            \
                      $(MAKEFILE_LIST))
endif

#-----------------------------------------------------------------------
# Create the *.l file and *.r files
#-----------------------------------------------------------------------

_SDE_DEP_DIVERSITY_MAKEFILES :=                                               \
                $(wildcard                                                    \
                    $(foreach                                                 \
                        foo,                                                  \
                        $(LIBS),                                              \
                        $(_$(foo)_DIR)/diversity.mk                           \
                    )                                                         \
                )

ifeq ($(filter clean clean_all,$(MAKECMDGOALS)),)

ifeq ($(_SDE_DIVERSITY),.)
_SDE_L_DIR :=   $(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)
else
_SDE_L_DIR :=   $(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)
endif
_SDE_R_DIR := $(_SDE_L_DIR)
_SDE_L_FILE :=  $(_SDE_L_DIR)/$(TARGET).l
_SDE_R_FILE :=  $(_SDE_R_DIR)/$(TARGET).r


$(_SDE_L_FILE):                                                               \
                $(_SDE_DEP_MAKEFILES)                                         \
                $(_SDE_DEP_DIVERSITY_MAKEFILES)
	$(_SDE_ECHO)$(MKDIR) $(_SDE_L_DIR);                                   \
	$(ECHO) $(LIBS) > $(_SDE_L_FILE)

$(_SDE_R_FILE):                                                               \
                $(_SDE_DEP_MAKEFILES)                                         \
                $(_SDE_DEP_DIVERSITY_MAKEFILES)
	$(_SDE_ECHO)$(MKDIR) $(_SDE_R_DIR);                                   \
	$(ECHO) $(REQUIRES) > $(_SDE_R_FILE)


_sde_libs::                                                                   \
                $(_SDE_L_FILE)						      \
		$(_SDE_R_FILE)
endif #ifeq ($(filter clean clean_all,$(MAKECMDGOALS)),)

# LIBS is the list of components without suffixes
#
# _SDE_LIBS_DIVERSITY is the list of components, each with its diversity
# (and REL_SUFFIX) appended.


# First level dep. *.l files only, it should be implemented for all *.l files

_SDE_DEP_L_FILES :=                                                           \
                $(wildcard                                                    \
                    $(foreach                                                 \
                        foo,                                                  \
                        $(LIBS),                                              \
                        $(_SDE_TMTGTBUILDROOT)/comps/$(foo)/tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/$(foo).l   \
                    )                                                         \
                )                                                             \
                $(_SDE_L_FILE)

#------------------------------------------------------------------------
# Add binary release for object files. Executables need these feature only
#------------------------------------------------------------------------
ifneq ($(_SDE_BUILD_TARGET),LIB)
_SDE_BIN_REL_OBJ := $(foreach ob, $(_SDE_OBJECTS),   \
                        $(if $(wildcard $(ob)),,     \
                           $(foreach re, $(REQUIRES),\
                               $(if $(wildcard $(_$(re)_DIR)/lib/$(_SDE_LIB_CONFIGURATION)/$(_$(re)_DIVERSITY)$(_SDE_ARSUFFIX)/$(ob)), \
                                               $(_$(re)_DIR)/lib/$(_SDE_LIB_CONFIGURATION)/$(_$(re)_DIVERSITY)$(_SDE_ARSUFFIX)/$(ob),) \
                           )                         \
                        )                            \
                     )

_SDE_OBJECTS     := $(foreach ob, $(_SDE_OBJECTS),                     \
                       $(if $(findstring $(ob), $(_SDE_BIN_REL_OBJ)),  \
                          $(filter %$(ob), $(_SDE_BIN_REL_OBJ)), $(ob) \
                       )                                               \
                    )
endif

#------------------------------------------------------------------------
# Define all vpaths here
#------------------------------------------------------------------------
vpath %.$(_SDE_LIB_EXTN) $(_SDE_DIR_LIB) $(_SDE_REQUIRED_PATH_LIBS)

ifdef _SDE_DLL_EXTN
vpath %.$(_SDE_LIB_EXTN) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)

vpath %.$(_SDE_DLL_EXTN) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)  \
                         $(_SDE_REQUIRED_PATH_DLLS)
endif

ifneq ($(CYGWIN),)
vpath %.$(_SDE_LIB_EXTN) $(foreach dir,$(_SDE_DIR_LIB) \
                         $(_SDE_REQUIRED_PATH_LIBS)    \
                         $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY),$(shell cygpath -u $(dir)))
ifdef _SDE_DLL_EXTN
vpath %.$(_SDE_DLL_EXTN) $(foreach dir,$(_SDE_DIR_LIB)/$(_SDE_DIVERSITY) \
                         $(_SDE_REQUIRED_PATH_DLLS),$(shell cygpath -u $(dir)))
endif
endif

vpath %.c    $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.cpp  $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.s    $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.S    $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.asm  $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.arm  $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.t  $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.java $(call _SDE_EXPAND_VPATH,$(CURDIR),src)

vpath %.h    $(call _SDE_EXPAND_VPATH,$(CURDIR),inc) \
             $(call _SDE_EXPAND_VPATH,$(CURDIR),src) \
             $(call _SDE_EXPAND_VPATH,$(_TMROOT),inc) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_DIR_REL_TO_LOCAL_ROOT),inc) \
             $(_SDE_DIR_LIB) \
             $(foreach COMPS,$(_SDE_REQPRJ),$(call _SDE_EXPAND_VPATH,$(COMPS),inc)) \
             $(foreach COMPS,$(_SDE_REQPRJ),$(call _SDE_EXPAND_VPATH,$(COMPS),types)) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_OSAPI_INC_DIR)) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_OSAPI_INC_DIR))

vpath %.hpp  $(call _SDE_EXPAND_VPATH,$(CURDIR),inc) \
             $(call _SDE_EXPAND_VPATH,$(CURDIR),src) \
             $(call _SDE_EXPAND_VPATH,$(_TMROOT),inc) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_DIR_REL_TO_LOCAL_ROOT),inc) \
             $(_SDE_DIR_LIB) \
             $(foreach COMPS,$(_SDE_REQPRJ),$(call _SDE_EXPAND_VPATH,$(COMPS),inc)) \
             $(foreach COMPS,$(_SDE_REQPRJ),$(call _SDE_EXPAND_VPATH,$(COMPS),types)) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_OSAPI_INC_DIR)) \
             $(call _SDE_EXPAND_VPATH,$(_SDE_OSAPI_INC_DIR))

#------------------------------------------------------------------------
# suppress dlls generation
#------------------------------------------------------------------------
ifeq (_sde_suppress_dlls,$(filter _sde_suppress_dlls,$(MAKECMDGOALS)))
override EXPORTS=
endif

_sde_suppress_dlls: all
	$(_SDE_ECHO)$(ECHO) 'SDE2 DLLs generation is suppressed.'

FORCE:

#------------------------------------------------------------------------
# copy objects and other stuff to release directory
#------------------------------------------------------------------------

_sde_copy_objects:
	$(_SDE_ECHO)$(ECHO) 'Copying $(_SDE_COPYLIST) to $(_SDE_DIR_LIB)'; \
	$(CP) $(_SDE_COPYLIST) $(_SDE_DIR_LIB);


FORCE:

#-----------------------------------------------------------------------
# Function to return a quoted shell string, using single quotes.
# Treats only single qoutes as special.
#-----------------------------------------------------------------------
_SDE_SQUOTE='$(subst ','"'"',$1)'

#-----------------------------------------------------------------------
# Function to return a quoted shell string, using double quotes.
# Treats only double qoutes as special.
#-----------------------------------------------------------------------
_SDE_DQUOTE="$(subst ","'"'",$1)"

#-----------------------------------------------------------------------
# Print a variable. For debug purposes.
#------------------------------------------------------------------------
_sde_print_var:
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var)=$($(var)))
_sde_print_vardef:
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var)=$(value $(var)))

#-----------------------------------------------------------------------
# Print 5 variables. For debug purposes.
#------------------------------------------------------------------------
_sde_print_var_5:
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var)=$($(var)))
ifdef var2
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var2)=$($(var2)))
endif
ifdef var3
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var3)=$($(var3)))
endif
ifdef var4
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var4)=$($(var4)))
endif
ifdef var5
	$(_SDE_ECHO)$(ECHO) $(call _SDE_SQUOTE,$(var5)=$($(var5)))
endif

#------------------------------------------------------------------------
# help  and version target
#------------------------------------------------------------------------
version:
	@$(ECHO) $(ECHOFLAGS) 'SDE2 version $(_SDE_VERSION)\n'

ifneq ($(filter help,$(MAKECMDGOALS)),)
GCC ?= $(CC)
GCC_VERSION = $(strip $(shell $(GCC) -dumpversion))
endif

help: version
	@$(ECHO) $(ECHOFLAGS) $(HELP_MESSAGE)

HELP_MESSAGE :=  \
'(c) NXP Semiconductors 2007\n\n \
Current settings:\n \
-------------------------------------\n \
ENDIAN          :  $(_TMTGTENDIAN)\n \
CPU             :  $(_TMTGTCPUTYPE)\n \
CPUCLASS        :  $(_TMTGTCPUCLASS)\n \
TOOLCHAIN       :  $(_TMTOOLCHAIN)\n \
OS              :  $(_TMTGTOS)\n \
Link Type       :  $(_TMLINKTYPE)\n \
Compile Mode    :  $(_TMTGTREL)\n \
build dir       :  $(_TMTGTBUILDROOT)\n \
NODEPENDENCIES  :  $(_TMNODEPENDENCIES)\n \
Diversity Flag  :  $(_TMDIVERSITY)\n \
\n\
GCC             :  $(GCC_VERSION)\n \
GMAKE           :  $(MAKE_VERSION)\n \
\n\
Currently Supported Configurations \n\n \
-------|---------------------------------------------------------------------\n\
Config\t| ce cexec ecos inte- linux mtos nt nucleus nullos os- psos ucos vx-\n \
      \t                grity                              can           works\n \
-------|---------------------------------------------------------------------\n\
8051\t| -    -    -     -     -    -   -     -       X    -   -    -     -  \n \
arm \t| X    X    -     -     X    -   -     X       X    X   -    X      X \n \
dsprd\t| -    -    -     -     -    -   -     -       X    -   -    -      - \n \
dspvd\t| -    -    -     -     -    -   -     -       X    -   -    -      - \n \
hp  \t| -    -    -     -     -    -   -     -       X    -   -    -      - \n \
mips\t| X    -    X     X     X    -   -     -       X    -   X    -      X \n \
tm  \t| -    -    -     -     -    -   -     -       -    -   X    -      - \n \
x86 \t| X    -    -     -     X    -   X     -       X    -   -    -      X \n \
-----------------------------------------------------------------------------\n\
\n\
possible general targets:\n\n \
\t_sde_bin_rel       - copy binary release files\n \
\t_sde_copy_objects  - copy files to the release directory\n \
\t_sde_libs          - create *.l files\n \
\t_sde_print_var     - print the value of the var\n \
\t   For eg: gmake var=_SDE_INCLUDES _sde_print_var\n \
\t_sde_suppress_dlls - suppress DLLs generation, if any\n \
\tclean              - remove intermediate files\n \
\tclean_all          - remove the whole build tree\n \
\tdevdoc             - generate developer autodocumentation \n \
\t                     (requires DOXYGEN)\n \
\thelp               - this help message\n \
\tlint               - run lint\n \
\tmakelist           - update project files\n \
\tqac                - run QAC on the component\n \
\tqmore              - run qmore on the component\n \
\trgd                - generate *.h file from rgd file\n \
\tuserdoc            - generate user autodocumentation\n \
\t                     (requires DOXYGEN)\n \
\tversion            - display the current SDE2 version\n'

FORCE:

.PHONY: configuration local_configuration makelist all target lib help clean_lib clean_all clean_target clean _sde_print_var rgd qac lint qmore _sde_libs _sde_suppress_dlls _sde_copy_objects default_rule chkconfig version install _sde_install_none

else #ifndef _SDE_INCLUDE_SDE_COMMON_MK_

#------------------------------------------------------------------------
# Command dependency tracking
#------------------------------------------------------------------------
-include $(DIR_INTERM)/.c.o.cmd
-include $(DIR_INTERM)/.cpp.o.cmd
-include $(DIR_INTERM)/.s.o.cmd

define _SDE_COMMAND_CHANGE_CHECK
ifneq "$(TMFL_MODE_KERNEL)" "1"
ifdef _SDE_C_COMPILE_CMD_PREV
ifneq ($(strip $(_SDE_C_COMPILE_CMD)),$(strip $(_SDE_C_COMPILE_CMD_PREV)))
$(C_SOURCES:%.c=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
endif #ifdef _SDE_C_COMPILE_CMD_PREV
endif #ifneq ($(strip $(_SDE_C_COMPILE_CMD)),$(strip $(_SDE_C_COMPILE_CMD_PREV)))

ifdef _SDE_CXX_COMPILE_CMD_PREV
ifneq ($(strip $(_SDE_CXX_COMPILE_CMD)),$(strip $(_SDE_CXX_COMPILE_CMD_PREV)))
$(CXX_SOURCES:%.cpp=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
endif #ifdef _SDE_CXX_COMPILE_CMD_PREV
endif #ifneq ($(strip $(_SDE_CXX_COMPILE_CMD)),$(strip $(_SDE_CXX_COMPILE_CMD_PREV)))

ifdef _SDE_A_COMPILE_CMD_PREV
ifneq ($(strip $(_SDE_A_COMPILE_CMD)),$(strip $(_SDE_A_COMPILE_CMD_PREV)))
$(S_SOURCES:%.s=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
$(S_SOURCES:%.S=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
$(S_SOURCES:%.asm=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
$(S_SOURCES:%.arm=$(DIR_INTERM)/%.$(_SDE_O)): FORCE
endif #ifdef _SDE_A_COMPILE_CMD_PREV
endif #ifneq ($(strip $(_SDE_A_COMPILE_CMD)),$(strip $(_SDE_A_COMPILE_CMD_PREV)))
endif
endef

_hash_ := \#
_dollar_ := $$
_SDE_STORE_CMD=$(ECHO) '$(1)_PREV:=$(subst $(_hash_),$$(_hash_),$(subst $(_dollar_),$$(_dollar_),$($(1))))' > $(DIR_INTERM)/.$(1)
#_SDE_STORE_CMD=$(ECHO) '$(1)_PREV:=$(subst $(_hash_),$$(_hash_),$(subst $(_dollar_),$$(_dollar_),$($(1))))' > $(DIR_INTERM)/.$(notdir $(2)).cmd

_SDE_END_COMMANDS += $(eval $(value _SDE_COMMAND_CHANGE_CHECK))

#------------------------------------------------------------------------
# Default install target
# By default run an empty install target:
all: install
install: _sde_install_none
_sde_install_none:;

# Utility target to use as a dependency on all object files.
# Added here so environment.mk (and any files included by it) can have targets
# that depend on this.
_sde_objects: $(_SDE_OBJECTS)

endif #ifndef _SDE_INCLUDE_SDE_COMMON_MK_
