_space_:=$(_empty_) $(_empty_)
_SDE_DIV_DEBUG:=warning
include diversity_common.mk

#-----------------------------------------------------------------------
# Basic ASSERT macros
#-----------------------------------------------------------------------

ASSERT_TRUE = $(if $(strip $1),,$(error FAILED to return non-blank string))
ASSERT_FALSE= $(if $(strip $1),$(error FAILED to return blank or empty string: $1))

$(call ASSERT_TRUE,blah)
$(call ASSERT_FALSE,)
#$(call ASSERT_FALSE,oops)
#$(call ASSERT_TRUE,)

ASSERT_EQUAL = $(strip $(if $(filter-out x$(subst $(_space_),_,$1),x$(subst $(_space_),_,$2)),\
  $(warning ASSERT_EQUAL arg 1 is $1) \
  $(warning ASSERT_EQUAL arg 2 is $2) \
  $(error ASSERT_EQUAL FAILED) \
))
$(call ASSERT_EQUAL,,)
$(call ASSERT_EQUAL,a,a)
$(call ASSERT_EQUAL,a a,a a)
#$(call ASSERT_EQUAL,a a,b)

#-----------------------------------------------------------------------
# Setup some _TMDIVERSITY value for test use:
_TMDIVERSITY:=_a_b_c_d_e_1_2_3_4_5
_SDE_CI:=tmTest
#-----------------------------------------------------------------------
# Interface diversity test
#-----------------------------------------------------------------------

# Macro: INTERFACE_DIVERSITY_TESTCASE
# Usage: $(call INTERFACE_DIVERSITY_TESTCASE,comp,diversities,result)
INTERFACE_DIVERSITY_TESTCASE=$(strip $(call INTERFACE_DIVERSITY_TESTCASE1,$1,$2,$3))
define INTERFACE_DIVERSITY_TESTCASE1
  $(warning INTERFACE_DIVERSITY_TESTCASE: $1, $2, $3)
  $(call INTERFACE_DIVERSITY,$1,$2)
  $(call ASSERT_EQUAL,$(_tmTest_DIVERSITY),$3)
endef
$(call INTERFACE_DIVERSITY_TESTCASE,tmTest,a,_a)
$(call INTERFACE_DIVERSITY_TESTCASE,tmInc,1,_1_a)

#-----------------------------------------------------------------------
# Implementation diversity test
#-----------------------------------------------------------------------

# Macro: IMPLEMENTATION_DIVERSITY_TESTCASE
# Usage: $(call IMPLEMENTATION_DIVERSITY_TESTCASE,comp,diversities,result)
IMPLEMENTATION_DIVERSITY_TESTCASE=$(strip $(call IMPLEMENTATION_DIVERSITY_TESTCASE1,$1,$2,$3))
define IMPLEMENTATION_DIVERSITY_TESTCASE1
  $(warning IMPLEMENTATION_DIVERSITY_TESTCASE: $1, $2, $3)
  $(call IMPLEMENTATION_DIVERSITY,$1,$2)
  $(call ASSERT_EQUAL,$(_tmTest_DIVERSITY),$3)
endef
$(call IMPLEMENTATION_DIVERSITY_TESTCASE,tmInc,2,_1_a)
$(call IMPLEMENTATION_DIVERSITY_TESTCASE,tmTest,xxx,_1_a)
$(call IMPLEMENTATION_DIVERSITY_TESTCASE,tmTest,b,_1_a_b)
$(call IMPLEMENTATION_DIVERSITY_TESTCASE,tmTest,a c d,_1_a_b_c_d)

#-----------------------------------------------------------------------
# DIVERSITY_ACTIONS macro test
#-----------------------------------------------------------------------

$(if $(_$(_SDE_CI)_DIVERSITIES),$(foreach div,$(_$(_SDE_CI)_DIVERSITIES),$(eval $(DIVERSITY_ACTIONS-$(div)))))

#-----------------------------------------------------------------------
# _TMDIVERSITY check macro ASSERT_DIVERSITY
#-----------------------------------------------------------------------

# Macro: ASSERT_DIVERSITY_TESTCASE
# Usage: $(call ASSERT_DIVERSITY_TESTCASE, true|false, ASSERT_DIVERSITY expression....)
ASSERT_DIVERSITY_TESTCASE=\
  $(warning \
    $(if $(filter $1,$(call ASSERT_DIVERSITY,$2,$3,$4,$5)),\
      PASSED,\
      **** **** FAILED !!!!!!!! **** ****))

# Macro: ASSERT_DIVERSITY_REPORT
# Usage: $(call ASSERT_DIVERSITY_REPORT, $(call ASSERT_DIVERSITY,....), <ASSERT_DIVERSITY_arg1>, <ASSERT_DIVERSITY_arg2>, ....)
ASSERT_DIVERSITY_REPORT=\
  $(if $(strip $1),\
    $(warning TRUE: $2 $3 $4 $5 (_TMDIVERSITY=$(_TMDIVERSITY)))true,\
    $(warning FALSE: $2 $3 $4 $5 (_TMDIVERSITY=$(_TMDIVERSITY)))false)

# Basic _SDE_CHECK_* macros tests
$(call ASSERT_TRUE,  $(call _SDE_CHECK_NUL_OF, a b, ))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_NUL_OF, a b, c))
$(call ASSERT_FALSE, $(call _SDE_CHECK_NUL_OF, a b, a))
$(call ASSERT_FALSE, $(call _SDE_CHECK_NUL_OF, a b, a b))
$(call ASSERT_FALSE, $(call _SDE_CHECK_NUL_OF, a b, a b c))

$(call ASSERT_TRUE,  $(call _SDE_CHECK_ONE_OF, a b, a))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ONE_OF, a b, b))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ONE_OF, a b, a b))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ONE_OF, a b, c))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ONE_OF, a b, a b c))

$(call ASSERT_FALSE, $(call _SDE_CHECK_ANY_OF, a b, ))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ANY_OF, a b, a))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ANY_OF, a b, b))
$(call ASSERT_TRUE , $(call _SDE_CHECK_ANY_OF, a b, a b))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ANY_OF, a b, c))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ANY_OF, a b, a c))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ANY_OF, a b, a b c))

$(call ASSERT_TRUE,  $(call _SDE_CHECK_ALL_OF, a b, ))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ALL_OF, a b, a))
$(call ASSERT_TRUE,  $(call _SDE_CHECK_ALL_OF, a b, b))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ALL_OF, a b, a b c))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ALL_OF, a b, c))
$(call ASSERT_FALSE, $(call _SDE_CHECK_ALL_OF, a b, a c))

## ASSERT_DIVERSITY tests

_TMDIVERSITY:=_a_b_c_d_

$(call ASSERT_DIVERSITY_TESTCASE, true , NUL_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, false, NUL_OF, a)

$(call ASSERT_DIVERSITY_TESTCASE, false, ONE_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ONE_OF, a)
$(call ASSERT_DIVERSITY_TESTCASE, true , ONE_OF, a 1)
$(call ASSERT_DIVERSITY_TESTCASE, false, ONE_OF, a b 1)

$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, 1)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a 1)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a b)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a b c)

$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, 1)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b c 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b c 1)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b c)

$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, 1,     REQUIRES_ONE_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a,     REQUIRES_ONE_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a,     REQUIRES_ONE_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a,     REQUIRES_ONE_OF, b)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a,     REQUIRES_ONE_OF, b 1)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a b 1, REQUIRES_ONE_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a b 1, REQUIRES_ONE_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a b 1, REQUIRES_ONE_OF, b)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a b 1, REQUIRES_ONE_OF, b 1)

$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, 1, REQUIRES_ANY_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, REQUIRES_ANY_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a, REQUIRES_ANY_OF, c d)

$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, 1, REQUIRES_ALL_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, REQUIRES_ALL_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, REQUIRES_ALL_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a, REQUIRES_ALL_OF, c d)

$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, 1, EXCLUDES_ANY_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, EXCLUDES_ANY_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, EXCLUDES_ANY_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, EXCLUDES_ANY_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a, EXCLUDES_ANY_OF, 1 2)

$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, 1, EXCLUDES_ALL_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, EXCLUDES_ALL_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a, EXCLUDES_ALL_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a, EXCLUDES_ALL_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, false, ANY_OF, a b, EXCLUDES_ALL_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, true , ANY_OF, a b, EXCLUDES_ALL_OF, 1 2)




$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, REQUIRES_ONE_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ONE_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ONE_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, REQUIRES_ONE_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, REQUIRES_ONE_OF, 1 c)

$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, REQUIRES_ANY_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ANY_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, REQUIRES_ANY_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, REQUIRES_ANY_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ANY_OF, 1 2)

$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, REQUIRES_ALL_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ALL_OF, 3 4)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, REQUIRES_ALL_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, REQUIRES_ALL_OF, c d)

$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, EXCLUDES_ANY_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, EXCLUDES_ANY_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, EXCLUDES_ANY_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, EXCLUDES_ANY_OF, 1 c)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, EXCLUDES_ANY_OF, 1 2)


$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, EXCLUDES_ALL_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, EXCLUDES_ALL_OF, c)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, 1 2, EXCLUDES_ALL_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, false, ALL_OF, a b, EXCLUDES_ALL_OF, c d)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, EXCLUDES_ALL_OF, 1 2)
$(call ASSERT_DIVERSITY_TESTCASE, true , ALL_OF, a b, EXCLUDES_ALL_OF, 1 c)
