#.      COPYRIGHT (c) 2009 NXP Semiconductors
# All Rights Reserved.
#
# Filename: linux/kmodule.mk
#
# Linux kernel mode/module building

# changes to this file force rebuild
_SDE_DEP_MAKEFILES +=   $(DIR_SDE)/$(_TMTGTOSCLASS)/kmodule.mk

include $(DIR_SDE)/kmodule_common.mk

#-----------------------------------------------------------------------

# Variable: KSRC
# Path to linux kernel. The default is $(_TMROOT)/linux/comps/$(_TMTGTOS)-$(_TMTGTOSVERSION).
KSRC ?= $(_TMROOT)/$(_TMTGTOSCLASS)/comps/$(_TMTGTOS)-$(_TMTGTOSVERSION)

# Compliance check:
ifneq "$(filter kmodule_generate_Kbuild_file,$(MAKECMDGOALS))" ""
  ifeq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"
    $(warning Variable KMODULE_INCLUDED_COMPS is not defined)
    $(warning Kernel module toplevel SDE2 component must define this even when empty)
    $(error Incorrect kernel module toplevel SDE2 component makefile)
  endif
endif

#-----------------------------------------------------------------------

# Function: KMODULE_KBUILD_GENERATED_FILE
# Returns path of a generated file for a SDE2 component,
# for including by a Kbuild file.
#
# Usage: $(call KMODULE_KBUILD_GENERATED_FILE,phCompName)
KMODULE_KBUILD_GENERATED_FILE=$(KMODULE_BUILD_DIR)/$1/Kbuild

kmodule_include: add_self_to_Kbuild_file

add_self_to_Kbuild_file: kmodule_init_Kbuild_file $(KMODULE_BUILD_TARGETS)
	$(_SDE_ECHO)mkdir -p $(KMODULE_BUILD_DIR)/$(_SDE_CI)
	$(_SDE_ECHO)cd $(KMODULE_BUILD_DIR)/$(_SDE_CI) && rm -f *.o && $(LB) xf $(call KMODULE_GET_LIB_PATH,$(_SDE_CI),$(call KMODULE_GET_LIB_NAME,$(_SDE_CI)))
	$(_SDE_ECHO)cd $(KMODULE_BUILD_DIR)/$(_SDE_CI) && { for f in $$($(LB) tf $(call KMODULE_GET_LIB_PATH,$(_SDE_CI),$(call KMODULE_GET_LIB_NAME,$(_SDE_CI)))); do \
		if ! cmp -s $$f $${f}_shipped; then cp -f $$f $${f}_shipped; fi; \
		echo "$(KMODULE_NAME)-objs += $(_SDE_CI)/$${f}_shipped"; \
	done } >>$(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI))

kmodule_init_Kbuild_file: FORCE
	@mkdir -p $(dir $(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)))
	@echo  >$(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)) ''
ifdef KMODULE_DEBUG_BUILD
	echo >>$(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)) '$$(warning Loading $(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)))'
endif

ifneq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"

#-----------------------------------------------------------------------
# WE ARE A TOPLEVEL COMPONENT REPRESENTING A KERNEL MODULE
#-----------------------------------------------------------------------


ifeq "$(findstring _kmodule_,$(_TMDIVERSITY))" "_kmodule_"
  clean:: kmodule_clean
  KBUILD_OBJ_KEY=m
else
  KBUILD_OBJ_KEY=y
endif

kmodule: kmodule_generate_Kbuild_file
ifeq ($(KERNELRELEASE),)
  # Outside linux kernel build system: call into it whilst specifying
  # with the M variable our $(KMODULE_BUILD_DIR) directory.
  KMODULE_LINUX_MAKE_OPTS = -C $(KSRC) M=$(KMODULE_BUILD_DIR) INSTALL_MOD_PATH=$(ROOTFS) $(KMODULE_GOALS) $(if $(CROSS_COMPILE),CROSS_COMPILE=$(CROSS_COMPILE))
  kmodule: kmodule_submake
  kmodule_submake: kmodule_generate_Kbuild_file
	$(_SDE_ECHO)echo include $(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)) >$(KMODULE_BUILD_DIR)/Kbuild
	$(_SDE_ECHO)$(MAKE) $(KMODULE_LINUX_MAKE_OPTS) modules modules_install
endif

kmodule_generate_Kbuild_file: add_self_to_Kbuild_file
kmodule_generate_Kbuild_file: $(patsubst %,%.add_comp_to_Kbuild_file,$(KMODULE_INCLUDED_COMPS))
kmodule_generate_Kbuild_file:
	$(_SDE_ECHO)echo >>$(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)) 'obj-$(KBUILD_OBJ_KEY) += $(KMODULE_NAME).o'

ifeq "$(strip $(BINARY_RELEASE_AVAILABILITY))" "true"
  kmodule_init_Kbuild_file: binary_release_available_message
  binary_release_available_message: configuration
else
  kmodule_init_Kbuild_file: lib
  lib:: configuration
endif
configuration:: $(if $(KMODULE_INCLUDED_COMPS),kmodule_included_comps)

%.add_comp_to_Kbuild_file: add_self_to_Kbuild_file
	$(_SDE_ECHO)mkdir -p $(KMODULE_BUILD_DIR)/$*
ifdef KMODULE_DEBUG_BUILD
	$(warning _SDE_GET_LIB_DIV($*)=$(call _SDE_GET_LIB_DIV,$*))
	$(warning KMODULE_GET_LIB_NAME($*)=$(call KMODULE_GET_LIB_NAME,$*))
	$(warning KMODULE_GET_LIB_PATH($*)=$(call KMODULE_GET_LIB_PATH,$*,$(call KMODULE_GET_LIB_NAME,$*)))
endif
	$(_SDE_ECHO)echo >>$(call KMODULE_KBUILD_GENERATED_FILE,$(_SDE_CI)) "include $(call KMODULE_KBUILD_GENERATED_FILE,$*)"; \

$(KMODULE_BUILD_DIR): configuration
	$(_SDE_ECHO)mkdir -p $(KMODULE_BUILD_DIR)

kmodule_clean: kmodule_clean_linux
kmodule_clean_linux:
	$(_SDE_ECHO)$(RM) -rf $(KMODULE_BUILD_DIR)/$(SDE_CI) $(call KMODULE_KBUILD_GENERATED_FILE,$(SDE_CI))




endif #ifeq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"

#-----------------------------------------------------------------------
# APPLICABLE TO ANYTHING BUILT IN KERNEL MODE
#-----------------------------------------------------------------------

KMODULE_MAKE_CMD_ARGS_ABSOLUTE_PATHS = $(shell echo ' $(subst ','"'"',$1)' |\
	perl -pe 's{(^|\s)(-I *|-include +)(?![/ ])}{$$1$$2$(KSRC)/}g;')
# " <= fixes editor syntax highlighting.

# Work out the exact options from a Linux kernel build.
# If we are called from within a Linux kernel build, then environment variables tell us:
ifneq ($(KERNELRELEASE),)
  KMODULE_COMMON_C_OPTIONS:=$(NOSTDINC_FLAGS) $(KBUILD_CPPFLAGS) $(KBUILD_CFLAGS) $(CPPFLAGS) $(CFLAGS)
  KMODULE_COMMON_A_OPTIONS:=$(NOSTDINC_FLAGS) $(KBUILD_CPPFLAGS) $(KBUILD_AFLAGS) $(CPPFLAGS) $(AFLAGS)
  ifeq "$(findstring _kmodule_,$(_TMDIVERSITY))" "_kmodule_"
    KMODULE_COMMON_C_OPTIONS+=$(CFLAGS_MODULE)
    KMODULE_COMMON_A_OPTIONS+=$(AFLAGS_MODULE)
  else
    KMODULE_COMMON_C_OPTIONS+=$(CFLAGS_KERNEL)
    KMODULE_COMMON_A_OPTIONS+=$(AFLAGS_KERNEL)
  endif
else # ifneq ($(KERNELRELEASE),)
  # Let's take the commands used for compiling Linux kernel/sched.c, and extract the relevant options.
  # TODO we should really take a file that is compiled as a module, if KMODULE=true.
  KMODULE_C_CMD_FILE=$(KSRC)/kernel/.sched.o.cmd
  KMODULE_A_CMD_FILE=$(KSRC)/arch/$(ARCH)/kernel/.head.o.cmd
  ifeq "$(wildcard $(KMODULE_C_CMD_FILE))" ""
    $(warning Linux $$(KSRC)/kernel/sched.c should have been compiled before we can build for kernel mode)
    $(error Cannot determine C compile flags for kernel mode)
  endif
  ifeq "$(wildcard $(KMODULE_A_CMD_FILE))" ""
    $(warning Linux $(KSRC)/arch/$(ARCH)/kernel/head.S should have been compiled before we can build for kernel mode)
    $(error Cannot determine assembler compile flags for kernel mode)
  endif
  KMODULE_COMMON_C_OPTIONS := $(shell perl -n -e 'next unless s{^cmd_kernel/sched.o.*?kernel/.sched.o.d (.*) -c .*}{$$1}; s{ -D"?KBUILD[^ ]*}{}g; print;' $(KMODULE_C_CMD_FILE))
  KMODULE_COMMON_A_OPTIONS := $(shell perl -n -e 'next unless s{^cmd_arch/$(ARCH)/kernel/head.o.*?kernel/.head.o.d (.*) -c .*}{$$1}; s{ -D"?KBUILD[^ ]*}{}g; print;' $(KMODULE_A_CMD_FILE))
  ifeq "$(findstring _kmodule_,$(_TMDIVERSITY))" "_kmodule_"
    # Modules don't want this:
    KMODULE_COMMON_C_OPTIONS := $(filter-out -fno-omit-frame-pointer,$(KMODULE_COMMON_C_OPTIONS))
    KMODULE_COMMON_A_OPTIONS := $(filter-out -fno-omit-frame-pointer,$(KMODULE_COMMON_A_OPTIONS))
    # In addition modules have this:
    KMODULE_COMMON_C_OPTIONS := $(KMODULE_COMMON_C_OPTIONS) -DMODULE -mlong-calls
    KMODULE_COMMON_A_OPTIONS := $(KMODULE_COMMON_A_OPTIONS) -DMODULE -mlong-calls
  endif
endif # ifneq ($(KERNELRELEASE),)

# Make relative paths absolute by adding $KSRC:
KMODULE_COMMON_C_OPTIONS := $(call KMODULE_MAKE_CMD_ARGS_ABSOLUTE_PATHS,$(KMODULE_COMMON_C_OPTIONS))
KMODULE_COMMON_A_OPTIONS := $(call KMODULE_MAKE_CMD_ARGS_ABSOLUTE_PATHS,$(KMODULE_COMMON_A_OPTIONS))

# Use the standard warning flags defined by the Linux kernel:
# otherwise SDE2 (especially with _TMTGTWARNINGS=3) adds some warning flags, which cause
# even the standard Linux kernel header files to give too many warnings.
# -Wpointer-arith is useful to find porting problems:
# MS compiler can't do ptr arithmetic on void*.
_SDE_WARNING_LEVEL = \
  -Wpointer-arith

# Ignore unknown GCC pragmas, we add them for QAC:
KMODULE_COMMON_C_OPTIONS += -Wno-unknown-pragmas

# To ease porting, we add this:
# (so that #include <string.h> works, and we don't have
#  to say  #include <linux/string.h>)
KMODULE_COMMON_C_OPTIONS += -I$(KSRC)/include/linux
KMODULE_COMMON_A_OPTIONS += -I$(KSRC)/include/linux

_SDE_KERNEL_OR_USER_MODE_C_OPTIONS += $(KMODULE_COMMON_C_OPTIONS)
_SDE_KERNEL_OR_USER_MODE_A_OPTIONS += $(KMODULE_COMMON_A_OPTIONS)
_SDE_KERNEL_OR_USER_MODE_C_OPTIONS += -D"KBUILD_STR(s)=\#s" -D"KBUILD_BASENAME=KBUILD_STR($(notdir $(<:.c=)))" -D"KBUILD_MODNAME=KBUILD_STR($(KMODULE_NAME))"
_SDE_KERNEL_OR_USER_MODE_A_OPTIONS += -D"KBUILD_STR(s)=\#s" -D"KBUILD_BASENAME=KBUILD_STR($(notdir $(<:.S=)))" -D"KBUILD_MODNAME=KBUILD_STR($(KMODULE_NAME))"

#-----------------------------------------------------------------------
# EXPORTS processing
#-----------------------------------------------------------------------

# We check EXPORTS_SAVED instead of EXPORTS since we have set EXPORTS to empty
# in makelib_ext0.mk, after copying it into EXPORTS_SAVED.
# See makelib_ext0.mk for the reason.

# Only generate _export_symbols_generated.c file if we are exporting something.
ifneq "$(strip $(filter-out true,$(EXPORTS_SAVED)))" ""
  KMODULE_EXPORT_SYMBOLS_CFILE = $(DIR_INTERM)/_export_symbols_generated.c
  KMODULE_EXPORT_SYMBOLS_OFILE = $(KMODULE_EXPORT_SYMBOLS_CFILE:.c=.$(_SDE_O))
  _SDE_OBJECTS += $(KMODULE_EXPORT_SYMBOLS_OFILE)
  $(KMODULE_EXPORT_SYMBOLS_OFILE) : $(KMODULE_EXPORT_SYMBOLS_CFILE)
	@$(ECHO) compiling $(notdir $<)
	$(_SDE_ECHO)$(CC) $(_SDE_COPTS) $(TARGET_CFLAGS) -c -o $@ $<
  $(_SDE_THISLIB): $(KMODULE_EXPORT_SYMBOLS_OFILE)

  ifdef KMODULE_DEBUG_BUILD
    $(warning 1 NOSTDINC_FLAGS = $(NOSTDINC_FLAGS))
    $(warning 1 CFLAGS = $(CFLAGS))
    $(warning 1 KMODULE_COMMON_C_OPTIONS = $(KMODULE_COMMON_C_OPTIONS))
    $(shell export | sort >~/tmp/exports.$$$$)
    $(warning See file ~/tmp/exports.$$$$)
  endif

  $(KMODULE_EXPORT_SYMBOLS_CFILE): $(_SDE_DEP_MAKEFILES)
  ifdef KMODULE_DEBUG_BUILD
	$(warning 2 NOSTDINC_FLAGS = $(NOSTDINC_FLAGS))
	$(warning 2 CFLAGS = $(CFLAGS))
	$(warning 2 KMODULE_COMMON_C_OPTIONS = $(KMODULE_COMMON_C_OPTIONS))
  endif
	$(_SDE_ECHO)$(MKDIR) $(dir $(KMODULE_EXPORT_SYMBOLS_CFILE))
	$(_SDE_ECHO)$(ECHO) '#include <linux/module.h>' >$(KMODULE_EXPORT_SYMBOLS_CFILE)
	$(_SDE_ECHO)for sym in $(EXPORTS_SAVED); do $(ECHO) "extern void *$$sym; EXPORT_SYMBOL($$sym);" >>$(KMODULE_EXPORT_SYMBOLS_CFILE); done
endif # ifneq "$(strip $(filter-out true,$(EXPORTS_SAVED)))" ""

#-----------------------------------------------------------------------
# Manifest (MODULE_INFO) Handling
#-----------------------------------------------------------------------

# Only generate manifest using MODULE_INFO) info if we are building
# the toplevel kernel module component:
ifneq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"
  KMODULE_MANIFEST_CFILE = $(DIR_INTERM)/_manifest_generated.c
  KMODULE_MANIFEST_OFILE = $(KMODULE_MANIFEST_CFILE:.c=.$(_SDE_O))
  _SDE_OBJECTS += $(KMODULE_MANIFEST_OFILE)
  $(KMODULE_MANIFEST_OFILE) : $(KMODULE_MANIFEST_CFILE)
	@$(ECHO) compiling $(notdir $<)
	$(_SDE_ECHO)$(CC) $(_SDE_COPTS) $(TARGET_CFLAGS) -c -o $@ $<
  $(_SDE_THISLIB): $(KMODULE_MANIFEST_OFILE)

  include $(PRJROOT)/etc/setup_productversion.mk
  $(KMODULE_MANIFEST_CFILE): $(_SDE_DEP_MAKEFILES)
	$(_SDE_ECHO)$(MKDIR) $(dir $(KMODULE_MANIFEST_CFILE))
	$(_SDE_ECHO)$(ECHO) '#include <linux/module.h>' >$(KMODULE_MANIFEST_CFILE)
	$(_SDE_ECHO)$(ECHO) 'MODULE_INFO(productversion, "$(PRODUCTVERSION)");' >>$(KMODULE_MANIFEST_CFILE);\
	$(ECHO) 'MODULE_INFO(builddate, __DATE__);' >>$(KMODULE_MANIFEST_CFILE);\
	$(ECHO) 'MODULE_INFO(buildtime, __TIME__);' >>$(KMODULE_MANIFEST_CFILE);\
	$(ECHO) 'MODULE_INFO(builduser, "$(firstword $(LOGNAME) $(USER))");' >>$(KMODULE_MANIFEST_CFILE);\
	$(ECHO) 'MODULE_INFO(buildhost, "$(shell hostname)");' >>$(KMODULE_MANIFEST_CFILE)
endif # ifneq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"
