#.      COPYRIGHT (c) 2009 NXP Semiconductors
# All Rights Reserved.
#
# Filename: makelib_ext.mk
#
# Included from the end of makelib.mk

# changes to this file force rebuild
_SDE_DEP_MAKEFILES +=   $(DIR_SDE)/$(_TMTGTOSCLASS)/makelib_ext.mk

ifeq "$(TMFL_MODE_KERNEL)" "1"
  include $(DIR_SDE)/linux/kmodule.mk
endif


# Define: _SDE_THISLIB_SUPPLIED
# This is $(_SDE_DIR_LIB_EXT)/$(notdir $(_SDE_THISLIB))
# that sits in the component's lib directory.
_SDE_THISLIB_SUPPLIED = $(_SDE_DIR_LIB_EXT)/$(notdir $(_SDE_THISLIB))

# Define: _SDE_THISDLL_SUPPLIED
# This is $(_SDE_DIR_DLL_EXT)/$(notdir $(_SDE_THISDLL))
# that sits in the component's lib directory.
_SDE_THISDLL_SUPPLIED = $(_SDE_DIR_LIB_EXT)/$(_SDE_DIVERSITY)/$(notdir $(_SDE_THISDLL))

# Define: _SDE_THISDLL_SUPPLIED
# This is $(_SDE_DIR_DLL_EXT)/$(notdir $(_SDE_THISDLL))
# that sits in the component's lib directory but has no diversity applied.
_SDE_THISDLL_SUPPLIED2 = $(_SDE_DIR_LIB_EXT)/all-diversities/$(notdir $(_SDE_THISDLL))

# Define: _SDE_THISLIB_ACTUAL
# This is the first file that exists of the following files:-
# - $(_SDE_THISLIB)
# - $(_SDE_THISLIB_SUPPLIED)
_SDE_THISLIB_ACTUAL = $(firstword $(shell ls 2>/dev/null $(_SDE_THISLIB) $(_SDE_THISLIB_SUPPLIED)) _SDE_THISLIB_ACTUAL__no_match_error)

# Define: _SDE_THISDLL_ACTUAL
# This is the first file that exists of the following files:-
# - $(_SDE_THISDLL)
# - $(_SDE_THISDLL_SUPPLIED)
_SDE_THISDLL_ACTUAL = $(firstword $(shell ls 2>/dev/null $(_SDE_THISDLL) $(_SDE_THISDLL_SUPPLIED) $(_SDE_THISDLL_SUPPLIED2)) _SDE_THISDLL_ACTUAL__no_match_error)

# Define: _SDE_LIB_INSTALL_TARGETNAME
# Name of installed library in either <STAGING_INSTALL_DIR>/lib
# or <INSTALL_DIR>/lib.
# SDE2 produces library names that have a diversity
# and a release type suffix (e.g. _g _a _t).
# However, this is inconvienient when installing a library,
# so _SDE_LIB_INSTALL_TARGETNAME is the library name without
# those suffixes.
_SDE_LIB_INSTALL_TARGETNAME=lib$(TARGET).$(_SDE_LIB_EXTN)

# Define: INSTALL_DIR_BUILD
# This is ${<STAGING_INSTALL_DIR>} if it is defined (not an empty string),
# otherwise it is ${<INSTALL_DIR>}.
INSTALL_DIR_BUILD = $(firstword $(STAGING_INSTALL_DIR) $(INSTALL_DIR))

# If we are a sub-component using the SDE_IN_SDE approach,
# then our object files are included in the parent component's library,
# and thus we do have to install our libraries:
ifneq "$(SDE_IN_SDE)" "."
  _SDE_NO_DEFAULT_INSTALL=1
endif

ifndef _SDE_NO_DEFAULT_INSTALL

# Target: install
# If <KERNEL_MODE> is true, installs the kernel module (see <kmodule_install>).
# Otherwise:
# Installs the library into the root file system:-
# - installs component's include files in $(<INSTALL_DIR>)/include
# - installs $(_SDE_THISLIB_ACTUAL) in $(<INSTALL_DIR_BUILD>)/lib
# - installs $(_SDE_THISDLL_ACTUAL) in $(<INSTALL_DIR>)/lib
  ifeq "$(TMFL_MODE_USER)" "1"
    install: install_lib

#-----------------------------------------------------------------------
# Headers
#-----------------------------------------------------------------------
# Define: INSTALLED_HEADERS
# List of all header files of the component once it is installed
# into directory ${<INSTALL_DIR_BUILD>}/include.
INSTALLED_HEADERS = $(foreach f,$(wildcard inc/*.h),\
  $(INSTALL_DIR_BUILD)/include/$(notdir $f))

install_lib: install_lib_headers
install_lib_headers: $(INSTALLED_HEADERS)
.PHONY: install_lib_headers
_sde_install_lib_headers_first_time=true
$(INSTALL_DIR_BUILD)/include/%.h: inc/%.h
	$(if $(_sde_install_lib_headers_first_time),@echo '  INSTALL inc/*.h -> $(dir $@)')
	$(if $(_sde_install_lib_headers_first_time),$(_SDE_ECHO)mkdir -p $(dir $@))
	$(_SDE_ECHO)cp -Lf $< $@
	$(if $(_sde_install_lib_headers_first_time),$(eval _sde_install_lib_headers_first_time=))
clean::
	$(_SDE_ECHO)rm -f $(INSTALLED_HEADERS)


#-----------------------------------------------------------------------
# Library
#-----------------------------------------------------------------------
INSTALLED_LIB = $(INSTALL_DIR_BUILD)/lib/$(_SDE_LIB_INSTALL_TARGETNAME)
install_lib: install_lib_library
install_lib_library: $(INSTALLED_LIB)
.PHONY: install_lib_library
$(INSTALLED_LIB): FORCE
	$(if $(shell cmp $(_SDE_THISLIB_ACTUAL) $@ 2>&1),\
		$(eval _sde_lib_needs_install:=true)\
		$(eval _sde_lib_version_used:=$(if $(filter $(_SDE_THISLIB),$(_SDE_THISLIB_ACTUAL)),built,pre-built)))
	$(if $(_sde_lib_needs_install),@echo '  INSTALL ($(_sde_lib_version_used)) $(notdir $(_SDE_THISLIB_ACTUAL)) -> $@')
	$(if $(_sde_lib_needs_install),$(_SDE_ECHO)mkdir -p $(dir $@))
	$(if $(_sde_lib_needs_install),$(_SDE_ECHO)cp -Lf $(_SDE_THISLIB_ACTUAL) $@)
clean::
	$(_SDE_ECHO)rm -f $(INSTALLED_LIB)

#-----------------------------------------------------------------------
# DLL?
#-----------------------------------------------------------------------
ifeq ($(EXPORTS),true)
  install_lib: install_lib_dll
  INSTALLED_DLL = $(INSTALL_DIR)/lib/$(notdir $(_SDE_THISDLL))
  install_lib_dll: $(INSTALLED_DLL)
  .PHONY: install_lib_dll
  $(INSTALLED_DLL): FORCE
	$(if $(shell cmp $(_SDE_THISDLL_ACTUAL) $@ 2>&1),\
		$(eval _sde_dll_needs_install:=true)\
		$(eval _sde_dll_version_used:=$(if $(filter $(_SDE_THISDLL),$(_SDE_THISDLL_ACTUAL)),built,pre-built)))
	$(if $(_sde_dll_needs_install),@echo '  INSTALL ($(_sde_dll_version_used)) $(notdir $(_SDE_THISDLL_ACTUAL)) -> $@')
	$(if $(_sde_dll_needs_install),$(_SDE_ECHO)mkdir -p $(dir $@))
	$(if $(_sde_dll_needs_install),$(_SDE_ECHO)cp -Lf $(_SDE_THISDLL_ACTUAL) $@)

  # We want to install the library in staging too so that the debugger can find the unstripped version even 
  # when we've stripped the one in rootfs. MAC 2010/03/02
  install_lib : install_staging_lib_dll
  STAGING_INSTALLED_DLL = $(if $(STAGING_INSTALL_DIR),$(STAGING_INSTALL_DIR)/lib/$(notdir $(_SDE_THISDLL)))
  install_staging_lib_dll: $(STAGING_INSTALLED_DLL)
  .PHONY: install_staging_lib_dll
  $(STAGING_INSTALLED_DLL): FORCE
	$(if $(shell cmp $(_SDE_THISDLL_ACTUAL) $@ 2>&1),\
		$(eval _sde_staging_dll_needs_install:=true)\
		$(eval _sde_staging_dll_version_used:=$(if $(filter $(_SDE_THISDLL),$(_SDE_THISDLL_ACTUAL)),built,pre-built)))
	$(if $(_sde_staging_dll_needs_install),@echo '  INSTALL ($(_sde_staging_dll_version_used)) $(notdir $(_SDE_THISDLL_ACTUAL)) -> $@')
	$(if $(_sde_staging_dll_needs_install),$(_SDE_ECHO)mkdir -p $(dir $@))
	$(if $(_sde_staging_dll_needs_install),$(_SDE_ECHO)cp -Lf $(_SDE_THISDLL_ACTUAL) $@)

  clean::
	$(_SDE_ECHO)rm -f $(INSTALLED_DLL)
	$(_SDE_ECHO)rm -f $(STAGING_INSTALLED_DLL)
endif # ifeq ($(EXPORTS),true)

    all: install
    .PHONY: install
  endif # ifeq "$(TMFL_MODE_USER)" "1"

endif # ifndef _SDE_NO_DEFAULT_INSTALL

