# Author: Cyril Germond, (c) NXP Semiconductors , 2006
# This script is added as a undocumented feature by request from the author.
#----------------------------------------------------------------------------
# Project  : Darwin-1
#----------------------------------------------------------------------------
# File     : $RCSfile: bin2rom.pl,v $
# Revision : $Revision: 1.1 $
# Date     : $Date: 2002/04/09 12:40:40 $
# Author   : $Author: castanha $
#
#-Description----------------------------------------------------------------
# 
# This programs generates the ROM image to be loaded in the memory models.
# 
#----------------------------------------------------------------------------

if    ( $#ARGV < 1 ) {
  &usage;
}

if( $ARGV[2] ) {
 $BINFILE=$ARGV[2];
} else {
 $BINFILE=$STDIN;
}

if( $ARGV[3] ) {
 $ROMFILE=$ARGV[3];
} else {
 $ROMFILE=$STDOUT;
}

if( $ARGV[0] == 8 ) {
  $rom_width = 1;
} elsif ( $ARGV[0] == 16 ) {
  $rom_width = 2;
} elsif ( $ARGV[0] == 32 ) {
  $rom_width = 4;
} else {
  &usage;
}

if( $ARGV[1] == 8 ) {
  $bus_width = 1;
} elsif ( $ARGV[1] == 16 ) {
  $bus_width = 2;
} elsif ( $ARGV[1] == 32 ) {
  $bus_width = 4;
} else {
  &usage;
}

if( $ARGV[0] > $ARGV[1] ) {
  &usage;
}

if( $ARGV[0] != $ARGV[1] ) {
  print "<memory_width <> bus_width not supported yet ...\n";
  exit;
}

##############################################################################
## Read .hex file
##############################################################################


##############################################################################
## Output ROM image file(s)
##############################################################################

if( $bus_width == $rom_width ) {
  open( HEXOUT, ">" . $ROMFILE ) || die "Error: Can't open file: $ROMFILE\n";

  open( BININ, $BINFILE ) || die "Error: Can't open file: $BINFILE\n";
  $dr = $rom_width*2;
  $st = "%0${dr}x";
  
  while(! eof BININ) {
   for( $i = 0; $i < $bus_width; $i+= 1 ) {
    @k[$i]= ord(getc BININ);
   }
   for( $i = 0; $i < $bus_width; $i+= $rom_width ) {
    for( $j = $rom_width-1 ; $j >= 0; $j-= 1 ) {
     printf HEXOUT "%02x",@k[$i+$j];
    }
   }
   printf HEXOUT "\n";
  }
  close( BININ );
  close( HEXOUT );
}

exit;

##############################################################################
## Usage
##############################################################################

sub usage {
  print( "\n" );
  print( "Generates ROM images suitable for Verilog simulation from a binary file\n" );
  print( "according to the bus and ROM size.\n" );
  print( "\n" );
  print( "Usage: $0 <memory_width>  <bus_width> <binary_file> <dest_file>\n " );
  print( "\n" );
  print( "  Parameters :\n" );
  print( "    <memory_width> : memory width in bits: [8|16|32]\n" );
  print( "    <bus_width>    : bus    width in bits: [8|16|32]\n" );
  print( "    with always <memory_width> <= <bus_width>\n" );
  print( "\n" );
  print( "  <binary_file> : source file generated by fromelf or arm-elf-objcopy.\n" );
  print( "\n" );
  print( "  <dest_file> : output file(s). If <memory width> < <bus width>,\n" );
  print( "     then multiple files are generated with always <dest_file>.0 = LSB.\n" );
  print( "     Generated format:\n" );
  print( "     0c\n" );
  print( "     00\n" );
  print( "     00\n" );
  print( "     ea\n" );
  print( "\n" );
  print( "  Example\n" );
  print( "    8 16 => 2 x 8 bits memories: (MSB) xxx.1 xxx.0 (LSB)\n" );
  print( "    8 32 => 4 x 8 bits memories: (MSB) xxx.3 xxx.2 xxx.1 xxx.0 (LSB)\n" );
  print( "\n" );
  exit 0;
}

## end of file ###############################################################
