use File::Path;
use IO::Handle;

###
# File makefile_template.pl
$Version          = "0.5";

# author: Shivaraj P (SP)
#         Roopa Mahishi (RM)
#
# Revision history:
# -----------|---------|----------------------------------------------------------
# date       | version | Description 
# -----------|---------|----------------------------------------------------------
# 2003-10-29 | 0.1, SP | Initial Version 
# -----------|---------|----------------------------------------------------------
# 2003-10-29 | 0.2, SP | Disply Usage correction
# -----------|---------|----------------------------------------------------------
# 2003-10-29 | 0.3, SP | Improve O/P format
# -----------|---------|----------------------------------------------------------
# 2003-06-05 | 0.4, SP | COrrect notdir function call
# -----------|---------|----------------------------------------------------------
# 2005-10-05 | 0.5, RM | Improve code quality of exisiting code
# -----------|---------|----------------------------------------------------------


$Logfile          = "makefile_template.log";
$TemplateMakeFile = "makefile"; # file with possible configurations in this project (containing wildcards)
$format           = '-' x80;
$format1          = '#' x80;

#
#	Sub-routines 
#

sub DisplayUsage
{

    my $string = qq (\nUsage :\n$0
$format\n
This script produces makefile template for editing application makefiles and to build applications outside SDE2
The makefile template is produced for the current configurations set in the environment
The following configurations are supported:
x86_nt, tm_psos, x86_ce, arm_ce, mips_ce, x86_vxworks, arm_vxworks, arm_nullos, hp_nullos\n
$format\n);
   print $string;
}
#------------------------------------------------------------------------------------
sub CheckEnvironment
{
    my $errors = 0;
    if( !exists $ENV {'_TMROOT'} )
    {
        print  "Environment variable \'_TMROOT\' not defined\n";
        $errors += 1;
    }
    if( !exists $ENV {'_TMREPORTS'} )
    {
        print "Environment variable \'_TMREPORTS\' not defined\n";
        $errors += 1;
    }
    if( !exists $ENV {'MAKE'} )
    {
        print "Environment variable \'MAKE\' not defined\n";
        $errors += 1;
    }
    exit if( $errors > 0 ) ;
}
#---------------------------------------------------------------------------------------------
sub CheckParameters
{
    for (@ARGV) {
        if( /^\-h/ )
        { # help
            DisplayUsage();
            exit 0;
        }
    }
}

sub GetValueFromAnyMakefile{
    my $string = shift;
    open(MAKEOUT, "$ENV{'MAKE'} -C $CompList -f makefile var=$string _sde_print_var |") or die "Can't start $ENV{'MAKE'}\n";
    while( defined($line = <MAKEOUT>) )
    {
        chomp $line;
        if( $line =~ /^\s*${string}\s*=\s*/ ) { $line =~ s/^\s*${string}\s*=\s*//;
        $line                                      =~ s/\s*$//g;
        $line                                      =~ s/\s+/ /g; # replace more spaces by one space
        $value                                     = $line; }
    }
    close MAKEOUT;
    return $value;
}

sub WriteTargetAll{
my $string = qq(#-----------------------------------------------------------------------------------------------------
# Define the required target that you need to build
# Choose either \$\(LIBRARY_FILE\) or \$\(EXE_FILE\) for building library or executable respectively\n
#------------------------------------------------------------------------------------------------------
all: \$\(OBJECT_FILES\) \$\(LIBRARY_FILE\) \$\(EXE_FILE\)\n\n);

print MAKEFILE $string;

}

sub Create_make_x86_nt
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");
    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.lib\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.exe\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");


    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = cl.exe\n";
    print MAKEFILE "CXX           = cl.exe\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = lib.exe\n";
    print MAKEFILE "AS            =\n";
    print MAKEFILE "LD            = link.exe\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CXX\) \$\(sde_cxxopts\) -c -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fo\$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) \/nologo \/verbose /out:\$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> -LIBPATH:\$\(LIBS_LOCATION\) <add other libs location if any> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) /out:\$@ \n\n";

    WriteTargetAll();
}
################################################################################################
sub Create_make_tm_psos
{
    print MAKEFILE "#--------------------------------------------\n";
    print MAKEFILE "# $configuration specific files\n";
    print MAKEFILE "#--------------------------------------------\n";
    print MAKEFILE "PSOS_SOURCES     = \$(SRC_LOCATION)/<PSOS specific source files>\n";
    print MAKEFILE "PSOS_FILES       = \$\(notdir \$(PSOS_SOURCES\)\)\n\n";
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");

    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
     }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n";

    print MAKEFILE "PSOS_OBJECTS =		\$\(PSOS_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.a \n\n";
    print MAKEFILE "PSOS_LIBRARY_FILE = \$(LIBS_LOCATION)\/bsp.a\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.out\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "TCS           = <Enter TCS location>\n";
    print MAKEFILE "CC            = \$\(TCS\)\/bin\/tmcc\n";
    print MAKEFILE "CXX           = \$\(TCS\)\/bin\/tmcpp\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = \$\(TCS\)\/bin\/tmar\n";
    print MAKEFILE "AS            =\$\(TCS\)\/bin\/tmas\n";
    print MAKEFILE "LD            = \$\(TCS\)\/bin\/tmld\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CXX\) \$\(sde_cxxopts\) -c -o \$@ \$	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) -rv -o \$@ \$\(OBJECT_FILES)  \n\n";

    print MAKEFILE "\$\(PSOS_LIBRARY_FILE\): \$\(PSOS_OBJECTS\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) -rv -o \$@ \$\(PSOS_OBJECTS)  \n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(CC\) \$\(OBJECT_FILES\) <Other Object Files> -L\$\(LIBS_LOCATION\) -L<add other libs location if any> <Add required DLLs and Libraries in correct order with -l suffix and without extension> \\\n";
    print MAKEFILE "		\$\(PSOS_LIBRARY_FILE\) \$\(sde_ldopts\) -o \$@ \n\n";

    WriteTargetAll();
}
######################################################################################

sub Create_make_x86_ce
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");

    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");


    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.lib\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.exe\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = cl\n";
    print MAKEFILE "CXX           = cl\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = link -lib\n";
    print MAKEFILE "AS            = cl\n";
    print MAKEFILE "LD            = link\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CXX\) \$\(sde_cxxopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) \/nologo /out:\$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "# Make sure to include all LIBPATHS and required libraries from the WinCE environment\n\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> -LIBPATH:\$\(LIBS_LOCATION\) <add other libs location if any> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) -out:\$@ \n\n";

    WriteTargetAll();
}
################################################################################################

sub Create_make_arm_ce
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");

    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.lib\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.exe\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = clarm\n";
    print MAKEFILE "CXX           = clarm\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = link -lib\n";
    print MAKEFILE "AS            = clarm\n";
    print MAKEFILE "LD            = link\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CXX\) \$\(sde_cxxopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) \/nologo /out:\$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "# Make sure to include all LIBPATHS and required libraries from the WinCE environment\n\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> -LIBPATH:\$\(LIBS_LOCATION\) <add other libs location if any> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) -out:\$@ \n\n";

    WriteTargetAll();
}
################################################################################################
sub Create_make_mips_ce
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");

    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.lib\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.exe\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");


    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = clmips\n";
    print MAKEFILE "CXX           = clmips\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = link -lib\n";
    print MAKEFILE "AS            = clmips\n";
    print MAKEFILE "LD            = link\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CXX\) \$\(sde_cxxopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -Fd\$\(@\:.$sde_o = _pdb.pdb) -Fo\$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) \/nologo /out:\$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "# Make sure to include all LIBPATHS and required libraries from the WinCE environment\n\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> -LIBPATH:\$\(LIBS_LOCATION\) <add other libs location if any> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) -out:\$@ \n\n";

    WriteTargetAll();
}
################################################################################################
sub Create_make_x86_vxworks
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");
   if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
       $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
   }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.a\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.out\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = ccsimpc\n";
    print MAKEFILE "CXX           = c++simpc\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = arsimpc\n";
    print MAKEFILE "AS            = ccsimpc\n";
    print MAKEFILE "LD            = ccsimpc\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) rsv \$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(sde_ldopts\) -o \$@ \$\(OBJECT_FILES\) <Other Object Files> -L\$\(LIBS_LOCATION\) <add other libs location if any> <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		  >\$\(basename \$@\).map \n\n";

    WriteTargetAll();
}

sub Create_make_arm_vxworks
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");
    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.a\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.out\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = ccarm\n";
    print MAKEFILE "CXX           = c++arm\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = ararm\n";
    print MAKEFILE "AS            = asarm\n";
    print MAKEFILE "LD            = ldarm\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) crus \$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(sde_ldopts\) -o \$@ \$\(OBJECT_FILES\) <Other Object Files> -L\$\(LIBS_LOCATION\) <add other libs location if any> <Add other DLLs and Libraries in correct order> \\\n";
    print MAKEFILE "		  >\$\(basename \$@\).map \n\n";

    WriteTargetAll();
}
#############################################################################################################
sub Create_make_arm_nullos
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");
    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.a\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.out\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    #------------------------------------------------------------------------------------
    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    # -----------------------------------------------------------------------------------
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = arm-elf-gcc\n";
    print MAKEFILE "CXX           = arm-elf-g++\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = arm-elf-ar\n";
    print MAKEFILE "AS            = arm-elf-gcc\n";
    print MAKEFILE "LD            = arm-elf-ld\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) -r \$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> -L\$\(LIBS_LOCATION\) <add other libs location if any>  \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) -o \$@ \n\n";

    WriteTargetAll();
}
#########################################################################################
sub Create_make_hp_nullos
{
    system("$ENV{MAKE} -f $ENV{_TMROOT}/sde/environment.mk makelist");
    if( !exists $ENV {'_TMTGTBUILDROOT'} ) { 
        $LocList_File = "$ENV{'_TMROOT'}/project/loc_list.mk"; 
    }
    else { 
        $LocList_File = "$ENV{'_TMTGTBUILDROOT'}/project/loc_list.mk"; 
    }
    open(LOCLIST, "$LocList_File") or die "Can't open loc_list.mk file to get components\n";
    while( $line = <LOCLIST> )
    {
        chomp $line;
        if( ($line !~ /^\#/) && ($line !~ /intfs/) )
        {
            if( $line =~ /:= / )
            {
                if( !$CompList ) { $CompList = $'; }
            }
        }
    }

    $sde_o = GetValueFromAnyMakefile("_SDE_O");

    print MAKEFILE "#-------------------------------------------------------------------------------\n";
    print MAKEFILE "# Definition of object files\n";
    print MAKEFILE "#-------------------------------------------------------------------------------\n";

    print MAKEFILE "OBJECT_FILES +=                                                         \\\n";
    print MAKEFILE "				\$\(CXX_FILES:%.cpp=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "				\$\(C_FILES:%.c=\$\(OBJ_LOCATION\)/%.$sde_o\)              \\\n";
    print MAKEFILE "				\$\(filter                                                       \\\n";
    print MAKEFILE "				    %.$sde_o,                                              \\\n";
    print MAKEFILE "				    \$\(S_FILES:%.s=\$\(OBJ_LOCATION\)\/%.$sde_o\)         \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.S=\$\(OBJ_LOCATION\)\/%.$sde_o\)          \\\n";
    print MAKEFILE "                                    \$\(S_FILES:%.asm=\$\(OBJ_LOCATION\)\/%.$sde_o\)        \\\n";
    print MAKEFILE "                                )																	\n\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Library file\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "LIBRARY_FILE = \$(LIBS_LOCATION)\/<Enter the library name>.a\n\n";
    print MAKEFILE "#----------------------------------------------------------------\n";
    print MAKEFILE "# Definition of executable file\n";
    print MAKEFILE "#-----------------------------------------------------------------\n";
    print MAKEFILE "EXE_FILE = \$(EXE_LOCATION)\/<Enter the executable name>.out\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_COPTS
    #-----------------------------------------------------------------------------------------
    $sde_copts = GetValueFromAnyMakefile("_SDE_COPTS");

    $sde_copts =~ s/\s-Fd\S*\s/ /;
    $sde_copts =~ s/-I\S*/  /g;
    $sde_copts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_copts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_CXXOPTS
    #-----------------------------------------------------------------------------------------
    $sde_cxxopts = GetValueFromAnyMakefile("_SDE_CXXOPTS");

    $sde_cxxopts =~ s/\s-Fd\S*\s/ /;
    $sde_cxxopts =~ s/-I\S*/  /g;
    $sde_cxxopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_cxxopts =~ s/\s{2}//g;

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_AOPTS
    #-----------------------------------------------------------------------------------------
    $sde_aopts = GetValueFromAnyMakefile("_SDE_AOPTS");

    $sde_aopts =~ s/\s-Fd\S*\s/ /;
    $sde_aopts =~ s/-I\S*/  /g;
    $sde_aopts =~ s/\s{2}/ -I. -I\$(SRC_LOCATION) -I\$(INC_LOCATION) -I<Add other include Paths Here>  /;
    $sde_aopts =~ s/\s{2}//g;
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Compiler options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_copts := $sde_copts\n";
    print MAKEFILE "sde_cxxopts := $sde_cxxopts\n";
    print MAKEFILE "sde_aopts := $sde_aopts\n\n";

    #----------------------------------------------------------------------------------------
    # Get the value of _SDE_LDOPTS
    #-----------------------------------------------------------------------------------------
    $sde_ldopts = GetValueFromAnyMakefile("_SDE_LDOPTS");

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Definition of Linker options\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "sde_ldopts := $sde_ldopts\n\n";

    print MAKEFILE "VPATH = \$\(SRC_LOCATION\) \$\(OBJ_LOCATION\) \$\(LIBS_LOCATION\)\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Compiler/Linker toolchains\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "CC            = gcc\n";
    print MAKEFILE "CXX           = g++\n";
    print MAKEFILE "IDL           =\n";
    print MAKEFILE "LB            = ar\n";
    print MAKEFILE "AS            = gcc\n";
    print MAKEFILE "LD            = ld\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating object files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.c \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;                \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.cpp \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;               \\\n";
    print MAKEFILE "	\$\(CC\) \$\(sde_copts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.s \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;              \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "\$\(OBJ_LOCATION\)\/%.$sde_o: %.asm \n";
    print MAKEFILE "	mkdir -p \$\(dir \$@\);   \\\n";
    print MAKEFILE "	echo compiling \$<;             \\\n";
    print MAKEFILE "	\$\(AS\) \$\(sde_aopts\) -c -o \$@ \$<	\n\n";

    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Library files\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(LIBRARY_FILE\): \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(LIBS_LOCATION);   \\\n";
    print MAKEFILE "	echo building library \$@;             \\\n";
    print MAKEFILE "	\$\(LB\) -r \$@ \$\(OBJECT_FILES)  \n\n";
    print MAKEFILE "#--------------------------------------------------------------\n";
    print MAKEFILE "# Rules for generating Executable\n";
    print MAKEFILE "#---------------------------------------------------------------\n";
    print MAKEFILE "\$\(EXE_FILE\): \$\(LIBRARY_FILE\) \$\(OBJECT_FILES\)\n";
    print MAKEFILE "	mkdir -p \$\(EXE_LOCATION);   \\\n";
    print MAKEFILE "	echo building executable \$@;             \\\n";
    print MAKEFILE "	\$\(LD\) \$\(OBJECT_FILES\) <Other Object Files> \$\(LIBRARY_FILE\) <Add other DLLs and Libraries in correct order> -L\$\(LIBS_LOCATION\) <add other libs location if any>  \\\n";
    print MAKEFILE "		\$\(sde_ldopts\) -o \$@ \n\n";

    WriteTargetAll();
}
################################################################################################
# Program start 
################################################################################################

print  "$format\n";
print  "SDE2 $0 (script version $Version)\n\n";
print  "(c) NXP Semiconductors , 2006\n";
print  "$format\n";

CheckEnvironment();
CheckParameters();

$configuration = "$ENV{'_TMTGTCPUCLASS'}_$ENV{'_TMTGTOSCLASS'}";

$location      = "$ENV{'_TMREPORTS'}/$configuration";

system "mkdir -p $location";
$TemplateMakefile = "$location/$TemplateMakeFile";

open(MAKEFILE, ">" . $TemplateMakefile) or die "Cannot open $TemplateMakefile\n$!\n";

my $makefileheader = qq($format1
#	Makefile template for building application outside SDE2		       #
$format1
#------------------------------------------------------------------------------------------
# This is a makefile template, that can be used to build an application out side
# SDE2 environment. The user can create his own directory structure to place the
# required source files, header files, libraries and build tools. Those directory
# locations need to be specified in the appropriate locations in this makefile
# template, to make this a complete wokable makefile
# --------------------------------------------------------------------------------------------
# Enter the location information here. Please do not add any extra space at the end of line
# --------------------------------------------------------------------------------------------
#ROOT_LOCATION --> Location of the root directory 
#SRC_LOCATION  --> Location of source files
#INC_LOCATION  --> Location of header files
#LIBS_LOCATION --> Location of library files
#OBJ_LOCATION  --> Location of object files 
#EXE_LOCATION  --> Location of executable generated
# ---------------------------------------------------------------------------
ROOT_LOCATION =\n
SRC_LOCATION  =\n
INC_LOCATION  =\n
LIBS_LOCATION =\n
OBJ_LOCATION =\n
EXE_LOCATION =\n
# -------------------------------------------------------------------------------------------
# Enter the source file information here.Please do not add any extra space at the end of line
# --------------------------------------------------------------------------------------------
C_SOURCES     = \$(SRC_LOCATION)/<C sourcefile>\n
C_FILES       = \$\(notdir \$\(C_SOURCES\)\)\n
CXX_SOURCES   = \$(SRC_LOCATION)/<C++ source file>\n
CXX_FILES     = \$\(notdir \$\(CXX_SOURCES\)\)\n
S_SOURCES     = \$(SRC_LOCATION)/<Assembly source files>\n
S_FILES       = \$\(notdir \$\(S_SOURCES\)\)\n);

print MAKEFILE $makefileheader;

print  "Building makefile template for $configuration in $ENV{'_TMREPORTS'}/$configuration ...\n";
if( $configuration eq "x86_nt" )
{
    Create_make_x86_nt();
}
elsif( $configuration eq "tm_psos" )
{
    Create_make_tm_psos();
}
elsif( $configuration eq "x86_ce" )
{
    Create_make_x86_ce();
}
elsif( $configuration eq "arm_ce" )
{
    Create_make_arm_ce();
}
elsif( $configuration eq "mips_ce" )
{
    Create_make_mips_ce();
}
elsif( $configuration eq "x86_vxworks" )
{
    Create_make_x86_vxworks();
}
elsif( $configuration eq "arm_vxworks" )
{
    Create_make_arm_vxworks();
}
elsif( $configuration eq "arm_nullos" )
{
    Create_make_arm_nullos();
}
elsif( $configuration eq "hp_nullos" )
{
    Create_make_hp_nullos();
}
else
{
    print "Invalid Configuration Specified\n";
}
close MAKEFILE;
