<?xml version="1.0" encoding="utf-8"?>

<!--
   Copyright (C) 2008 NXP B.V.,
   All Rights Reserved.

Filename: manifest.xsl

Creates (html) view on a packlist as defined in manifest.dtd'

Rev Date       Author      Comments
  1 20080520   batelaan    Initial
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <xsl:output method="html" encoding="utf-8"/>

  <xsl:template match="/">
    <html>
      <head>
        <title>Software Platform Manifest</title>
        <style type="text/css">
table {
padding: 0px;
border: 1px solid black;
margin: 0px;
border-collapse: collapse;
}

th {
background-color: lightblue;
color: white;
font-weight: bold;
text-align: center;
border: 1px solid black;
}

thead {
background-color: lightblue;
color: white;
font-weight: bold;
text-align: center;
border: 1px solid black;
}

td {
padding: 0px 3px 0px 3px;
text-align: left;
border: 1px solid black;

}        </style>
      </head>
      <body>
        <h1>Software Platform Manifest</h1>
        <xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="/manifest">
    <xsl:apply-templates select="text"/>
    <xsl:call-template name="jumpto"/>
    <xsl:call-template name="apps"/>
    <xsl:call-template name="firmwares"/>
    <xsl:call-template name="kmodules"/>
    <xsl:call-template name="libs"/>
  </xsl:template>

  <xsl:template name="jumpto">
    Jump to
      [<a href="#apps">Applications</a>]
      [<a href="#firmwares">Firmwares</a>]
      [<a href="#kmodules">Kernel Modules</a>]
      [<a href="#libs">Libraries</a>]
  </xsl:template>

  <xsl:template name="apps">
    <h2 id="apps">Applications</h2>
    <xsl:call-template name="fileInfoList">
      <xsl:with-param name="type"  select="'app'"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="firmwares">
    <h2 id="firmwares">Firmwares</h2>
    <xsl:call-template name="fileInfoList">
      <xsl:with-param name="type"  select="'firmware'"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="kmodules">
    <h2 id="kmodules">Kernel Modules</h2>
    <xsl:call-template name="fileInfoList">
      <xsl:with-param name="type"  select="'kmodule'"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="libs">
    <h2 id="libs">Libraries</h2>
    <xsl:call-template name="fileInfoList">
      <xsl:with-param name="type"  select="'lib'"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="fileInfoList">
    <xsl:param name="type"/>
    <table>
      <thead>
        <tr>
          <td>Name</td><td>Path</td><td>Size</td><td>ProductVersion</td><td>Host</td><td>User</td><td>Date</td><td>Time</td>
        </tr>
      </thead>
      <xsl:for-each select=".//*[name()=$type]">
        <xsl:sort select="@path"/>
        <tr>
          <td><xsl:value-of select="@name"/></td>
          <td><xsl:value-of select="@path"/></td>
          <td align="right"><xsl:value-of select="@size"/></td>
          <td><xsl:value-of select="buildinfo/@productversion"/></td>
          <td><xsl:value-of select="buildinfo/@host"/></td>
          <td><xsl:value-of select="buildinfo/@user"/></td>
          <td><xsl:value-of select="buildinfo/@date"/></td>
          <td><xsl:value-of select="buildinfo/@time"/></td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

</xsl:stylesheet>
