/*
 * Copyright 2010 Trident Microsystems (Far East) Ltd. All rights reserved.
 * All prior copyright rights in this work and the accompanying software
 * products transferred to Trident Microsystems (Far East) Ltd. by written
 * agreement.
 */
/*****************************************************************************/
/* Copyright (c) 2010 NXP B.V.  All rights are reserved.                     */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* Redistributions of source code must retain the above copyright            */
/* notice, this list of conditions and the following disclaimer.             */
/*                                                                           */
/* Redistributions in binary form must reproduce the above copyright         */
/* notice, this list of conditions and the following disclaimer in the       */
/* documentation and/or other materials provided with the distribution.      */
/*                                                                           */
/* Neither the name of NXP nor the names of its                              */
/* contributors may be used to endorse or promote products derived from      */
/* this software without specific prior written permission.                  */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     */
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     */
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE         */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH          */
/* DAMAGE.                                                                   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbStreamingSystem_SBMSinkPin.h
    @brief  Interface for a SBM Sink Pin.

@b  Component:  phStbStreamingSystem.

    This header file is the interface of the SBMSinkPin class from the
    Streaming System detailed design.
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20060804    G.Lawrance  Original
  2 20060808    G.Lawrance  Rework
  3 20080625    burningh    Update license text
  4 20090120    batelaan    Update file copyright/license text
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

#ifndef PHSTBSTREAMINGSYSTEM_SBMSINKPIN_H
#define PHSTBSTREAMINGSYSTEM_SBMSINKPIN_H

/*******************
* INCLUDE FILES    *
********************/

#include "sbm_ctl.h"
#include "phStbStreamingSystem_InstanceTypes.h"
#include "phStbStreamingSystem_SBMPinClient.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************************
* EXPORTED MACROS         PH[<LNX|TM|NONE>]_IPSTBSTREAMINGSYSTEM_SBMSINKPIN_<_WORD>+        *
***************************************************************************/

/*********************************************************************
* EXPORTED TYPEDEFS       [p]ph[<lnx|tm|NONE>]StbStreamingSystem_SBMSinkPin_<Word>+_t  *
**********************************************************************/

/**************************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]ph[<lnx|tm|NONE>]StbStreamingSystem_SBMSinkPin_<Word>+ *
***************************************************************************/

/****************************************************************************
* EXPORTED FUNCTION PROTOTYPES     ph[<lnx|tm|NONE>]StbStreamingSystem_SBMSinkPin[_<Word>+]   *
*****************************************************************************/

/* Class Operations */

/******************************************************************************/
/** Create new SBMSinkPin instance

    @param defaultBuffer            IN  Default SBM buffer ID
    @param pinFlags                 IN  Pin options
    @param pSbmPinClient            IN  SBMPinClient interface callbacks to driver
    @param pSbmPinClientUserData    IN  SBMPinClient callback user defined data
    @param sinkPinID                IN  Index of SBM output pin that will
                                        correspond to the new SBMSinkPin instance
    @param ppSinkPin                OUT Newly created object

    @retval TM_OK           Success
    @retval AnotherValue    Error occured.
*/

tmErrorCode_t phStbStreamingSystem_SBMSinkPin_Create(
    SBM_CTL_BufferID defaultBuffer,
    UInt32 pinFlags,
    phStbStreamingSystem_SBMPinClient_t pSbmPinClient,
    void* pSbmPinClientUserData,
    int sinkPinID,
    phStbStreamingSystem_SinkPin_t** ppSinkPin);


/* Instance Operations */

/* None */

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* PHSTBSTREAMINGSYSTEM_SBMSINKPIN_H      Do not add any thing below this line */
